@@ -2,67 +2,20 @@
 // Name:        cat_classes.h
 // Purpose:     Classes-by-category page of the Doxygen manual
 // Author:      wxWidgets team
-// RCS-ID:      $Id: cat_classes.h 52468 2008-03-13 16:30:38Z FM $
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 /**
 
-@page page_class_cat Summarized Class List by Category
+@page page_class_cat Overview of Available Classes
+
+@tableofcontents
 
 This page contains a summarized listing of classes, please see the
-@ref group_class page for a full listing.
+@ref group_class page for a full listing by category or the
+<a href="classes.html">full list of classes</a> in alphabetical order.
 
-@beginInvisibleTable
-<tr><td>
-@li @ref page_class_cat_basicwnd
-@li @ref page_class_cat_winlayout
-@li @ref page_class_cat_managedwnd
-@li @ref page_class_cat_menus
-@li @ref page_class_cat_ctrl
-@li @ref page_class_cat_validator
-@li @ref page_class_cat_pickers
-@li @ref page_class_cat_aui
-@li @ref page_class_cat_cmndlg
-@li @ref page_class_cat_html
-@li @ref page_class_cat_miscwnd
-</td><td>
-@li @ref page_class_cat_dc
-@li @ref page_class_cat_gdi
-@li @ref page_class_cat_gc
-@li @ref page_class_cat_image
-@li @ref page_class_cat_events
-@li @ref page_class_cat_appmanagement
-@li @ref page_class_cat_printing
-@li @ref page_class_cat_dvf
-@li @ref page_class_cat_dnd
-@li @ref page_class_cat_vfs
-@li @ref page_class_cat_threading
-</td><td>
-@li @ref page_class_cat_rtti
-@li @ref page_class_cat_debugging
-@li @ref page_class_cat_logging
-@li @ref page_class_cat_data
-@li @ref page_class_cat_conv
-@li @ref page_class_cat_containers
-@li @ref page_class_cat_smartpointers
-@li @ref page_class_cat_file
-@li @ref page_class_cat_streams
-@li @ref page_class_cat_xml
-@li @ref page_class_cat_xrc
-</td><td>
-@li @ref page_class_cat_net
-@li @ref page_class_cat_archive
-@li @ref page_class_cat_ipc
-@li @ref page_class_cat_help
-@li @ref page_class_cat_media
-@li @ref page_class_cat_gl
-@li @ref page_class_cat_misc
-</td></tr>
-@endTable
-
-
-<hr>
+For a more visual approach, see @ref page_screenshots "the screenshots" page.
 
 
 @section page_class_cat_basicwnd Basic Windows
@@ -140,6 +93,7 @@ Related Overviews: @ref overview_cmndlg
 Typically, these are small windows which provide interaction with the user.
 Controls that are not static can have wxValidator associated with them.
 
+@li wxActivityIndicator: A small animated control indicating program activity
 @li wxAnimationCtrl: A control to display an animation
 @li wxControl: The base class for controls
 @li wxBitmapButton: Push button control, displaying a bitmap
@@ -191,6 +145,7 @@ Controls that are not static can have wxValidator associated with them.
 @li wxTextCtrl: Single or multiline text editing control
 @li wxToggleButton: A button which stays pressed when clicked by user.
 @li wxTreeCtrl: Tree (hierarchy) control
+@li wxTreeListCtrl: Multi-column tree control with simple interface
 @li wxStaticBitmap: A control to display a bitmap
 @li wxStyledTextCtrl: A wxWidgets implementation of the Scintilla source code
     editing component for plain text editing.
@@ -208,13 +163,15 @@ Related Overviews: @ref overview_validator
 @li wxValidator: Base validator class
 @li wxTextValidator: Text control validator class
 @li wxGenericValidator: Generic control validator class
+@li wxIntegerValidator: Text control validator class for integer numbers
+@li wxFloatingPointValidator: Text control validator class for floating point numbers
 
 
 
 @section page_class_cat_pickers Picker Controls
 
-A picker control is a control whose appearance and behaviour is highly
-platform-dependent.
+These controls provide the user with the possibility to choose something (file
+or directory, font or colour, ...) directly from the window containing them.
 
 @li wxColourPickerCtrl: A control which allows the user to choose a colour
 @li wxDirPickerCtrl: A control which allows the user to choose a directory
@@ -240,6 +197,7 @@ The following are a variety of classes that are derived from wxWindow.
     variable sizes.
 @li wxGrid: A grid (table) window
 @li wxInfoBar: An information bar usually shown on top of the main window.
+@li wxRichToolTip: A customizable tooltip.
 @li wxSplitterWindow: Window which can be split vertically or horizontally
 @li wxStatusBar: Implements the status bar on a frame
 @li wxToolBar: Toolbar class
@@ -250,9 +208,11 @@ The following are a variety of classes that are derived from wxWindow.
 @li wxSashWindow: Window with four optional sashes that can be dragged
 @li wxSashLayoutWindow: Window that can be involved in an IDE-like layout
     arrangement
+@li wxSimplebook: Another book control but one allowing only the program, not
+    the user, to change its current page.
 @li wxWizardPage: A base class for the page in wizard dialog.
 @li wxWizardPageSimple: A page in wizard dialog.
-
+@li wxCustomBackgroundWindow: A window allowing to set a custom bitmap.
 
 
 @section page_class_cat_aui Window Docking (wxAUI)
@@ -294,6 +254,7 @@ Related Overviews: @ref overview_cmndlg
 @li wxRearrangeDialog: Dialog allowing the user to rearrange a list of items.
 @li wxRichTextFormattingDialog: A dialog for formatting the content of a
     wxRichTextCtrl
+@li wxRichMessageDialog: Nicer message box dialog
 @li wxSingleChoiceDialog: Dialog to get a single selection from a list and
     return the string
 @li wxSymbolPickerDialog: Symbol selector dialog
@@ -652,13 +613,13 @@ These are the data structure classes supported by wxWidgets.
 These classes define objects for performing conversions between different
 multibyte and Unicode encodings and wide character strings.
 
-@li wxMBConv: Base class for all convertors, defines the API implemented by all
-    the other convertor classes.
-@li wxMBConvUTF7: Convertor for UTF-7
-@li wxMBConvUTF8: Convertor for UTF-8
-@li wxMBConvUTF16: Convertor for UTF-16
-@li wxMBConvUTF32: Convertor for UTF-32
-@li wxCSConv: Convertor for any system-supported encoding which can be
+@li wxMBConv: Base class for all converters, defines the API implemented by all
+    the other converter classes.
+@li wxMBConvUTF7: Converter for UTF-7
+@li wxMBConvUTF8: Converter for UTF-8
+@li wxMBConvUTF16: Converter for UTF-16
+@li wxMBConvUTF32: Converter for UTF-32
+@li wxCSConv: Converter for any system-supported encoding which can be
     specified by name.
 
 Related Overviews: @ref overview_mbconv
