@@ -2,7 +2,6 @@
 // Name:        wx/gtk1/cursor.h
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: cursor.h,v 1.19 2006/10/30 19:26:23 VZ Exp $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -10,8 +9,6 @@
 #ifndef __GTKCURSORH__
 #define __GTKCURSORH__
 
-#include "wx/defs.h"
-#include "wx/object.h"
 #include "wx/gdicmn.h"
 
 #if wxUSE_IMAGE
@@ -22,28 +19,43 @@
 // wxCursor
 //-----------------------------------------------------------------------------
 
-class WXDLLIMPEXP_CORE wxCursor: public wxObject
+class WXDLLIMPEXP_CORE wxCursor : public wxCursorBase
 {
 public:
-
     wxCursor();
-    wxCursor( int cursorId );
+    wxCursor(wxStockCursor id) { InitFromStock(id); }
+#if WXWIN_COMPATIBILITY_2_8
+    wxCursor(int id) { InitFromStock((wxStockCursor)id); }
+#endif
 #if wxUSE_IMAGE
     wxCursor( const wxImage & image );
 #endif
     wxCursor( const char bits[], int width, int  height,
               int hotSpotX=-1, int hotSpotY=-1,
-              const char maskBits[] = NULL, const wxColour *fg = NULL, const wxColour *bg = NULL );
+              const char maskBits[] = NULL,
+              const wxColour* fg = NULL, const wxColour* bg = NULL);
+
+    /* WARNING: the following ctor is missing:
+
+        wxCursor(const wxString& name,
+                wxBitmapType type = wxCURSOR_DEFAULT_TYPE,
+                int hotSpotX = 0, int hotSpotY = 0);
+    */
+
     virtual ~wxCursor();
-    bool Ok() const { return IsOk(); }
-    bool IsOk() const;
 
     // implementation
 
     GdkCursor *GetCursor() const;
 
+protected:
+    void InitFromStock(wxStockCursor);
+
+    virtual wxGDIRefData *CreateGDIRefData() const;
+    virtual wxGDIRefData *CloneGDIRefData(const wxGDIRefData *data) const;
+
 private:
-    DECLARE_DYNAMIC_CLASS(wxCursor)
+    wxDECLARE_DYNAMIC_CLASS(wxCursor);
 };
 
 #endif // __GTKCURSORH__
