@@ -4,7 +4,6 @@
 // Author:      Denis Pershin
 // Modified by:
 // Created:     08/08/98
-// RCS-ID:      $Id$
 // Copyright:   (c) Denis Pershin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -63,14 +62,6 @@ static const int wxTREE_HITTEST_ONITEM  = wxTREE_HITTEST_ONITEMICON |
                                           wxTREE_HITTEST_ONITEMLABEL |
                                           wxTREE_HITTEST_ONITEMSTATEICON;
 
-#if WXWIN_COMPATIBILITY_2_6
-    // NB: all the following flags are for compatbility only and will be removed in
-    //     next versions
-    // flags for deprecated InsertItem() variant
-    #define wxTREE_INSERT_FIRST 0xFFFF0001
-    #define wxTREE_INSERT_LAST  0xFFFF0002
-#endif
-
 // ----------------------------------------------------------------------------
 // wxTreeItemId identifies an element of the tree. In this implementation, it's
 // just a trivial wrapper around GTK GtkTreeItem *. It's opaque for the
@@ -332,39 +323,6 @@ public:
     void SortChildren(const wxTreeItemId& item,
                       wxTreeItemCmpFunc *cmpFunction = NULL);
 
-    // deprecated
-    // ----------
-
-#if WXWIN_COMPATIBILITY_2_6
-    // these methods are deprecated and will be removed in future versions of
-    // wxWidgets, they're here for compatibility only, don't use them in new
-    // code (the comments indicate why these methods are now useless and how to
-    // replace them)
-
-        // use Expand, Collapse, CollapseAndReset or Toggle
-    wxDEPRECATED( void ExpandItem(const wxTreeItemId& item, int action) );
-
-        // use SetImageList
-    wxDEPRECATED( void SetImageList(wxImageList *imageList, int) )
-        { SetImageList(imageList); }
-
-        // use Set/GetItemImage directly
-    wxDEPRECATED( int GetItemSelectedImage(const wxTreeItemId& item) const );
-    wxDEPRECATED( void SetItemSelectedImage(const wxTreeItemId& item, int image) );
-
-        // get the first child of this item
-    wxDEPRECATED( wxTreeItemId GetFirstChild(const wxTreeItemId& item, long& cookie) const );
-        // get the next child (after GetFirstChild or GetNextChild)
-    wxDEPRECATED( wxTreeItemId GetNextChild(const wxTreeItemId& item, long& cookie) const );
-
-        // use AddRoot, PrependItem or AppendItem
-    wxDEPRECATED( wxTreeItemId InsertItem(const wxTreeItemId& parent,
-                                          const wxString& text,
-                                          int image = -1, int selImage = -1,
-                                          long insertAfter = wxTREE_INSERT_LAST) );
-
-#endif // WXWIN_COMPATIBILITY_2_6
-
         // use Set/GetImageList and Set/GetStateImageList
     wxImageList *GetImageList(int) const
         { return GetImageList(); }
@@ -399,7 +357,7 @@ protected:
                               wxTreeItemData *data);
 
 
-    DECLARE_DYNAMIC_CLASS(wxTreeCtrl)
+    wxDECLARE_DYNAMIC_CLASS(wxTreeCtrl);
 };
 
 #endif
