@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     17/09/98
-// RCS-ID:      $Id: combobox.h,v 1.24 2006/09/05 20:46:11 VZ Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -13,12 +12,12 @@
 #define _WX_COMBOBOX_H_
 
 #include "wx/choice.h"
+#include "wx/textentry.h"
 
 // Combobox item
-class WXDLLEXPORT wxComboBox: public wxChoice
+class WXDLLIMPEXP_CORE wxComboBox : public wxChoice,
+                               public wxTextEntry
 {
-    DECLARE_DYNAMIC_CLASS(wxComboBox)
-
 public:
     wxComboBox() { m_inSetSelection = false; }
     virtual ~wxComboBox();
@@ -69,34 +68,35 @@ public:
         const wxValidator& validator = wxDefaultValidator,
         const wxString& name = wxComboBoxNameStr);
 
-    // implementation of wxControlWithItems
-    virtual int DoAppend(const wxString& item);
-    virtual int DoInsert(const wxString& item, unsigned int pos);
-    virtual void Delete(unsigned int n);
+    // See wxComboBoxBase discussion of IsEmpty().
+    bool IsListEmpty() const { return wxItemContainer::IsEmpty(); }
+    bool IsTextEmpty() const { return wxTextEntry::IsEmpty(); }
+
+    // resolve ambiguities among virtual functions inherited from both base
+    // classes
     virtual void Clear();
+    virtual wxString GetValue() const { return wxTextEntry::GetValue(); }
+    virtual void SetValue(const wxString& value);
+    virtual wxString GetStringSelection() const
+        { return wxChoice::GetStringSelection(); }
+
+    virtual void SetSelection(long from, long to)
+        { wxTextEntry::SetSelection(from, to); }
+    virtual void GetSelection(long *from, long *to) const
+        { wxTextEntry::GetSelection(from, to); }
+
+
+    // implementation of wxControlWithItems
+    virtual int DoInsertItems(const wxArrayStringsAdapter& items,
+                              unsigned int pos,
+                              void **clientData, wxClientDataType type);
+    virtual void DoDeleteOneItem(unsigned int n);
     virtual int GetSelection() const ;
     virtual void SetSelection(int n);
     virtual int FindString(const wxString& s, bool bCase = false) const;
     virtual wxString GetString(unsigned int n) const ;
     virtual void SetString(unsigned int n, const wxString& s);
 
-    // Text field functions
-    virtual wxString GetValue() const ;
-    virtual void SetValue(const wxString& value);
-
-    // Clipboard operations
-    virtual void Copy();
-    virtual void Cut();
-    virtual void Paste();
-    virtual void SetInsertionPoint(long pos);
-    virtual void SetInsertionPointEnd();
-    virtual long GetInsertionPoint() const ;
-    virtual wxTextPos GetLastPosition() const ;
-    virtual void Replace(long from, long to, const wxString& value);
-    virtual void Remove(long from, long to);
-    virtual void SetSelection(long from, long to);
-    virtual void SetEditable(bool editable);
-
     // Implementation
     virtual void ChangeFont(bool keepOriginalSize = true);
     virtual void ChangeBackgroundColour();
@@ -104,11 +104,20 @@ public:
     WXWidget GetTopWidget() const { return m_mainWidget; }
     WXWidget GetMainWidget() const { return m_mainWidget; }
 
-    virtual wxSize DoGetBestSize() const;
+   //Copied from wxComboBoxBase because for wxMOTIF wxComboBox does not inherit from it.
+    virtual void Popup() { wxFAIL_MSG( wxT("Not implemented") ); }
+    virtual void Dismiss() { wxFAIL_MSG( wxT("Not implemented") ); }
+
 protected:
+    virtual wxSize DoGetBestSize() const;
     virtual void DoSetSize(int x, int y,
-        int width, int height,
-        int sizeFlags = wxSIZE_AUTO);
+                           int width, int height,
+                           int sizeFlags = wxSIZE_AUTO);
+
+    // implement wxTextEntry pure virtual methods
+    virtual wxWindow *GetEditableWindow() { return this; }
+    virtual WXWidget GetTextWidget() const;
+
 private:
     // only implemented for native combo box
     void AdjustDropDownListSize();
@@ -116,7 +125,8 @@ private:
     // implementation detail, should really be private
 public:
     bool m_inSetSelection;
+
+    wxDECLARE_DYNAMIC_CLASS(wxComboBox);
 };
 
-#endif
-// _WX_COMBOBOX_H_
+#endif // _WX_COMBOBOX_H_
