@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: combobox.h,v 1.33 2005/03/18 19:32:14 ABX Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -12,11 +11,8 @@
 #ifndef _WX_COMBOBOX_H_
 #define _WX_COMBOBOX_H_
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma interface "combobox.h"
-#endif
-
 #include "wx/choice.h"
+#include "wx/textentry.h"
 
 #if wxUSE_COMBOBOX
 
@@ -24,7 +20,8 @@
 // Combobox control
 // ----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxComboBox: public wxChoice
+class WXDLLIMPEXP_CORE wxComboBox : public wxChoice,
+                                    public wxTextEntry
 {
 public:
     wxComboBox() { Init(); }
@@ -39,9 +36,10 @@ public:
             const wxString& name = wxComboBoxNameStr)
     {
         Init();
-
         Create(parent, id, value, pos, size, n, choices, style, validator, name);
+
     }
+
     wxComboBox(wxWindow *parent, wxWindowID id,
             const wxString& value,
             const wxPoint& pos,
@@ -76,46 +74,33 @@ public:
                 const wxValidator& validator = wxDefaultValidator,
                 const wxString& name = wxComboBoxNameStr);
 
-    // List functions: see wxChoice
+    // See wxComboBoxBase discussion of IsEmpty().
+    bool IsListEmpty() const { return wxItemContainer::IsEmpty(); }
+    bool IsTextEmpty() const { return wxTextEntry::IsEmpty(); }
 
-    // Text field functions
-    wxString GetValue() const { return m_value; }
+    // resolve ambiguities among virtual functions inherited from both base
+    // classes
+    virtual void Clear();
+    virtual wxString GetValue() const;
     virtual void SetValue(const wxString& value);
-
-    // Clipboard operations
-    virtual void Copy();
-    virtual void Cut();
-    virtual void Paste();
-    virtual bool CanCopy() const;
-    virtual bool CanCut() const;
-    virtual bool CanPaste() const;
-    virtual void SetInsertionPoint(long pos);
-    virtual void SetInsertionPointEnd();
-    virtual long GetInsertionPoint() const;
-    virtual wxTextPos GetLastPosition() const;
-    virtual void Replace(long from, long to, const wxString& value);
-    virtual void Remove(long from, long to);
+    virtual wxString GetStringSelection() const
+        { return wxChoice::GetStringSelection(); }
+    virtual void Popup() { MSWDoPopupOrDismiss(true); }
+    virtual void Dismiss() { MSWDoPopupOrDismiss(false); }
     virtual void SetSelection(int n) { wxChoice::SetSelection(n); }
-    virtual void SetSelection(long from, long to);
-    virtual int GetSelection() const;
-    virtual void GetSelection(long* from, long* to) const;
-    virtual void SetEditable(bool editable);
-    virtual void Clear() { wxChoice::Clear(); m_selectionOld = -1; }
+    virtual void SetSelection(long from, long to)
+        { wxTextEntry::SetSelection(from, to); }
+    virtual int GetSelection() const { return wxChoice::GetSelection(); }
+    virtual bool ContainsHWND(WXHWND hWnd) const;
+    virtual void GetSelection(long *from, long *to) const;
 
-    virtual void Undo() ;
-    virtual void Redo() ;
-    virtual bool CanUndo() const;
-    virtual bool CanRedo() const;
-    virtual void SelectAll();
-    virtual bool IsEditable() const ;
-    virtual bool HasSelection() const;
+    virtual bool IsEditable() const;
 
     // implementation only from now on
     virtual bool MSWCommand(WXUINT param, WXWORD id);
     bool MSWProcessEditMsg(WXUINT msg, WXWPARAM wParam, WXLPARAM lParam);
     virtual WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
-
-    WXHWND GetEditHWND() const;
+    bool MSWShouldPreProcessMessage(WXMSG *pMsg);
 
     // Standard event handling
     void OnCut(wxCommandEvent& event);
@@ -134,24 +119,58 @@ public:
     void OnUpdateDelete(wxUpdateUIEvent& event);
     void OnUpdateSelectAll(wxUpdateUIEvent& event);
 
-protected:
     virtual WXDWORD MSWGetStyle(long style, WXDWORD *exstyle) const;
 
-    // common part of all ctors
-    void Init() { m_selectionOld = -1; }
+#if wxUSE_UXTHEME
+    // override wxTextEntry method to work around Windows bug
+    virtual bool SetHint(const wxString& hint);
+#endif // wxUSE_UXTHEME
 
+    virtual void SetLayoutDirection(wxLayoutDirection dir) wxOVERRIDE;
 
-    // the previous selection (see MSWCommand() to see why it is needed)
-    int m_selectionOld;
+protected:
+#if wxUSE_TOOLTIPS
+    virtual void DoSetToolTip(wxToolTip *tip);
+#endif
 
-    // the current selection (also see MSWCommand())
-    wxString m_value;
+    virtual wxSize DoGetSizeFromTextSize(int xlen, int ylen = -1) const;
+
+    // Override this one to avoid eating events from our popup listbox.
+    virtual wxWindow *MSWFindItem(long id, WXHWND hWnd) const;
+
+    // this is the implementation of GetEditHWND() which can also be used when
+    // we don't have the edit control, it simply returns NULL then
+    //
+    // try not to use this function unless absolutely necessary (as in the
+    // message handling code where the edit control might not be created yet
+    // for the messages we receive during the control creation) as normally
+    // just testing for IsEditable() and using GetEditHWND() should be enough
+    WXHWND GetEditHWNDIfAvailable() const;
+
+    virtual void EnableTextChangedEvents(bool enable)
+    {
+        m_allowTextEvents = enable;
+    }
 
 private:
-    DECLARE_DYNAMIC_CLASS_NO_COPY(wxComboBox)
-    DECLARE_EVENT_TABLE()
+    // there are the overridden wxTextEntry methods which should only be called
+    // when we do have an edit control so they assert if this is not the case
+    virtual wxWindow *GetEditableWindow();
+    virtual WXHWND GetEditHWND() const;
+
+    // common part of all ctors
+    void Init()
+    {
+        m_allowTextEvents = true;
+    }
+
+    // normally true, false if text events are currently disabled
+    bool m_allowTextEvents;
+
+    wxDECLARE_DYNAMIC_CLASS_NO_COPY(wxComboBox);
+    wxDECLARE_EVENT_TABLE();
 };
 
 #endif // wxUSE_COMBOBOX
-#endif
-    // _WX_COMBOBOX_H_
+
+#endif // _WX_COMBOBOX_H_
