@@ -45,11 +45,6 @@
 #define wxLIST_STATE_FOCUSED        0x0002
 #define wxLIST_STATE_SELECTED       0x0004
 #define wxLIST_STATE_CUT            0x0008      // MSW only
-#define wxLIST_STATE_DISABLED       0x0010      // OS2 only
-#define wxLIST_STATE_FILTERED       0x0020      // OS2 only
-#define wxLIST_STATE_INUSE          0x0040      // OS2 only
-#define wxLIST_STATE_PICKED         0x0080      // OS2 only
-#define wxLIST_STATE_SOURCE         0x0100      // OS2 only
 
 /// Hit test flags, used in HitTest
 #define wxLIST_HITTEST_ABOVE            0x0001  // Above the client area.
@@ -261,8 +256,18 @@ enum
     @event{EVT_LIST_CACHE_HINT(id, func)}
            Prepare cache for a virtual list control.
            Processes a @c wxEVT_LIST_CACHE_HINT event type.
+    @event{EVT_LIST_ITEM_CHECKED(id, func)}
+           The item has been checked.
+           Processes a @c wxEVT_LIST_ITEM_CHECKED event type (new since wxWidgets 3.1.0).
+    @event{EVT_LIST_ITEM_UNCHECKED(id, func)}
+           The item has been unchecked.
+           Processes a @c wxEVT_LIST_ITEM_UNCHECKED event type (new since wxWidgets 3.1.0).
     @endEventTable
 
+    @note Under wxMSW this control uses wxSystemThemedControl for an explorer
+    style appearance by default since wxWidgets 3.1.0. If this is not desired,
+    you can call wxSystemThemedControl::EnableSystemTheme with @c false
+    argument to disable this.
 
     @library{wxcore}
     @category{ctrl}
@@ -575,7 +580,7 @@ public:
         Returns @NULL if no label is being edited.
 
         @note It is currently only implemented for wxMSW and the generic version,
-              not for the native Mac OS X version.
+              not for the native OS X version.
     */
     wxTextCtrl* GetEditControl() const;
 
@@ -790,6 +795,14 @@ public:
      */
     void SetAlternateRowColour(const wxColour& colour);
 
+    /**
+        Get the alternative row background colour.
+
+        @since 3.1.0
+        @see SetAlternateRowColour()
+     */
+    wxColour GetAlternateRowColour() const;
+
     /**
         Determines which item (if any) is at the specified point, giving details
         in @a flags. Returns index of the item or @c wxNOT_FOUND if no item is at
@@ -1149,6 +1162,10 @@ public:
 
         Consider using wxListView if possible to avoid dealing with this
         error-prone and confusing method.
+
+        Also notice that contrary to the usual rule that only user actions
+        generate events, this method does generate wxEVT_LIST_ITEM_SELECTED
+        event when it is used to select an item.
     */
     bool SetItemState(long item, long state, long stateMask);
 
@@ -1192,8 +1209,8 @@ public:
         second one and positive value if the first one is greater than the second one
         (the same convention as used by @c qsort(3)).
 
-        The parameter @e item1 is the client data associated with the first item (NOT the index).
-        The parameter @e item2 is the client data associated with the second item (NOT the index).
+        The parameter @e item1 is the client data associated with the first item (@b NOT the index).
+        The parameter @e item2 is the client data associated with the second item (@b NOT the index).
         The parameter @e data is the value passed to SortItems() itself.
 
         Notice that the control may only be sorted on client data associated with
@@ -1210,6 +1227,49 @@ public:
     */
     bool SortItems(wxListCtrlCompare fnSortCallBack, wxIntPtr data);
 
+    /**
+        Returns true if checkboxes are enabled for list items.
+
+        @see EnableCheckboxes()
+
+        @since 3.1.0
+    */
+    bool HasCheckboxes() const;
+
+    /**
+        Enable or disable checkboxes for list items.
+
+        @param enable If @true, enable checkboxes, otherwise disable checkboxes.
+        @return @true if checkboxes are supported, @false otherwise.
+
+        @since 3.1.0
+    */
+    bool EnableCheckboxes(bool enable = true);
+
+    /**
+        Return true if the checkbox for the given wxListItem is checked.
+
+        Always returns false if checkboxes support hadn't been enabled.
+
+        @param item Item (zero-based) index.
+
+        @since 3.1.0
+    */
+    bool IsItemChecked(long item) const;
+
+    /**
+        Check or uncheck a wxListItem in a control using checkboxes.
+
+        This method only works if checkboxes support had been successfully
+        enabled using EnableCheckboxes().
+
+        @param item Item (zero-based) index.
+        @param check If @true, check the item, otherwise uncheck.
+
+        @since 3.1.0
+    */
+    void CheckItem(long item, bool check);
+
 protected:
 
     /**
@@ -1332,6 +1392,10 @@ protected:
         A column has been resized by the user.
     @event{EVT_LIST_CACHE_HINT(id, func)}
         Prepare cache for a virtual list control
+    @event{EVT_LIST_ITEM_CHECKED(id, func)}
+        The item has been checked (new since wxWidgets 3.1.0).
+    @event{EVT_LIST_ITEM_UNCHECKED(id, func)}
+        The item has been unchecked (new since wxWidgets 3.1.0).
     @endEventTable
 
 
@@ -1422,44 +1486,6 @@ public:
         admittedly rare case when the user wants to rename it to an empty string).
     */
     bool IsEditCancelled() const;
-
-    
-    /**
-       @see GetKeyCode()
-    */
-    void SetKeyCode(int code);
-
-    /**
-       @see GetIndex()
-    */
-    void SetIndex(long index);
-
-    /**
-       @see GetColumn()
-    */
-    void SetColumn(int col);
-
-    /**
-       @see GetPoint()
-    */
-    void SetPoint(const wxPoint& point);
-
-    /**
-       @see GetItem()
-    */
-    void SetItem(const wxListItem& item);
-
-
-    /**
-       @see GetCacheFrom()
-    */
-    void SetCacheFrom(long cacheFrom);
-
-    /**
-       @see GetCacheTo()
-    */
-    void SetCacheTo(long cacheTo);
-
 };
 
 
@@ -1483,6 +1509,8 @@ wxEventType wxEVT_LIST_COL_BEGIN_DRAG;
 wxEventType wxEVT_LIST_COL_DRAGGING;
 wxEventType wxEVT_LIST_COL_END_DRAG;
 wxEventType wxEVT_LIST_ITEM_FOCUSED;
+wxEventType wxEVT_LIST_ITEM_CHECKED;
+wxEventType wxEVT_LIST_ITEM_UNCHECKED;
 
 
 /**
@@ -1670,12 +1698,15 @@ public:
     /**
         Selects or unselects the given item.
 
+        Notice that this method inherits the unusual behaviour of
+        wxListCtrl::SetItemState() which sends a wxEVT_LIST_ITEM_SELECTED event
+        when it is used to select an item, contrary to the usual rule that only
+        the user actions result in selection.
+
         @param n
             the item to select or unselect
         @param on
             if @true (default), selects the item, otherwise unselects it
-
-        @see wxListCtrl::SetItemState
     */
     void Select(long n, bool on = true);
 
