@@ -59,7 +59,7 @@ public:
                            const wxRect& rect,
                            int horizAlign,
                            int vertAlign,
-                           int WXUNUSED(textOrientation)) const
+                           int WXUNUSED(textOrientation)) const wxOVERRIDE
     {
         dc.SetTextForeground(m_colFg);
         dc.SetFont(wxITALIC_FONT->Bold());
@@ -68,7 +68,7 @@ public:
 
     virtual void DrawBorder(const wxGrid& WXUNUSED(grid),
                             wxDC& dc,
-                            wxRect& rect) const
+                            wxRect& rect) const wxOVERRIDE
     {
         dc.SetPen(*wxTRANSPARENT_PEN);
         dc.SetBrush(wxBrush(m_colBg));
@@ -99,7 +99,7 @@ public:
     void UseCustomColHeaders(bool use = true) { m_useCustom = use; }
 
 protected:
-    virtual const wxGridColumnHeaderRenderer& GetColumnHeaderRenderer(int col)
+    virtual const wxGridColumnHeaderRenderer& GetColumnHeaderRenderer(int col) wxOVERRIDE
     {
         // if enabled, use custom renderers
         if ( m_useCustom )
@@ -125,7 +125,7 @@ private:
 // wxWin macros
 // ----------------------------------------------------------------------------
 
-IMPLEMENT_APP( GridApp )
+wxIMPLEMENT_APP(GridApp);
 
 // ============================================================================
 // implementation
@@ -1789,10 +1789,10 @@ public:
 
     TabularGridTable() { m_sortOrder = NULL; }
 
-    virtual int GetNumberRows() { return ROW_MAX; }
-    virtual int GetNumberCols() { return COL_MAX; }
+    virtual int GetNumberRows() wxOVERRIDE { return ROW_MAX; }
+    virtual int GetNumberCols() wxOVERRIDE { return COL_MAX; }
 
-    virtual wxString GetValue(int row, int col)
+    virtual wxString GetValue(int row, int col) wxOVERRIDE
     {
         if ( m_sortOrder )
             row = m_sortOrder[row];
@@ -1817,12 +1817,12 @@ public:
         return wxString();
     }
 
-    virtual void SetValue(int, int, const wxString&)
+    virtual void SetValue(int, int, const wxString&) wxOVERRIDE
     {
         wxFAIL_MSG( "shouldn't be called" );
     }
 
-    virtual wxString GetColLabelValue(int col)
+    virtual wxString GetColLabelValue(int col) wxOVERRIDE
     {
         // notice that column parameter here always refers to the internal
         // column index, independently of its position on the screen
@@ -1832,7 +1832,7 @@ public:
         return labels[col];
     }
 
-    virtual void SetColLabelValue(int, const wxString&)
+    virtual void SetColLabelValue(int, const wxString&) wxOVERRIDE
     {
         wxFAIL_MSG( "shouldn't be called" );
     }
@@ -1909,7 +1909,7 @@ public:
     }
 
 protected:
-    virtual wxSize DoGetBestSize() const
+    virtual wxSize DoGetBestSize() const wxOVERRIDE
     {
         wxSize size = wxTextCtrl::DoGetBestSize();
         size.x = 3*GetCharWidth();
