@@ -4,7 +4,6 @@
 // Author:      Vadim Zeitlin/Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -37,7 +36,7 @@
 #include "wx/log.h"
 #include "wx/cmdline.h"
 
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -51,9 +50,9 @@ class MyApp: public wxApp
 public:
     MyApp() { m_lang = wxLANGUAGE_UNKNOWN; }
 
-    virtual void OnInitCmdLine(wxCmdLineParser& parser);
-    virtual bool OnCmdLineParsed(wxCmdLineParser& parser);
-    virtual bool OnInit();
+    virtual void OnInitCmdLine(wxCmdLineParser& parser) wxOVERRIDE;
+    virtual bool OnCmdLineParsed(wxCmdLineParser& parser) wxOVERRIDE;
+    virtual bool OnInit() wxOVERRIDE;
 
 protected:
     wxLanguage m_lang;  // language specified by user
@@ -78,7 +77,7 @@ public:
     void OnTest3(wxCommandEvent& event);
     void OnTestMsgBox(wxCommandEvent& event);
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 
     wxLocale& m_locale;
 };
@@ -155,7 +154,7 @@ wxCOMPILE_TIME_ASSERT( WXSIZEOF(langNames) == WXSIZEOF(langIds),
 // wxWidgets macros
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(INTERNAT_TEST, MyFrame::OnTestLocaleAvail)
     EVT_MENU(wxID_ABOUT, MyFrame::OnAbout)
     EVT_MENU(wxID_EXIT, MyFrame::OnQuit)
@@ -166,9 +165,9 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(INTERNAT_TEST_2, MyFrame::OnTest2)
     EVT_MENU(INTERNAT_TEST_3, MyFrame::OnTest3)
     EVT_MENU(INTERNAT_TEST_MSGBOX, MyFrame::OnTestMsgBox)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-IMPLEMENT_APP(MyApp)
+wxIMPLEMENT_APP(MyApp);
 
 // ============================================================================
 // implementation
@@ -271,7 +270,6 @@ bool MyApp::OnInit()
 
     // Show the frame
     frame->Show(true);
-    SetTopWindow(frame);
 
     return true;
 }
@@ -296,7 +294,7 @@ MyFrame::MyFrame(wxLocale& locale)
 
     // since wxID_ABOUT and wxID_EXIT are stock IDs they will automatically get
     // translated help strings; nice isn't it?
-    file_menu->Append(wxID_ABOUT, _("&About..."));
+    file_menu->Append(wxID_ABOUT, _("&About"));
     file_menu->AppendSeparator();
     file_menu->Append(wxID_EXIT, _("E&xit"));
 
@@ -399,7 +397,7 @@ void MyFrame::OnPlay(wxCommandEvent& WXUNUSED(event))
         str = wxGetTranslation(wxTRANSLATE("Bad luck! try again..."));
 
         // note also that if we want 'str' to contain a localized string
-        // we need to use wxGetTranslation explicitely as wxTRANSLATE just
+        // we need to use wxGetTranslation explicitly as wxTRANSLATE just
         // tells xgettext to extract the string but has no effect on the
         // runtime of the program!
     }
