@@ -4,7 +4,6 @@
 // Author:      Ryan Norton
 // Modified by:
 // Created:     11/10/04
-// RCS-ID:      $Id$
 // Copyright:   (c) Ryan Norton
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -28,8 +27,6 @@
 //
 // 1) Certain backends can't play the same media file at the same time (MCI,
 //    Cocoa NSMovieView-Quicktime).
-// 2) Positioning on Mac Carbon is messed up if put in a sub-control like a
-//    Notebook (like this sample does).
 // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 // ============================================================================
@@ -54,20 +51,32 @@
 // Headers
 // ----------------------------------------------------------------------------
 
-#include "wx/mediactrl.h"   //for wxMediaCtrl
-#include "wx/filedlg.h"     //for opening files from OpenFile
-#include "wx/slider.h"      //for a slider for seeking within media
-#include "wx/sizer.h"       //for positioning controls/wxBoxSizer
-#include "wx/timer.h"       //timer for updating status bar
-#include "wx/textdlg.h"     //for getting user text from OpenURL/Debug
-#include "wx/notebook.h"    //for wxNotebook and putting movies in pages
-#include "wx/cmdline.h"     //for wxCmdLineParser (optional)
-#include "wx/listctrl.h"    //for wxListCtrl
-#include "wx/dnd.h"         //drag and drop for the playlist
-#include "wx/filename.h"    //For wxFileName::GetName()
-#include "wx/config.h"      //for native wxConfig
+#include "wx/mediactrl.h"   // for wxMediaCtrl
+#include "wx/filedlg.h"     // for opening files from OpenFile
+#include "wx/slider.h"      // for a slider for seeking within media
+#include "wx/sizer.h"       // for positioning controls/wxBoxSizer
+#include "wx/timer.h"       // timer for updating status bar
+#include "wx/textdlg.h"     // for getting user text from OpenURL/Debug
+#include "wx/notebook.h"    // for wxNotebook and putting movies in pages
+#include "wx/cmdline.h"     // for wxCmdLineParser (optional)
+#include "wx/listctrl.h"    // for wxListCtrl
+#include "wx/dnd.h"         // drag and drop for the playlist
+#include "wx/filename.h"    // For wxFileName::GetName()
+#include "wx/config.h"      // for native wxConfig
+#include "wx/vector.h"
 
-#ifndef __WXMSW__
+// Under MSW we have several different backends but when linking statically
+// they may be discarded by the linker (this definitely happens with MSVC) so
+// force linking them. You don't have to do this in your code if you don't plan
+// to use them, of course.
+#if defined(__WXMSW__) && !defined(WXUSINGDLL)
+    #include "wx/link.h"
+    wxFORCE_LINK_MODULE(wxmediabackend_am)
+    wxFORCE_LINK_MODULE(wxmediabackend_qt)
+    wxFORCE_LINK_MODULE(wxmediabackend_wmp10)
+#endif // static wxMSW build
+
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -76,12 +85,6 @@
 // things we need
 // ----------------------------------------------------------------------------
 
-// RN:  I'm not sure why this is here - even minimal doesn't check for
-//      wxUSE_GUI.  I may have added it myself though...
-#if !wxUSE_GUI
-#error "This is a GUI sample"
-#endif
-
 #if !wxUSE_MEDIACTRL || !wxUSE_MENUS || !wxUSE_SLIDER || !wxUSE_TIMER || \
     !wxUSE_NOTEBOOK || !wxUSE_LISTCTRL
 #error "Not all required elements are enabled.  Please modify setup.h!"
@@ -138,10 +141,18 @@ class wxMediaPlayerApp : public wxApp
 {
 public:
 #ifdef __WXMAC__
-    virtual void MacOpenFile(const wxString & fileName );
+    virtual void MacOpenFiles(const wxArrayString & fileNames );
 #endif
 
-    virtual bool OnInit();
+#if wxUSE_CMDLINE_PARSER
+    virtual void OnInitCmdLine(wxCmdLineParser& parser) wxOVERRIDE;
+    virtual bool OnCmdLineParsed(wxCmdLineParser& parser) wxOVERRIDE;
+
+    // Files specified on the command line, if any.
+    wxVector<wxString> m_params;
+#endif // wxUSE_CMDLINE_PARSER
+
+    virtual bool OnInit() wxOVERRIDE;
 
 protected:
     class wxMediaPlayerFrame* m_frame;
@@ -203,8 +214,8 @@ private:
     void DoOpenFile(const wxString& path, bool bNewPage);
     void DoPlayFile(const wxString& path);
 
-    class wxMediaPlayerTimer* m_timer;     //Timer to write info to status bar
-    wxNotebook* m_notebook;     //Notebook containing our pages
+    class wxMediaPlayerTimer* m_timer;     // Timer to write info to status bar
+    wxNotebook* m_notebook;     // Notebook containing our pages
 
     // Maybe I should use more accessors, but for simplicity
     // I'll allow the other classes access to our members
@@ -237,30 +248,30 @@ class wxMediaPlayerNotebookPage : public wxPanel
     void OnMediaFinished(wxMediaEvent& event);
 
 public:
-    bool IsBeingDragged();      //accessor for m_bIsBeingDragged
+    bool IsBeingDragged();      // accessor for m_bIsBeingDragged
 
-    //make wxMediaPlayerFrame able to access the private members
+    // make wxMediaPlayerFrame able to access the private members
     friend class wxMediaPlayerFrame;
 
-    int      m_nLastFileId;     //List ID of played file in listctrl
-    wxString m_szFile;          //Name of currently playing file/location
+    int      m_nLastFileId;     // List ID of played file in listctrl
+    wxString m_szFile;          // Name of currently playing file/location
 
-    wxMediaCtrl* m_mediactrl;   //Our media control
-    class wxMediaPlayerListCtrl* m_playlist;  //Our playlist
-    wxSlider* m_slider;         //The slider below our media control
-    wxSlider* m_pbSlider;       //Lower-left slider for adjusting speed
-    wxSlider* m_volSlider;      //Lower-right slider for adjusting volume
-    int m_nLoops;               //Number of times media has looped
-    bool m_bLoop;               //Whether we are looping or not
-    bool m_bIsBeingDragged;     //Whether the user is dragging the scroll bar
-    wxMediaPlayerFrame* m_parentFrame;  //Main wxFrame of our sample
-    wxButton* m_prevButton;     //Go to previous file button
-    wxButton* m_playButton;     //Play/pause file button
-    wxButton* m_stopButton;     //Stop playing file button
-    wxButton* m_nextButton;     //Next file button
-    wxButton* m_vdButton;       //Volume down button
-    wxButton* m_vuButton;       //Volume up button
-    wxGauge*  m_gauge;          //Gauge to keep in line with slider
+    wxMediaCtrl* m_mediactrl;   // Our media control
+    class wxMediaPlayerListCtrl* m_playlist;  // Our playlist
+    wxSlider* m_slider;         // The slider below our media control
+    wxSlider* m_pbSlider;       // Lower-left slider for adjusting speed
+    wxSlider* m_volSlider;      // Lower-right slider for adjusting volume
+    int m_nLoops;               // Number of times media has looped
+    bool m_bLoop;               // Whether we are looping or not
+    bool m_bIsBeingDragged;     // Whether the user is dragging the scroll bar
+    wxMediaPlayerFrame* m_parentFrame;  // Main wxFrame of our sample
+    wxButton* m_prevButton;     // Go to previous file button
+    wxButton* m_playButton;     // Play/pause file button
+    wxButton* m_stopButton;     // Stop playing file button
+    wxButton* m_nextButton;     // Next file button
+    wxButton* m_vdButton;       // Volume down button
+    wxButton* m_vuButton;       // Volume up button
+    wxGauge*  m_gauge;          // Gauge to keep in line with slider
 };
 
 // ----------------------------------------------------------------------------
@@ -270,13 +281,13 @@ public:
 class wxMediaPlayerTimer : public wxTimer
 {
 public:
-    //Ctor
+    // Ctor
     wxMediaPlayerTimer(wxMediaPlayerFrame* frame) {m_frame = frame;}
 
-    //Called each time the timer's timeout expires
-    void Notify();
+    // Called each time the timer's timeout expires
+    void Notify() wxOVERRIDE;
 
-    wxMediaPlayerFrame* m_frame;       //The wxMediaPlayerFrame
+    wxMediaPlayerFrame* m_frame;       // The wxMediaPlayerFrame
 };
 
 // ----------------------------------------------------------------------------
@@ -330,8 +341,8 @@ public:
 // ----------------------------------------------------------------------------
 // wxPlayListDropTarget
 //
-//  Drop target for playlist (i.e. user drags a file from explorer unto
-//  playlist it adds the file)
+//  Drop target for playlist (i.e. allows users to drag a file from explorer into
+//  the playlist to add that file)
 // ----------------------------------------------------------------------------
 #if wxUSE_DRAG_AND_DROP
 class wxPlayListDropTarget : public wxFileDropTarget
@@ -340,7 +351,7 @@ public:
     wxPlayListDropTarget(wxMediaPlayerListCtrl& list) : m_list(list) {}
     ~wxPlayListDropTarget(){}
         virtual bool OnDropFiles(wxCoord WXUNUSED(x), wxCoord WXUNUSED(y),
-                         const wxArrayString& files)
+                         const wxArrayString& files) wxOVERRIDE
     {
         for (size_t i = 0; i < files.GetCount(); ++i)
         {
@@ -397,11 +408,39 @@ const wxChar* wxGetMediaStateText(int nState)
 // handle of the application.  These routines in wx _DO NOT_ check to see if
 // the wxApp exists, and thus will crash the application if you try it.
 //
-// IMPLEMENT_APP does this, and also implements the platform-specific entry
-// routine, such as main or WinMain().  Use IMPLEMENT_APP_NO_MAIN if you do
-// not desire this behavior.
+// wxIMPLEMENT_APP does this, and also implements the platform-specific entry
+// routine, such as main or WinMain().  Use wxIMPLEMENT_APP_NO_MAIN if you do
+// not desire this behaviour.
 // ----------------------------------------------------------------------------
-IMPLEMENT_APP(wxMediaPlayerApp)
+wxIMPLEMENT_APP(wxMediaPlayerApp);
+
+// ----------------------------------------------------------------------------
+// wxMediaPlayerApp command line parsing
+// ----------------------------------------------------------------------------
+
+#if wxUSE_CMDLINE_PARSER
+
+void wxMediaPlayerApp::OnInitCmdLine(wxCmdLineParser& parser)
+{
+    wxApp::OnInitCmdLine(parser);
+
+    parser.AddParam("input files",
+                    wxCMD_LINE_VAL_STRING,
+                    wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE);
+}
+
+bool wxMediaPlayerApp::OnCmdLineParsed(wxCmdLineParser& parser)
+{
+    if ( !wxApp::OnCmdLineParsed(parser) )
+        return false;
+
+    for (size_t paramNr=0; paramNr < parser.GetParamCount(); ++paramNr)
+        m_params.push_back(parser.GetParam(paramNr));
+
+    return true;
+}
+
+#endif // wxUSE_CMDLINE_PARSER
 
 // ----------------------------------------------------------------------------
 // wxMediaPlayerApp::OnInit
@@ -424,44 +463,25 @@ bool wxMediaPlayerApp::OnInit()
     frame->Show(true);
 
 #if wxUSE_CMDLINE_PARSER
-    //
-    //  What this does is get all the command line arguments
-    //  and treat each one as a file to put to the initial playlist
-    //
-    wxCmdLineEntryDesc cmdLineDesc[2];
-    cmdLineDesc[0].kind = wxCMD_LINE_PARAM;
-    cmdLineDesc[0].shortName = NULL;
-    cmdLineDesc[0].longName = NULL;
-    cmdLineDesc[0].description = "input files";
-    cmdLineDesc[0].type = wxCMD_LINE_VAL_STRING;
-    cmdLineDesc[0].flags = wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE;
-
-    cmdLineDesc[1].kind = wxCMD_LINE_NONE;
-
-    //gets the passed media files from cmd line
-    wxCmdLineParser parser (cmdLineDesc, argc, argv);
-
-    // get filenames from the commandline
-    if (parser.Parse() == 0)
+    if ( !m_params.empty() )
     {
-        for (size_t paramNr=0; paramNr < parser.GetParamCount(); ++paramNr)
-        {
-            frame->AddToPlayList((parser.GetParam (paramNr)));
-        }
-        wxCommandEvent theEvent(wxEVT_COMMAND_MENU_SELECTED, wxID_NEXT);
+        for ( size_t n = 0; n < m_params.size(); n++ )
+            frame->AddToPlayList(m_params[n]);
+
+        wxCommandEvent theEvent(wxEVT_MENU, wxID_NEXT);
         frame->AddPendingEvent(theEvent);
     }
-#endif
+#endif // wxUSE_CMDLINE_PARSER
 
     return true;
 }
 
 #ifdef __WXMAC__
 
-void wxMediaPlayerApp::MacOpenFile(const wxString & fileName )
+void wxMediaPlayerApp::MacOpenFiles(const wxArrayString & fileNames )
 {
-    //Called when a user drags a file over our app
-    m_frame->DoOpenFile(fileName, true /* new page */);
+    // Called when a user drags files over our app
+    m_frame->DoOpenFile(fileNames[0], true /* new page */);
 }
 
 #endif // __WXMAC__
@@ -530,7 +550,7 @@ wxMediaPlayerFrame::wxMediaPlayerFrame(const wxString& title)
                      wxT("Select a backend manually"));
 
     helpMenu->Append(wxID_ABOUT,
-                     wxT("&About...\tF1"),
+                     wxT("&About\tF1"),
                      wxT("Show about dialog"));
 
 
@@ -567,9 +587,9 @@ wxMediaPlayerFrame::wxMediaPlayerFrame(const wxString& title)
     //  class you want to use for events, such as wxMediaPlayerFrame.
     //
     //  Then after your class declaration you put
-    //  BEGIN_EVENT_TABLE(wxMediaPlayerFrame, wxFrame)
+    //  wxBEGIN_EVENT_TABLE(wxMediaPlayerFrame, wxFrame)
     //  EVT_XXX(XXX)...
-    //  END_EVENT_TABLE()
+    //  wxEND_EVENT_TABLE()
     //
     //  Where wxMediaPlayerFrame is the class with the DECLARE_MESSAGE_MAP
     //  in it.  EVT_XXX(XXX) are each of your handlers, such
@@ -628,46 +648,46 @@ wxMediaPlayerFrame::wxMediaPlayerFrame(const wxString& title)
     //
     // Menu events
     //
-    this->Connect(wxID_EXIT, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_EXIT, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnQuit));
 
-    this->Connect(wxID_ABOUT, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_ABOUT, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnAbout));
 
-    this->Connect(wxID_LOOP, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_LOOP, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnLoop));
 
-    this->Connect(wxID_SHOWINTERFACE, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_SHOWINTERFACE, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnShowInterface));
 
-    this->Connect(wxID_OPENFILENEWPAGE, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_OPENFILENEWPAGE, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnOpenFileNewPage));
 
-    this->Connect(wxID_OPENFILESAMEPAGE, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_OPENFILESAMEPAGE, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnOpenFileSamePage));
 
-    this->Connect(wxID_OPENURLNEWPAGE, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_OPENURLNEWPAGE, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnOpenURLNewPage));
 
-    this->Connect(wxID_OPENURLSAMEPAGE, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_OPENURLSAMEPAGE, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnOpenURLSamePage));
 
-    this->Connect(wxID_CLOSECURRENTPAGE, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_CLOSECURRENTPAGE, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnCloseCurrentPage));
 
-    this->Connect(wxID_PLAY, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_PLAY, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnPlay));
 
-    this->Connect(wxID_STOP, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_STOP, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnStop));
 
-    this->Connect(wxID_NEXT, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_NEXT, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnNext));
 
-    this->Connect(wxID_PREV, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_PREV, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnPrev));
 
-    this->Connect(wxID_SELECTBACKEND, wxEVT_COMMAND_MENU_SELECTED,
+    this->Connect(wxID_SELECTBACKEND, wxEVT_MENU,
                   wxCommandEventHandler(wxMediaPlayerFrame::OnSelectBackend));
 
     //
@@ -697,24 +717,34 @@ wxMediaPlayerFrame::wxMediaPlayerFrame(const wxString& title)
                         wxT(""),
                         true);
 
-    //
-    //  Here we load the our configuration -
-    //  in our case we load all the files that were left in
-    //  the playlist the last time the user closed our application
-    //
-    //  As an exercise to the reader try modifying it so that
-    //  it properly loads the playlist for each page without
-    //  conflicting (loading the same data) with the other ones.
-    //
-    wxConfig conf;
-    wxString key, outstring;
-    for(int i = 0; ; ++i)
+
+    // Don't load previous files if we have some specified on the command line,
+    // we wouldn't play them otherwise (they'd have to be inserted into the
+    // play list at the beginning instead of being appended but we don't
+    // support this).
+#if wxUSE_CMDLINE_PARSER
+    if ( wxGetApp().m_params.empty() )
+#endif // wxUSE_CMDLINE_PARSER
     {
-        key.clear();
-        key << i;
-        if(!conf.Read(key, &outstring))
-            break;
-        page->m_playlist->AddToPlayList(outstring);
+        //
+        //  Here we load the our configuration -
+        //  in our case we load all the files that were left in
+        //  the playlist the last time the user closed our application
+        //
+        //  As an exercise to the reader try modifying it so that
+        //  it properly loads the playlist for each page without
+        //  conflicting (loading the same data) with the other ones.
+        //
+        wxConfig conf;
+        wxString key, outstring;
+        for(int i = 0; ; ++i)
+        {
+            key.clear();
+            key << i;
+            if(!conf.Read(key, &outstring))
+                break;
+            page->m_playlist->AddToPlayList(outstring);
+        }
     }
 
     //
@@ -776,7 +806,7 @@ wxMediaPlayerFrame::~wxMediaPlayerFrame()
 // ----------------------------------------------------------------------------
 void wxMediaPlayerFrame::OnClose(wxCloseEvent& event)
 {
-    event.Skip(); //really close the frame
+    event.Skip(); // really close the frame
 }
 
 // ----------------------------------------------------------------------------
@@ -813,14 +843,14 @@ void wxMediaPlayerFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
     wxString msg;
     msg.Printf( wxT("This is a test of wxMediaCtrl.\n\n")
 
-                wxT("Intructions:\n")
+                wxT("Instructions:\n")
 
                 wxT("The top slider shows the current the current position, ")
                 wxT("which you can change by dragging and releasing it.\n")
 
                 wxT("The gauge (progress bar) shows the progress in ")
                 wxT("downloading data of the current file - it may always be ")
-                wxT("Empty due to lack of support from the current backend.\n")
+                wxT("empty due to lack of support from the current backend.\n")
 
                 wxT("The lower-left slider controls the volume and the lower-")
                 wxT("right slider controls the playback rate/speed of the ")
@@ -861,7 +891,7 @@ void wxMediaPlayerFrame::OnShowInterface(wxCommandEvent& event)
             wxMEDIACTRLPLAYERCONTROLS_DEFAULT :
              wxMEDIACTRLPLAYERCONTROLS_NONE)    )
     {
-        //error - uncheck and warn user
+        // error - uncheck and warn user
         wxMenuItem* pSIItem = GetMenuBar()->FindItem(wxID_SHOWINTERFACE);
         wxASSERT(pSIItem);
         pSIItem->Check(!event.IsChecked());
@@ -980,7 +1010,7 @@ void wxMediaPlayerFrame::DoPlayFile(const wxString& path)
       )
     {
         if(currentpage->m_mediactrl->GetState() == wxMEDIASTATE_PLAYING)
-    {
+        {
             if( !currentpage->m_mediactrl->Pause() )
                 wxMessageBox(wxT("Couldn't pause movie!"));
         }
@@ -1071,7 +1101,7 @@ void wxMediaPlayerFrame::OnSelectBackend(wxCommandEvent& WXUNUSED(evt))
 {
     wxString sBackend = wxGetTextFromUser(wxT("Enter backend to use"));
 
-    if(sBackend.empty() == false)  //could have been cancelled by the user
+    if(sBackend.empty() == false)  // could have been cancelled by the user
     {
         int sel = m_notebook->GetSelection();
 
@@ -1124,7 +1154,7 @@ void wxMediaPlayerFrame::OpenURL(bool bNewPage)
         wxT("Enter the URL that has the movie to play")
                                      );
 
-    if(sUrl.empty() == false) //could have been cancelled by user
+    if(sUrl.empty() == false) // could have been cancelled by user
     {
         DoOpenFile(sUrl, bNewPage);
     }
@@ -1176,7 +1206,7 @@ void wxMediaPlayerFrame::OnPlay(wxCommandEvent& WXUNUSED(event))
                                          wxLIST_NEXT_ALL,
                                          wxLIST_STATE_DONTCARE)) == -1)
         {
-            //no items in list
+            // no items in list
             wxMessageBox(wxT("No items in playlist!"));
     }
         else
@@ -1208,7 +1238,7 @@ void wxMediaPlayerFrame::OnKeyDown(wxKeyEvent& event)
     {
         wxMediaPlayerNotebookPage* currentpage =
             (wxMediaPlayerNotebookPage*) m_notebook->GetCurrentPage();
-       //delete all selected items
+       // delete all selected items
        while(true)
        {
            wxInt32 nSelectedItem = currentpage->m_playlist->GetNextItem(
@@ -1225,7 +1255,7 @@ void wxMediaPlayerFrame::OnKeyDown(wxKeyEvent& event)
        }
     }
 
-   //Could be wxGetTextFromUser or something else important
+   // Could be wxGetTextFromUser or something else important
    if(event.GetEventObject() != this)
        event.Skip();
 }
@@ -1267,7 +1297,7 @@ void wxMediaPlayerFrame::OnChangeSong(wxListEvent& WXUNUSED(evt))
     wxListItem listitem;
     currentpage->m_playlist->GetSelectedItem(listitem);
     if(listitem.GetData())
-    DoPlayFile((*((wxString*) listitem.GetData())));
+        DoPlayFile((*((wxString*) listitem.GetData())));
     else
         wxMessageBox(wxT("No selected item!"));
 }
@@ -1299,7 +1329,7 @@ void wxMediaPlayerFrame::OnPrev(wxCommandEvent& WXUNUSED(event))
 
     if (nLastSelectedItem == -1)
     {
-        //nothing selected, default to the file before the currently playing one
+        // nothing selected, default to the file before the currently playing one
         if(currentpage->m_nLastFileId == 0)
             nLastSelectedItem = currentpage->m_playlist->GetItemCount() - 1;
     else
@@ -1311,7 +1341,7 @@ void wxMediaPlayerFrame::OnPrev(wxCommandEvent& WXUNUSED(event))
         nLastSelectedItem -= 1;
 
     if(nLastSelectedItem == currentpage->m_nLastFileId)
-        return; //already playing... nothing to do
+        return; // already playing... nothing to do
 
     wxListItem listitem;
     listitem.SetId(nLastSelectedItem);
@@ -1363,7 +1393,7 @@ void wxMediaPlayerFrame::OnNext(wxCommandEvent& WXUNUSED(event))
             nLastSelectedItem += 1;
 
     if(nLastSelectedItem == currentpage->m_nLastFileId)
-        return; //already playing... nothing to do
+        return; // already playing... nothing to do
 
     wxListItem listitem;
     listitem.SetMask(wxLIST_MASK_TEXT |  wxLIST_MASK_DATA);
@@ -1547,14 +1577,14 @@ wxMediaPlayerNotebookPage::wxMediaPlayerNotebookPage(wxMediaPlayerFrame* parentF
     //  Make sure creation was successful
     bool bOK = m_mediactrl->Create(this, wxID_MEDIACTRL, wxEmptyString,
                                     wxDefaultPosition, wxDefaultSize, 0,
-//you could specify a macrod backend here like
+// you could specify a macro backend here like
 //  wxMEDIABACKEND_WMP10);
 //        wxT("wxPDFMediaBackend"));
                                    szBackend);
-//you could change the cursor here like
+// you could change the cursor here like
 //    m_mediactrl->SetCursor(wxCURSOR_BLANK);
-//note that this may not effect it if SetPlayerControls
-//is set to something else than wxMEDIACTRLPLAYERCONTROLS_NONE
+// note that this may not effect it if SetPlayerControls
+// is set to something else than wxMEDIACTRLPLAYERCONTROLS_NONE
     wxASSERT_MSG(bOK, wxT("Could not create media control!"));
     wxUnusedVar(bOK);
 
@@ -1566,11 +1596,11 @@ wxMediaPlayerNotebookPage::wxMediaPlayerNotebookPage(wxMediaPlayerFrame* parentF
     m_playlist = new wxMediaPlayerListCtrl();
     m_playlist->Create(this, wxID_LISTCTRL, wxDefaultPosition,
                     wxDefaultSize,
-                    wxLC_REPORT //wxLC_LIST
+                    wxLC_REPORT // wxLC_LIST
                     | wxSUNKEN_BORDER);
 
     //  Set the background of our listctrl to white
-    m_playlist->SetBackgroundColour(wxColour(255,255,255));
+    m_playlist->SetBackgroundColour(*wxWHITE);
 
     //  The layout of the headers of the listctrl are like
     //  |   | File               |  Length
@@ -1587,9 +1617,9 @@ wxMediaPlayerNotebookPage::wxMediaPlayerNotebookPage(wxMediaPlayerFrame* parentF
     //  Column two is the name of the file
     //
     //  Column three is the length in seconds of the file
-    m_playlist->InsertColumn(0,_(""), wxLIST_FORMAT_CENTER, 20);
-    m_playlist->InsertColumn(1,_("File"), wxLIST_FORMAT_LEFT, /*wxLIST_AUTOSIZE_USEHEADER*/305);
-    m_playlist->InsertColumn(2,_("Length"), wxLIST_FORMAT_CENTER, 75);
+    m_playlist->AppendColumn(_(""), wxLIST_FORMAT_CENTER, 20);
+    m_playlist->AppendColumn(_("File"), wxLIST_FORMAT_LEFT, /*wxLIST_AUTOSIZE_USEHEADER*/305);
+    m_playlist->AppendColumn(_("Length"), wxLIST_FORMAT_CENTER, 75);
 
 #if wxUSE_DRAG_AND_DROP
     m_playlist->SetDropTarget(new wxPlayListDropTarget(*m_playlist));
@@ -1613,11 +1643,18 @@ wxMediaPlayerNotebookPage::wxMediaPlayerNotebookPage(wxMediaPlayerFrame* parentF
     m_vuButton = new wxButton();
 
     m_prevButton->Create(this, wxID_BUTTONPREV, wxT("|<"));
+    m_prevButton->SetToolTip("Previous");
     m_playButton->Create(this, wxID_BUTTONPLAY, wxT(">"));
+    m_playButton->SetToolTip("Play");
     m_stopButton->Create(this, wxID_BUTTONSTOP, wxT("[]"));
+    m_stopButton->SetToolTip("Stop");
     m_nextButton->Create(this, wxID_BUTTONNEXT, wxT(">|"));
+    m_nextButton->SetToolTip("Next");
     m_vdButton->Create(this, wxID_BUTTONVD, wxT("(("));
+    m_vdButton->SetToolTip("Volume down");
     m_vuButton->Create(this, wxID_BUTTONVU, wxT("))"));
+    m_vuButton->SetToolTip("Volume up");
+
     vertsizer->Add(m_prevButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
     vertsizer->Add(m_playButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
     vertsizer->Add(m_stopButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
@@ -1631,9 +1668,9 @@ wxMediaPlayerNotebookPage::wxMediaPlayerNotebookPage(wxMediaPlayerFrame* parentF
     //
     //  Create our slider
     //
-    m_slider = new wxSlider(this, wxID_SLIDER, 0, //init
-                            0, //start
-                            0, //end
+    m_slider = new wxSlider(this, wxID_SLIDER, 0, // init
+                            0, // start
+                            1, // end, dummy but must be greater than start
                             wxDefaultPosition, wxDefaultSize,
                             wxSL_HORIZONTAL );
     sizer->Add(m_slider, 0, wxALIGN_CENTER_HORIZONTAL|wxALL|wxEXPAND , 5);
@@ -1673,7 +1710,7 @@ wxMediaPlayerNotebookPage::wxMediaPlayerNotebookPage(wxMediaPlayerFrame* parentF
     //
     // ListCtrl events
     //
-    this->Connect( wxID_LISTCTRL, wxEVT_COMMAND_LIST_ITEM_ACTIVATED,
+    this->Connect( wxID_LISTCTRL, wxEVT_LIST_ITEM_ACTIVATED,
         wxListEventHandler(wxMediaPlayerFrame::OnChangeSong),
         (wxObject*)0, parentFrame);
 
@@ -1707,22 +1744,22 @@ wxMediaPlayerNotebookPage::wxMediaPlayerNotebookPage(wxMediaPlayerFrame* parentF
     //
     // Button events
     //
-    this->Connect( wxID_BUTTONPREV, wxEVT_COMMAND_BUTTON_CLICKED,
+    this->Connect( wxID_BUTTONPREV, wxEVT_BUTTON,
         wxCommandEventHandler(wxMediaPlayerFrame::OnPrev),
         (wxObject*)0, parentFrame);
-    this->Connect( wxID_BUTTONPLAY, wxEVT_COMMAND_BUTTON_CLICKED,
+    this->Connect( wxID_BUTTONPLAY, wxEVT_BUTTON,
         wxCommandEventHandler(wxMediaPlayerFrame::OnPlay),
         (wxObject*)0, parentFrame);
-    this->Connect( wxID_BUTTONSTOP, wxEVT_COMMAND_BUTTON_CLICKED,
+    this->Connect( wxID_BUTTONSTOP, wxEVT_BUTTON,
         wxCommandEventHandler(wxMediaPlayerFrame::OnStop),
         (wxObject*)0, parentFrame);
-    this->Connect( wxID_BUTTONNEXT, wxEVT_COMMAND_BUTTON_CLICKED,
+    this->Connect( wxID_BUTTONNEXT, wxEVT_BUTTON,
         wxCommandEventHandler(wxMediaPlayerFrame::OnNext),
         (wxObject*)0, parentFrame);
-    this->Connect( wxID_BUTTONVD, wxEVT_COMMAND_BUTTON_CLICKED,
+    this->Connect( wxID_BUTTONVD, wxEVT_BUTTON,
         wxCommandEventHandler(wxMediaPlayerFrame::OnVolumeDown),
         (wxObject*)0, parentFrame);
-    this->Connect( wxID_BUTTONVU, wxEVT_COMMAND_BUTTON_CLICKED,
+    this->Connect( wxID_BUTTONVU, wxEVT_BUTTON,
         wxCommandEventHandler(wxMediaPlayerFrame::OnVolumeUp),
         (wxObject*)0, parentFrame);
 }
