@@ -11,7 +11,7 @@
 #define _EDIT_H_
 
 //----------------------------------------------------------------------------
-// informations
+// information
 //----------------------------------------------------------------------------
 
 
@@ -79,7 +79,7 @@ public:
     void OnEditSelectAll (wxCommandEvent &event);
     void OnEditSelectLine (wxCommandEvent &event);
     //! view
-    void OnHilightLang (wxCommandEvent &event);
+    void OnHighlightLang (wxCommandEvent &event);
     void OnDisplayEOL (wxCommandEvent &event);
     void OnIndentGuide (wxCommandEvent &event);
     void OnLineNumber (wxCommandEvent &event);
@@ -98,11 +98,16 @@ public:
     //! extra
     void OnChangeCase (wxCommandEvent &event);
     void OnConvertEOL (wxCommandEvent &event);
+    void OnMultipleSelections(wxCommandEvent& event);
+    void OnMultiPaste(wxCommandEvent& event);
+    void OnMultipleSelectionsTyping(wxCommandEvent& event);
     // stc
     void OnMarginClick (wxStyledTextEvent &event);
     void OnCharAdded  (wxStyledTextEvent &event);
     void OnKey  (wxStyledTextEvent &event);
 
+    void OnKeyDown(wxKeyEvent &event);
+
     //! language/lexer
     wxString DeterminePrefs (const wxString &filename);
     bool InitializePrefs (const wxString &filename);
@@ -122,7 +127,7 @@ private:
     // file
     wxString m_filename;
 
-    // lanugage properties
+    // language properties
     LanguageInfo const* m_language;
 
     // margin variables
@@ -132,7 +137,7 @@ private:
     int m_FoldingMargin;
     int m_DividerID;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 //----------------------------------------------------------------------------
