@@ -4,7 +4,6 @@
 // Author:      Benjamin I. Williams
 // Modified by:
 // Created:     2005-05-17
-// RCS-ID:      $Id$
 // Copyright:   (C) Copyright 2005-2006, Kirix Corporation, All Rights Reserved
 // Licence:     wxWindows Library Licence, Version 3.1
 ///////////////////////////////////////////////////////////////////////////////
@@ -36,7 +35,7 @@
 #include "wx/msw/private.h"
 #endif
 
-IMPLEMENT_CLASS(wxAuiFloatingFrame, wxAuiFloatingFrameBaseClass)
+wxIMPLEMENT_CLASS(wxAuiFloatingFrame, wxAuiFloatingFrameBaseClass);
 
 wxAuiFloatingFrame::wxAuiFloatingFrame(wxWindow* parent,
                 wxAuiManager* owner_mgr,
@@ -54,17 +53,17 @@ wxAuiFloatingFrame::wxAuiFloatingFrame(wxWindow* parent,
                         (pane.IsFixed()?0:wxRESIZE_BORDER)
                         )
 {
-    m_owner_mgr = owner_mgr;
+    m_ownerMgr = owner_mgr;
     m_moving = false;
     m_mgr.SetManagedWindow(this);
-    m_solid_drag = true;
+    m_solidDrag = true;
 
     // find out if the system supports solid window drag.
     // on non-msw systems, this is assumed to be the case
 #ifdef __WXMSW__
     BOOL b = TRUE;
     SystemParametersInfo(38 /*SPI_GETDRAGFULLWINDOWS*/, 0, &b, 0);
-    m_solid_drag = b ? true : false;
+    m_solidDrag = b ? true : false;
 #endif
 
     SetExtraStyle(wxWS_EX_PROCESS_IDLE);
@@ -73,9 +72,9 @@ wxAuiFloatingFrame::wxAuiFloatingFrame(wxWindow* parent,
 wxAuiFloatingFrame::~wxAuiFloatingFrame()
 {
     // if we do not do this, then we can crash...
-    if (m_owner_mgr && m_owner_mgr->m_action_window == this)
+    if (m_ownerMgr && m_ownerMgr->m_actionWindow == this)
     {
-        m_owner_mgr->m_action_window = NULL;
+        m_ownerMgr->m_actionWindow = NULL;
     }
 
     m_mgr.UnInit();
@@ -83,8 +82,8 @@ wxAuiFloatingFrame::~wxAuiFloatingFrame()
 
 void wxAuiFloatingFrame::SetPaneWindow(const wxAuiPaneInfo& pane)
 {
-    m_pane_window = pane.window;
-    m_pane_window->Reparent(this);
+    m_paneWindow = pane.window;
+    m_paneWindow->Reparent(this);
 
     wxAuiPaneInfo contained_pane = pane;
     contained_pane.Dock().Center().Show().
@@ -108,7 +107,7 @@ void wxAuiFloatingFrame::SetPaneWindow(const wxAuiPaneInfo& pane)
 
     SetMinSize(pane.window->GetMinSize());
 
-    m_mgr.AddPane(m_pane_window, contained_pane);
+    m_mgr.AddPane(m_paneWindow, contained_pane);
     m_mgr.Update();
 
     if (pane.min_size.IsFullySpecified())
@@ -123,7 +122,22 @@ void wxAuiFloatingFrame::SetPaneWindow(const wxAuiPaneInfo& pane)
 
     SetTitle(pane.caption);
 
-    if (pane.floating_size != wxDefaultSize)
+    // This code is slightly awkward because we need to reset wxRESIZE_BORDER
+    // before calling SetClientSize() below as doing it after setting the
+    // client size would actually change it, at least under MSW, where the
+    // total window size doesn't change and hence, as the borders size changes,
+    // the client size does change.
+    //
+    // So we must call it first but doing it generates a size event and updates
+    // pane.floating_size from inside it so we must also record its original
+    // value before doing it.
+    const bool hasFloatingSize = pane.floating_size != wxDefaultSize;
+    if (pane.IsFixed())
+    {
+        SetWindowStyleFlag(GetWindowStyleFlag() & ~wxRESIZE_BORDER);
+    }
+
+    if ( hasFloatingSize )
     {
         SetSize(pane.floating_size);
     }
@@ -133,54 +147,65 @@ void wxAuiFloatingFrame::SetPaneWindow(const wxAuiPaneInfo& pane)
         if (size == wxDefaultSize)
             size = pane.min_size;
         if (size == wxDefaultSize)
-            size = m_pane_window->GetSize();
-        if (m_owner_mgr && pane.HasGripper())
+            size = m_paneWindow->GetSize();
+        if (m_ownerMgr && pane.HasGripper())
         {
             if (pane.HasGripperTop())
-                size.y += m_owner_mgr->m_art->GetMetric(wxAUI_DOCKART_GRIPPER_SIZE);
+                size.y += m_ownerMgr->m_art->GetMetric(wxAUI_DOCKART_GRIPPER_SIZE);
             else
-                size.x += m_owner_mgr->m_art->GetMetric(wxAUI_DOCKART_GRIPPER_SIZE);
+                size.x += m_ownerMgr->m_art->GetMetric(wxAUI_DOCKART_GRIPPER_SIZE);
         }
 
         SetClientSize(size);
     }
-
-    if (pane.IsFixed())
-    {
-        SetWindowStyleFlag(GetWindowStyleFlag() & ~wxRESIZE_BORDER);
-    }
 }
 
 wxAuiManager* wxAuiFloatingFrame::GetOwnerManager() const
 {
-    return m_owner_mgr;
+    return m_ownerMgr;
 }
 
+bool wxAuiFloatingFrame::IsTopNavigationDomain(NavigationKind kind) const
+{
+    switch ( kind )
+    {
+        case Navigation_Tab:
+            break;
+
+        case Navigation_Accel:
+            // Floating frames are often used as tool palettes and it's
+            // convenient for the accelerators defined in the parent frame to
+            // work in them, so don't block their propagation.
+            return false;
+    }
+
+    return wxAuiFloatingFrameBaseClass::IsTopNavigationDomain(kind);
+}
 
 void wxAuiFloatingFrame::OnSize(wxSizeEvent& WXUNUSED(event))
 {
-    if (m_owner_mgr)
+    if (m_ownerMgr)
     {
-        m_owner_mgr->OnFloatingPaneResized(m_pane_window, GetRect());
+        m_ownerMgr->OnFloatingPaneResized(m_paneWindow, GetRect());
     }
 }
 
 void wxAuiFloatingFrame::OnClose(wxCloseEvent& evt)
 {
-    if (m_owner_mgr)
+    if (m_ownerMgr)
     {
-        m_owner_mgr->OnFloatingPaneClosed(m_pane_window, evt);
+        m_ownerMgr->OnFloatingPaneClosed(m_paneWindow, evt);
     }
     if (!evt.GetVeto())
     {
-        m_mgr.DetachPane(m_pane_window);
+        m_mgr.DetachPane(m_paneWindow);
         Destroy();
     }
 }
 
 void wxAuiFloatingFrame::OnMoveEvent(wxMoveEvent& event)
 {
-    if (!m_solid_drag)
+    if (!m_solidDrag)
     {
         // systems without solid window dragging need to be
         // handled slightly differently, due to the lack of
@@ -194,61 +219,74 @@ void wxAuiFloatingFrame::OnMoveEvent(wxMoveEvent& event)
     }
 
 
-    wxRect win_rect = GetRect();
+    wxRect winRect = GetRect();
 
-    if (win_rect == m_last_rect)
+    if (winRect == m_lastRect)
         return;
 
     // skip the first move event
-    if (m_last_rect.IsEmpty())
+    if (m_lastRect.IsEmpty())
     {
-        m_last_rect = win_rect;
+        m_lastRect = winRect;
         return;
     }
 
+    // as on OSX moving windows are not getting all move events, only sporadically, this difference
+    // is almost always big on OSX, so avoid this early exit opportunity
+#ifndef __WXOSX__
     // skip if moving too fast to avoid massive redraws and
     // jumping hint windows
-    if ((abs(win_rect.x - m_last_rect.x) > 3) ||
-        (abs(win_rect.y - m_last_rect.y) > 3))
+    if ((abs(winRect.x - m_lastRect.x) > 3) ||
+        (abs(winRect.y - m_lastRect.y) > 3))
     {
-        m_last3_rect = m_last2_rect;
-        m_last2_rect = m_last_rect;
-        m_last_rect = win_rect;
+        m_last3Rect = m_last2Rect;
+        m_last2Rect = m_lastRect;
+        m_lastRect = winRect;
+
+        // However still update the internally stored position to avoid
+        // snapping back to the old one later.
+        if (m_ownerMgr)
+        {
+            m_ownerMgr->GetPane(m_paneWindow).
+                floating_pos = winRect.GetPosition();
+        }
+
         return;
     }
+#endif
 
     // prevent frame redocking during resize
-    if (m_last_rect.GetSize() != win_rect.GetSize())
+    if (m_lastRect.GetSize() != winRect.GetSize())
     {
-        m_last3_rect = m_last2_rect;
-        m_last2_rect = m_last_rect;
-        m_last_rect = win_rect;
+        m_last3Rect = m_last2Rect;
+        m_last2Rect = m_lastRect;
+        m_lastRect = winRect;
         return;
     }
 
     wxDirection dir = wxALL;
 
-    int horiz_dist = abs(win_rect.x - m_last3_rect.x);
-    int vert_dist = abs(win_rect.y - m_last3_rect.y);
+    int horiz_dist = abs(winRect.x - m_last3Rect.x);
+    int vert_dist = abs(winRect.y - m_last3Rect.y);
 
     if (vert_dist >= horiz_dist)
     {
-        if (win_rect.y < m_last3_rect.y)
+        if (winRect.y < m_last3Rect.y)
             dir = wxNORTH;
         else
             dir = wxSOUTH;
     }
     else
     {
-        if (win_rect.x < m_last3_rect.x)
+        if (winRect.x < m_last3Rect.x)
             dir = wxWEST;
         else
             dir = wxEAST;
     }
 
-    m_last3_rect = m_last2_rect;
-    m_last2_rect = m_last_rect;
-    m_last_rect = win_rect;
+    m_last3Rect = m_last2Rect;
+    m_last2Rect = m_lastRect;
+    m_lastRect = winRect;
 
     if (!isMouseDown())
         return;
@@ -259,10 +297,13 @@ void wxAuiFloatingFrame::OnMoveEvent(wxMoveEvent& event)
         m_moving = true;
     }
 
-    if (m_last3_rect.IsEmpty())
+    if (m_last3Rect.IsEmpty())
         return;
 
-    OnMoving(event.GetRect(), dir);
+    if ( event.GetEventType() == wxEVT_MOVING )
+        OnMoving(event.GetRect(), dir);
+    else
+        OnMoving(wxRect(event.GetPosition(),GetSize()), dir);
 }
 
 void wxAuiFloatingFrame::OnIdle(wxIdleEvent& event)
@@ -284,18 +325,18 @@ void wxAuiFloatingFrame::OnIdle(wxIdleEvent& event)
 void wxAuiFloatingFrame::OnMoveStart()
 {
     // notify the owner manager that the pane has started to move
-    if (m_owner_mgr)
+    if (m_ownerMgr)
     {
-        m_owner_mgr->OnFloatingPaneMoveStart(m_pane_window);
+        m_ownerMgr->OnFloatingPaneMoveStart(m_paneWindow);
     }
 }
 
 void wxAuiFloatingFrame::OnMoving(const wxRect& WXUNUSED(window_rect), wxDirection dir)
 {
     // notify the owner manager that the pane is moving
-    if (m_owner_mgr)
+    if (m_ownerMgr)
     {
-        m_owner_mgr->OnFloatingPaneMoving(m_pane_window, dir);
+        m_ownerMgr->OnFloatingPaneMoving(m_paneWindow, dir);
     }
     m_lastDirection = dir;
 }
@@ -303,22 +344,22 @@ void wxAuiFloatingFrame::OnMoving(const wxRect& WXUNUSED(window_rect), wxDirecti
 void wxAuiFloatingFrame::OnMoveFinished()
 {
     // notify the owner manager that the pane has finished moving
-    if (m_owner_mgr)
+    if (m_ownerMgr)
     {
-        m_owner_mgr->OnFloatingPaneMoved(m_pane_window, m_lastDirection);
+        m_ownerMgr->OnFloatingPaneMoved(m_paneWindow, m_lastDirection);
     }
 }
 
 void wxAuiFloatingFrame::OnActivate(wxActivateEvent& event)
 {
-    if (m_owner_mgr && event.GetActive())
+    if (m_ownerMgr && event.GetActive())
     {
-        m_owner_mgr->OnFloatingPaneActivated(m_pane_window);
+        m_ownerMgr->OnFloatingPaneActivated(m_paneWindow);
     }
 }
 
 // utility function which determines the state of the mouse button
-// (independant of having a wxMouseEvent handy) - utimately a better
+// (independent of having a wxMouseEvent handy) - utimately a better
 // mechanism for this should be found (possibly by adding the
 // functionality to wxWidgets itself)
 bool wxAuiFloatingFrame::isMouseDown()
@@ -327,14 +368,14 @@ bool wxAuiFloatingFrame::isMouseDown()
 }
 
 
-BEGIN_EVENT_TABLE(wxAuiFloatingFrame, wxAuiFloatingFrameBaseClass)
+wxBEGIN_EVENT_TABLE(wxAuiFloatingFrame, wxAuiFloatingFrameBaseClass)
     EVT_SIZE(wxAuiFloatingFrame::OnSize)
     EVT_MOVE(wxAuiFloatingFrame::OnMoveEvent)
     EVT_MOVING(wxAuiFloatingFrame::OnMoveEvent)
     EVT_CLOSE(wxAuiFloatingFrame::OnClose)
     EVT_IDLE(wxAuiFloatingFrame::OnIdle)
     EVT_ACTIVATE(wxAuiFloatingFrame::OnActivate)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 
 #endif // wxUSE_AUI
