@@ -4,7 +4,6 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     05.01.00
-// RCS-ID:      $Id$
 // Copyright:   (c) 2000 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -65,95 +64,78 @@ static wxString GetLongOptionName(wxString::const_iterator p,
 // private structs
 // ----------------------------------------------------------------------------
 
-// an internal representation of an option
-struct wxCmdLineOption
+
+class wxCmdLineArgImpl: public wxCmdLineArg
 {
-    wxCmdLineOption(wxCmdLineEntryType k,
-                    const wxString& shrt,
-                    const wxString& lng,
-                    const wxString& desc,
-                    wxCmdLineParamType typ,
-                    int fl)
-    {
-        // wxCMD_LINE_USAGE_TEXT uses only description, shortName and longName is empty
-        if ( k != wxCMD_LINE_USAGE_TEXT )
-        {
-            wxASSERT_MSG
-            (
-                !shrt.empty() || !lng.empty(),
-                 wxT("option should have at least one name")
-            );
+public:
+    wxCmdLineArgImpl(wxCmdLineEntryType k,
+                 const wxString& shrt,
+                 const wxString& lng,
+                 wxCmdLineParamType typ);
 
-            wxASSERT_MSG
-            (
-                GetShortOptionName(shrt.begin(), shrt.end()).Len() == shrt.Len(),
-                wxT("Short option contains invalid characters")
-            );
-
-            wxASSERT_MSG
-            (
-                GetLongOptionName(lng.begin(), lng.end()).Len() == lng.Len(),
-                wxT("Long option contains invalid characters")
-            );
-        }
-
-        kind = k;
-
-        shortName = shrt;
-        longName = lng;
-        description = desc;
-
-        type = typ;
-        flags = fl;
-
-        m_hasVal = false;
-    }
-
-    // can't use union easily here, so just store all possible data fields, we
-    // don't waste much (might still use union later if the number of supported
-    // types increases, so always use the accessor functions and don't access
-    // the fields directly!)
-
-    void Check(wxCmdLineParamType WXUNUSED_UNLESS_DEBUG(typ)) const
-    {
-        wxASSERT_MSG( type == typ, wxT("type mismatch in wxCmdLineOption") );
-    }
-
-    double GetDoubleVal() const
-        { Check(wxCMD_LINE_VAL_DOUBLE); return m_doubleVal; }
-    long GetLongVal() const
-        { Check(wxCMD_LINE_VAL_NUMBER); return m_longVal; }
-    const wxString& GetStrVal() const
-        { Check(wxCMD_LINE_VAL_STRING); return m_strVal;  }
+    wxCmdLineArgImpl& SetDoubleVal(double val);
+    wxCmdLineArgImpl& SetLongVal(long val);
+    wxCmdLineArgImpl& SetStrVal(const wxString& val);
 #if wxUSE_DATETIME
-    const wxDateTime& GetDateVal() const
-        { Check(wxCMD_LINE_VAL_DATE);   return m_dateVal; }
+    wxCmdLineArgImpl& SetDateVal(const wxDateTime& val);
 #endif // wxUSE_DATETIME
 
-    void SetDoubleVal(double val)
-        { Check(wxCMD_LINE_VAL_DOUBLE); m_doubleVal = val; m_hasVal = true; }
-    void SetLongVal(long val)
-        { Check(wxCMD_LINE_VAL_NUMBER); m_longVal = val; m_hasVal = true; }
-    void SetStrVal(const wxString& val)
-        { Check(wxCMD_LINE_VAL_STRING); m_strVal = val; m_hasVal = true; }
-#if wxUSE_DATETIME
-    void SetDateVal(const wxDateTime& val)
-        { Check(wxCMD_LINE_VAL_DATE); m_dateVal = val; m_hasVal = true; }
-#endif // wxUSE_DATETIME
-
-    void SetHasValue(bool hasValue = true) { m_hasVal = hasValue; }
     bool HasValue() const { return m_hasVal; }
+    wxCmdLineArgImpl& SetHasValue() { m_hasVal = true; return *this; }
+
+    wxCmdLineArgImpl& SetNegated() { m_isNegated = true; return *this; }
+
+    // Reset to the initial state, called before parsing another command line.
+    void Reset()
+    {
+        m_hasVal =
+        m_isNegated = false;
+    }
 
 public:
     wxCmdLineEntryType kind;
     wxString shortName,
-             longName,
-             description;
+             longName;
     wxCmdLineParamType type;
-    int flags;
+
+    // from wxCmdLineArg
+    virtual wxCmdLineEntryType GetKind() const wxOVERRIDE { return kind; }
+    virtual wxString GetShortName() const wxOVERRIDE {
+        wxASSERT_MSG( kind == wxCMD_LINE_OPTION || kind == wxCMD_LINE_SWITCH,
+                      wxT("kind mismatch in wxCmdLineArg") );
+        return shortName;
+    }
+    virtual wxString GetLongName() const wxOVERRIDE {
+        wxASSERT_MSG( kind == wxCMD_LINE_OPTION || kind == wxCMD_LINE_SWITCH,
+                      wxT("kind mismatch in wxCmdLineArg") );
+        return longName;
+    }
+    virtual wxCmdLineParamType GetType() const wxOVERRIDE {
+        wxASSERT_MSG( kind == wxCMD_LINE_OPTION,
+                      wxT("kind mismatch in wxCmdLineArg") );
+        return type;
+    }
+    double GetDoubleVal() const wxOVERRIDE;
+    long GetLongVal() const wxOVERRIDE;
+    const wxString& GetStrVal() const wxOVERRIDE;
+#if wxUSE_DATETIME
+    const wxDateTime& GetDateVal() const wxOVERRIDE;
+#endif // wxUSE_DATETIME
+    bool IsNegated() const wxOVERRIDE {
+        wxASSERT_MSG( kind == wxCMD_LINE_SWITCH,
+                      wxT("kind mismatch in wxCmdLineArg") );
+        return m_isNegated;
+    }
 
 private:
+    // can't use union easily here, so just store all possible data fields, we
+    // don't waste much (might still use union later if the number of supported
+    // types increases)
+
+    void Check(wxCmdLineParamType typ) const;
+
     bool m_hasVal;
+    bool m_isNegated;
 
     double m_doubleVal;
     long m_longVal;
@@ -161,6 +143,27 @@ private:
 #if wxUSE_DATETIME
     wxDateTime m_dateVal;
 #endif // wxUSE_DATETIME
+
+};
+
+// an internal representation of an option
+struct wxCmdLineOption: public wxCmdLineArgImpl
+{
+    wxCmdLineOption(wxCmdLineEntryType k,
+                    const wxString& shrt,
+                    const wxString& lng,
+                    const wxString& desc,
+                    wxCmdLineParamType typ,
+                    int fl)
+                    : wxCmdLineArgImpl(k, shrt, lng, typ)
+    {
+        description = desc;
+
+        flags = fl;
+    }
+
+    wxString description;
+    int flags;
 };
 
 struct wxCmdLineParam
@@ -181,11 +184,13 @@ struct wxCmdLineParam
 
 WX_DECLARE_OBJARRAY(wxCmdLineOption, wxArrayOptions);
 WX_DECLARE_OBJARRAY(wxCmdLineParam, wxArrayParams);
+WX_DECLARE_OBJARRAY(wxCmdLineArgImpl, wxArrayArgs);
 
 #include "wx/arrimpl.cpp"
 
 WX_DEFINE_OBJARRAY(wxArrayOptions)
 WX_DEFINE_OBJARRAY(wxArrayParams)
+WX_DEFINE_OBJARRAY(wxArrayArgs)
 
 // the parser internal state
 struct wxCmdLineParserData
@@ -200,6 +205,7 @@ struct wxCmdLineParserData
     wxArrayOptions m_options;   // all possible options and switches
     wxArrayParams m_paramDesc;  // description of all possible params
     wxArrayString m_parameters; // all params found
+    wxArrayArgs m_parsedArguments; // all options and parameters in parsing order
 
     // methods
     wxCmdLineParserData();
@@ -212,12 +218,178 @@ struct wxCmdLineParserData
 
     int FindOption(const wxString& name);
     int FindOptionByLongName(const wxString& name);
+
+    // Find the option by either its short or long name.
+    //
+    // Asserts and returns NULL if option with this name is not found.
+    const wxCmdLineOption* FindOptionByAnyName(const wxString& name);
 };
 
 // ============================================================================
 // implementation
 // ============================================================================
 
+// ----------------------------------------------------------------------------
+// wxCmdLineArg
+// ----------------------------------------------------------------------------
+
+wxCmdLineArgImpl::wxCmdLineArgImpl(wxCmdLineEntryType k,
+                const wxString& shrt,
+                const wxString& lng,
+                wxCmdLineParamType typ)
+{
+    // wxCMD_LINE_USAGE_TEXT uses only description, shortName and longName is empty
+    if ( k != wxCMD_LINE_USAGE_TEXT && k != wxCMD_LINE_PARAM)
+    {
+        wxASSERT_MSG
+        (
+            !shrt.empty() || !lng.empty(),
+                wxT("option should have at least one name")
+        );
+
+        wxASSERT_MSG
+        (
+            GetShortOptionName(shrt.begin(), shrt.end()).Len() == shrt.Len(),
+            wxT("Short option contains invalid characters")
+        );
+
+        wxASSERT_MSG
+        (
+            GetLongOptionName(lng.begin(), lng.end()).Len() == lng.Len(),
+            wxT("Long option contains invalid characters")
+        );
+    }
+
+    kind = k;
+
+    shortName = shrt;
+    longName = lng;
+
+    type = typ;
+
+    Reset();
+}
+
+void wxCmdLineArgImpl::Check(wxCmdLineParamType WXUNUSED_UNLESS_DEBUG(typ)) const
+{
+    // NB: Type is always wxCMD_LINE_VAL_NONE for booleans, so mismatch between
+    //  switches / options / params is well checked by this test
+    // The parameters have type == wxCMD_LINE_VAL_STRING and thus can be
+    //  retrieved only by GetStrVal()
+    wxASSERT_MSG( type == typ, wxT("type mismatch in wxCmdLineArg") );
+}
+
+double wxCmdLineArgImpl::GetDoubleVal() const
+{
+    Check(wxCMD_LINE_VAL_DOUBLE);
+    return m_doubleVal;
+}
+
+long wxCmdLineArgImpl::GetLongVal() const
+{
+    Check(wxCMD_LINE_VAL_NUMBER);
+    return m_longVal;
+}
+
+const wxString& wxCmdLineArgImpl::GetStrVal() const
+{
+    Check(wxCMD_LINE_VAL_STRING);
+    return m_strVal;
+}
+
+#if wxUSE_DATETIME
+const wxDateTime& wxCmdLineArgImpl::GetDateVal() const
+{
+    Check(wxCMD_LINE_VAL_DATE);
+    return m_dateVal;
+}
+#endif // wxUSE_DATETIME
+
+wxCmdLineArgImpl& wxCmdLineArgImpl::SetDoubleVal(double val)
+{
+    Check(wxCMD_LINE_VAL_DOUBLE);
+    m_doubleVal = val;
+    m_hasVal = true;
+    return *this;
+}
+
+wxCmdLineArgImpl& wxCmdLineArgImpl::SetLongVal(long val)
+{
+    Check(wxCMD_LINE_VAL_NUMBER);
+    m_longVal = val;
+    m_hasVal = true;
+    return *this;
+}
+
+wxCmdLineArgImpl& wxCmdLineArgImpl::SetStrVal(const wxString& val)
+{
+    Check(wxCMD_LINE_VAL_STRING);
+    m_strVal = val;
+    m_hasVal = true;
+    return *this;
+}
+
+#if wxUSE_DATETIME
+wxCmdLineArgImpl& wxCmdLineArgImpl::SetDateVal(const wxDateTime& val)
+{
+    Check(wxCMD_LINE_VAL_DATE);
+    m_dateVal = val;
+    m_hasVal = true;
+    return *this;
+}
+#endif // wxUSE_DATETIME
+
+// ----------------------------------------------------------------------------
+// wxCmdLineArgsArrayRef
+// ----------------------------------------------------------------------------
+
+size_t wxCmdLineArgs::size() const
+{
+    return m_parser.m_data->m_parsedArguments.GetCount();
+}
+
+// ----------------------------------------------------------------------------
+// wxCmdLineArgsArrayRef::const_iterator
+// ----------------------------------------------------------------------------
+
+wxCmdLineArgs::const_iterator::reference
+    wxCmdLineArgs::const_iterator::operator *() const
+{
+    return m_parser->m_data->m_parsedArguments[m_index];
+}
+
+wxCmdLineArgs::const_iterator::pointer
+    wxCmdLineArgs::const_iterator::operator ->() const
+{
+    return &**this;
+}
+
+wxCmdLineArgs::const_iterator &wxCmdLineArgs::const_iterator::operator ++ ()
+{
+    ++m_index;
+    return *this;
+}
+
+wxCmdLineArgs::const_iterator wxCmdLineArgs::const_iterator::operator ++ (int)
+{
+    wxCmdLineArgs::const_iterator tmp(*this);
+    ++*this;
+    return tmp;
+}
+
+wxCmdLineArgs::const_iterator &wxCmdLineArgs::const_iterator::operator -- ()
+{
+    --m_index;
+    return *this;
+}
+
+wxCmdLineArgs::const_iterator wxCmdLineArgs::const_iterator::operator -- (int)
+{
+    wxCmdLineArgs::const_iterator tmp(*this);
+    --*this;
+    return tmp;
+}
+
 // ----------------------------------------------------------------------------
 // wxCmdLineParserData
 // ----------------------------------------------------------------------------
@@ -257,7 +429,8 @@ void wxCmdLineParserData::SetArguments(int argc, char **argv)
     // temporarily change the locale here. The only drawback is that changing
     // the locale is thread-unsafe but precisely because we're called so early
     // it's hopefully safe to assume that no other threads had been created yet.
-    char * const locOld = SetAllLocaleFacets("");
+    char * const locOld = SetAllLocaleFacets(NULL);
+    SetAllLocaleFacets("");
     wxON_BLOCK_EXIT1( SetAllLocaleFacets, locOld );
 
     for ( int n = 0; n < argc; n++ )
@@ -341,6 +514,24 @@ int wxCmdLineParserData::FindOptionByLongName(const wxString& name)
     return wxNOT_FOUND;
 }
 
+const wxCmdLineOption*
+wxCmdLineParserData::FindOptionByAnyName(const wxString& name)
+{
+    int i = FindOption(name);
+    if ( i == wxNOT_FOUND )
+    {
+        i = FindOptionByLongName(name);
+
+        if ( i == wxNOT_FOUND )
+        {
+            wxFAIL_MSG( wxS("Unknown option ") + name );
+            return NULL;
+        }
+    }
+
+    return &m_options[(size_t)i];
+}
+
 // ----------------------------------------------------------------------------
 // construction and destruction
 // ----------------------------------------------------------------------------
@@ -436,7 +627,7 @@ void wxCmdLineParser::SetDesc(const wxCmdLineEntryDesc *desc)
 
             default:
                 wxFAIL_MSG( wxT("unknown command line entry type") );
-                // still fall through
+                wxFALLTHROUGH;
 
             case wxCMD_LINE_NONE:
                 return;
@@ -519,72 +710,59 @@ void wxCmdLineParser::AddUsageText(const wxString& text)
 
 bool wxCmdLineParser::Found(const wxString& name) const
 {
-    int i = m_data->FindOption(name);
-    if ( i == wxNOT_FOUND )
-        i = m_data->FindOptionByLongName(name);
+    const wxCmdLineOption* const opt = m_data->FindOptionByAnyName(name);
 
-    wxCHECK_MSG( i != wxNOT_FOUND, false, wxT("unknown switch") );
+    return opt && opt->HasValue();
+}
 
-    wxCmdLineOption& opt = m_data->m_options[(size_t)i];
-    if ( !opt.HasValue() )
-        return false;
+wxCmdLineSwitchState wxCmdLineParser::FoundSwitch(const wxString& name) const
+{
+    const wxCmdLineOption* const opt = m_data->FindOptionByAnyName(name);
 
-    return true;
+    if ( !opt || !opt->HasValue() )
+        return wxCMD_SWITCH_NOT_FOUND;
+
+    return opt->IsNegated() ? wxCMD_SWITCH_OFF : wxCMD_SWITCH_ON;
 }
 
 bool wxCmdLineParser::Found(const wxString& name, wxString *value) const
 {
-    int i = m_data->FindOption(name);
-    if ( i == wxNOT_FOUND )
-        i = m_data->FindOptionByLongName(name);
+    const wxCmdLineOption* const opt = m_data->FindOptionByAnyName(name);
 
-    wxCHECK_MSG( i != wxNOT_FOUND, false, wxT("unknown option") );
-
-    wxCmdLineOption& opt = m_data->m_options[(size_t)i];
-    if ( !opt.HasValue() )
+    if ( !opt || !opt->HasValue() )
         return false;
 
     wxCHECK_MSG( value, false, wxT("NULL pointer in wxCmdLineOption::Found") );
 
-    *value = opt.GetStrVal();
+    *value = opt->GetStrVal();
 
     return true;
 }
 
 bool wxCmdLineParser::Found(const wxString& name, long *value) const
 {
-    int i = m_data->FindOption(name);
-    if ( i == wxNOT_FOUND )
-        i = m_data->FindOptionByLongName(name);
+    const wxCmdLineOption* const opt = m_data->FindOptionByAnyName(name);
 
-    wxCHECK_MSG( i != wxNOT_FOUND, false, wxT("unknown option") );
-
-    wxCmdLineOption& opt = m_data->m_options[(size_t)i];
-    if ( !opt.HasValue() )
+    if ( !opt || !opt->HasValue() )
         return false;
 
     wxCHECK_MSG( value, false, wxT("NULL pointer in wxCmdLineOption::Found") );
 
-    *value = opt.GetLongVal();
+    *value = opt->GetLongVal();
 
     return true;
 }
 
 bool wxCmdLineParser::Found(const wxString& name, double *value) const
 {
-    int i = m_data->FindOption(name);
-    if ( i == wxNOT_FOUND )
-        i = m_data->FindOptionByLongName(name);
+    const wxCmdLineOption* const opt = m_data->FindOptionByAnyName(name);
 
-    wxCHECK_MSG( i != wxNOT_FOUND, false, wxT("unknown option") );
-
-    wxCmdLineOption& opt = m_data->m_options[(size_t)i];
-    if ( !opt.HasValue() )
+    if ( !opt || !opt->HasValue() )
         return false;
 
     wxCHECK_MSG( value, false, wxT("NULL pointer in wxCmdLineOption::Found") );
 
-    *value = opt.GetDoubleVal();
+    *value = opt->GetDoubleVal();
 
     return true;
 }
@@ -592,19 +770,14 @@ bool wxCmdLineParser::Found(const wxString& name, double *value) const
 #if wxUSE_DATETIME
 bool wxCmdLineParser::Found(const wxString& name, wxDateTime *value) const
 {
-    int i = m_data->FindOption(name);
-    if ( i == wxNOT_FOUND )
-        i = m_data->FindOptionByLongName(name);
+    const wxCmdLineOption* const opt = m_data->FindOptionByAnyName(name);
 
-    wxCHECK_MSG( i != wxNOT_FOUND, false, wxT("unknown option") );
-
-    wxCmdLineOption& opt = m_data->m_options[(size_t)i];
-    if ( !opt.HasValue() )
+    if ( !opt || !opt->HasValue() )
         return false;
 
     wxCHECK_MSG( value, false, wxT("NULL pointer in wxCmdLineOption::Found") );
 
-    *value = opt.GetDateVal();
+    *value = opt->GetDateVal();
 
     return true;
 }
@@ -627,9 +800,10 @@ void wxCmdLineParser::Reset()
 {
     for ( size_t i = 0; i < m_data->m_options.GetCount(); i++ )
     {
-        wxCmdLineOption& opt = m_data->m_options[i];
-        opt.SetHasValue(false);
+        m_data->m_options[i].Reset();
     }
+
+    m_data->m_parsedArguments.Empty();
 }
 
 
@@ -652,6 +826,7 @@ int wxCmdLineParser::Parse(bool showUsage)
     Reset();
 
     // parse everything
+    m_data->m_parameters.clear();
     wxString arg;
     size_t count = m_data->m_arguments.size();
     for ( size_t n = 1; ok && (n < count); n++ )    // 0 is program name
@@ -667,7 +842,19 @@ int wxCmdLineParser::Parse(bool showUsage)
 
             continue;
         }
-
+#ifdef __WXOSX__
+        if ( arg == wxS("-ApplePersistenceIgnoreState") ||
+             arg == wxS("-AppleTextDirection") ||
+             arg == wxS("-AppleLocale") ||
+             arg == wxS("-AppleLanguages") )
+        {
+            maybeOption = false;
+            n++;
+            
+            continue;
+        }
+#endif
+        
         // empty argument or just '-' is not an option but a parameter
         if ( maybeOption && arg.length() > 1 &&
                 // FIXME-UTF8: use wc_str() after removing ANSI build
@@ -692,11 +879,45 @@ int wxCmdLineParser::Parse(bool showUsage)
 
                 if (longOptionsEnabled)
                 {
+                    wxString errorOpt;
+
                     optInd = m_data->FindOptionByLongName(name);
                     if ( optInd == wxNOT_FOUND )
                     {
-                        errorMsg << wxString::Format(_("Unknown long option '%s'"), name.c_str())
-                                 << wxT('\n');
+                        // Check if this could be a negatable long option.
+                        if ( name.Last() == '-' )
+                        {
+                            name.RemoveLast();
+
+                            optInd = m_data->FindOptionByLongName(name);
+                            if ( optInd != wxNOT_FOUND )
+                            {
+                                if ( !(m_data->m_options[optInd].flags &
+                                        wxCMD_LINE_SWITCH_NEGATABLE) )
+                                {
+                                    errorOpt.Printf
+                                             (
+                                              _("Option '%s' can't be negated"),
+                                              name
+                                             );
+                                    optInd = wxNOT_FOUND;
+                                }
+                            }
+                        }
+
+                        if ( optInd == wxNOT_FOUND )
+                        {
+                            if ( errorOpt.empty() )
+                            {
+                                errorOpt.Printf
+                                         (
+                                          _("Unknown long option '%s'"),
+                                          name
+                                         );
+                            }
+
+                            errorMsg << errorOpt << wxT('\n');
+                        }
                     }
                 }
                 else
@@ -755,6 +976,14 @@ int wxCmdLineParser::Parse(bool showUsage)
                     if ( m_data->m_options[(size_t)optInd].kind
                             == wxCMD_LINE_SWITCH )
                     {
+                        // if the switch is negatable and it is just followed
+                        // by '-' the '-' is considered to be part of this
+                        // switch
+                        if ( (m_data->m_options[(size_t)optInd].flags &
+                                    wxCMD_LINE_SWITCH_NEGATABLE) &&
+                                arg[len] == '-' )
+                            ++len;
+
                         // pretend that all the rest of the argument is the
                         // next argument, in fact
                         wxString arg2 = arg[0u];
@@ -792,7 +1021,10 @@ int wxCmdLineParser::Parse(bool showUsage)
             if ( opt.kind == wxCMD_LINE_SWITCH )
             {
                 // we must check that there is no value following the switch
-                if ( p != arg.end() )
+                bool negated = (opt.flags & wxCMD_LINE_SWITCH_NEGATABLE) &&
+                                    p != arg.end() && *p == '-';
+
+                if ( !negated && p != arg.end() )
                 {
                     errorMsg << wxString::Format(_("Unexpected characters following option '%s'."), name.c_str())
                              << wxT('\n');
@@ -802,6 +1034,8 @@ int wxCmdLineParser::Parse(bool showUsage)
                 {
                     // nothing more to do
                     opt.SetHasValue();
+                    if ( negated )
+                        opt.SetNegated();
 
                     if ( opt.flags & wxCMD_LINE_OPTION_HELP )
                     {
@@ -861,7 +1095,7 @@ int wxCmdLineParser::Parse(bool showUsage)
                     {
                         default:
                             wxFAIL_MSG( wxT("unknown option type") );
-                            // still fall through
+                            wxFALLTHROUGH;
 
                         case wxCMD_LINE_VAL_STRING:
                             opt.SetStrVal(value);
@@ -907,8 +1141,8 @@ int wxCmdLineParser::Parse(bool showUsage)
                         case wxCMD_LINE_VAL_DATE:
                             {
                                 wxDateTime dt;
-                                wxString::const_iterator end;
-                                if ( !dt.ParseDate(value, &end) || end != value.end() )
+                                wxString::const_iterator endDate;
+                                if ( !dt.ParseDate(value, &endDate) || endDate != value.end() )
                                 {
                                     errorMsg << wxString::Format(_("Option '%s': '%s' cannot be converted to a date."),
                                                                  name.c_str(), value.c_str())
@@ -926,6 +1160,9 @@ int wxCmdLineParser::Parse(bool showUsage)
                     }
                 }
             }
+
+            if (ok)
+                m_data->m_parsedArguments.push_back (opt);
         }
         else // not an option, must be a parameter
         {
@@ -936,6 +1173,9 @@ int wxCmdLineParser::Parse(bool showUsage)
                 // TODO check the param type
 
                 m_data->m_parameters.push_back(arg);
+                m_data->m_parsedArguments.push_back (
+                    wxCmdLineArgImpl(wxCMD_LINE_PARAM, wxString(), wxString(),
+                                     wxCMD_LINE_VAL_STRING).SetStrVal(arg));
 
                 if ( !(param.flags & wxCMD_LINE_PARAM_MULTIPLE) )
                 {
@@ -1095,7 +1335,7 @@ wxString wxCmdLineParser::GetUsageString() const
     for ( n = 0; n < count; n++ )
     {
         wxCmdLineOption& opt = m_data->m_options[n];
-        wxString option;
+        wxString option, negator;
 
         if ( opt.kind != wxCMD_LINE_USAGE_TEXT )
         {
@@ -1105,13 +1345,16 @@ wxString wxCmdLineParser::GetUsageString() const
                 usage << wxT('[');
             }
 
+            if ( opt.flags & wxCMD_LINE_SWITCH_NEGATABLE )
+                negator = wxT("[-]");
+
             if ( !opt.shortName.empty() )
             {
-                usage << chSwitch << opt.shortName;
+                usage << chSwitch << opt.shortName << negator;
             }
             else if ( areLongOptionsEnabled && !opt.longName.empty() )
             {
-                usage << wxT("--") << opt.longName;
+                usage << wxT("--") << opt.longName << negator;
             }
             else
             {
@@ -1233,7 +1476,7 @@ static wxString GetTypeName(wxCmdLineParamType type)
     {
         default:
             wxFAIL_MSG( wxT("unknown option type") );
-            // still fall through
+            wxFALLTHROUGH;
 
         case wxCMD_LINE_VAL_STRING:
             s = _("str");
@@ -1312,6 +1555,23 @@ static wxString GetLongOptionName(wxString::const_iterator p,
    Windows conventions for the command line handling, not Unix ones. For
    instance, backslash is not special except when it precedes double quote when
    it does quote it.
+
+   TODO: Rewrite this to follow the even more complicated rule used by Windows
+         CommandLineToArgv():
+
+    * A string of backslashes not followed by a quotation mark has no special
+      meaning.
+    * An even number of backslashes followed by a quotation mark is treated as
+      pairs of protected backslashes, followed by a word terminator.
+    * An odd number of backslashes followed by a quotation mark is treated as
+      pairs of protected backslashes, followed by a protected quotation mark.
+
+    See http://blogs.msdn.com/b/oldnewthing/archive/2010/09/17/10063629.aspx
+
+    It could also be useful to provide a converse function which is also
+    non-trivial, see
+
+    http://blogs.msdn.com/b/twistylittlepassagesallalike/archive/2011/04/23/everyone-quotes-arguments-the-wrong-way.aspx
  */
 
 /* static */
