@@ -18,7 +18,6 @@
 #if wxUSE_GRAPHICS_CONTEXT
 
 #include "wx/dcgraph.h"
-#include "wx/rawbmp.h"
 
 #ifndef WX_PRECOMP
     #include "wx/icon.h"
@@ -38,11 +37,6 @@ static const double RAD2DEG = 180.0 / M_PI;
 // Local functions
 //-----------------------------------------------------------------------------
 
-static inline double DegToRad(double deg)
-{
-    return (deg * M_PI) / 180.0;
-}
-
 static wxCompositionMode TranslateRasterOp(wxRasterOperationMode function)
 {
     switch ( function )
@@ -85,7 +79,7 @@ static wxCompositionMode TranslateRasterOp(wxRasterOperationMode function)
 // wxDC bridge class
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxGCDC, wxDC)
+wxIMPLEMENT_DYNAMIC_CLASS(wxGCDC, wxDC);
 
 wxGCDC::wxGCDC(const wxWindowDC& dc) :
   wxDC( new wxGCDCImpl( this, dc ) )
@@ -126,21 +120,7 @@ wxGCDC::~wxGCDC()
 {
 }
 
-wxGraphicsContext* wxGCDC::GetGraphicsContext() const
-{
-    if (!m_pimpl) return NULL;
-    wxGCDCImpl *gc_impl = (wxGCDCImpl*) m_pimpl;
-    return gc_impl->GetGraphicsContext();
-}
-
-void wxGCDC::SetGraphicsContext( wxGraphicsContext* ctx )
-{
-    if (!m_pimpl) return;
-    wxGCDCImpl *gc_impl = (wxGCDCImpl*) m_pimpl;
-    gc_impl->SetGraphicsContext( ctx );
-}
-
-IMPLEMENT_ABSTRACT_CLASS(wxGCDCImpl, wxDCImpl)
+wxIMPLEMENT_ABSTRACT_CLASS(wxGCDCImpl, wxDCImpl);
 
 wxGCDCImpl::wxGCDCImpl( wxDC *owner ) :
    wxDCImpl( owner )
@@ -174,61 +154,6 @@ wxGCDCImpl::wxGCDCImpl( wxDC *owner, const wxWindowDC& dc ) :
 wxGCDCImpl::wxGCDCImpl( wxDC *owner, const wxMemoryDC& dc ) :
    wxDCImpl( owner )
 {
-#if defined(__WXMSW__) && wxUSE_WXDIB
-    // It seems that GDI+ sets invalid values for alpha channel when used with
-    // a compatible bitmap (DDB). So we need to convert the currently selected
-    // bitmap to a DIB before using it with any GDI+ functions to ensure that
-    // we get the correct alpha channel values in it at the end.
-
-    wxBitmap bmp = dc.GetSelectedBitmap();
-    wxASSERT_MSG( bmp.IsOk(), "Should select a bitmap before creating wxGCDC" );
-
-    // We don't need to convert it if it can't have alpha at all (any depth but
-    // 32) or is already a DIB with alpha.
-    if ( bmp.GetDepth() == 32 && (!bmp.IsDIB() || !bmp.HasAlpha()) )
-    {
-        // We need to temporarily deselect this bitmap from the memory DC
-        // before modifying it.
-        const_cast<wxMemoryDC&>(dc).SelectObject(wxNullBitmap);
-
-        bmp.ConvertToDIB(); // Does nothing if already a DIB.
-
-        if( !bmp.HasAlpha() )
-        {
-            // Initialize alpha channel, even if we don't have any alpha yet,
-            // we should have correct (opaque) alpha values in it for GDI+
-            // functions to work correctly.
-            {
-                wxAlphaPixelData data(bmp);
-                if ( data )
-                {
-                    wxAlphaPixelData::Iterator p(data);
-                    for ( int y = 0; y < data.GetHeight(); y++ )
-                    {
-                        wxAlphaPixelData::Iterator rowStart = p;
-
-                        for ( int x = 0; x < data.GetWidth(); x++ )
-                        {
-                            p.Alpha() = wxALPHA_OPAQUE;
-                            ++p;
-                        }
-
-                        p = rowStart;
-                        p.OffsetY(data, 1);
-                    }
-                }
-            } // End of block modifying the bitmap.
-
-            // Using wxAlphaPixelData sets the internal "has alpha" flag but we
-            // don't really have any alpha yet, so reset it back for now.
-            bmp.ResetAlpha();
-        }
-
-        // Undo SelectObject() at the beginning of this block.
-        const_cast<wxMemoryDC&>(dc).SelectObjectAsSource(bmp);
-    }
-#endif // wxUSE_WXDIB
-
     Init(wxGraphicsContext::Create(dc));
 }
 
@@ -289,9 +214,9 @@ void wxGCDCImpl::DoDrawBitmap( const wxBitmap &bmp, wxCoord x, wxCoord y,
     if ( bmp.GetDepth() == 1 )
     {
         m_graphicContext->SetPen(*wxTRANSPARENT_PEN);
-        m_graphicContext->SetBrush( wxBrush( m_textBackgroundColour , wxSOLID ) );
+        m_graphicContext->SetBrush(m_textBackgroundColour);
         m_graphicContext->DrawRectangle( x, y, w, h );
-        m_graphicContext->SetBrush( wxBrush( m_textForegroundColour , wxSOLID ) );
+        m_graphicContext->SetBrush(m_textForegroundColour);
         m_graphicContext->DrawBitmap( bmp, x, y, w, h );
         m_graphicContext->SetBrush( m_graphicContext->CreateBrush(m_brush));
         m_graphicContext->SetPen( m_graphicContext->CreatePen(m_pen));
@@ -325,21 +250,24 @@ void wxGCDCImpl::DoDrawIcon( const wxIcon &icon, wxCoord x, wxCoord y )
     CalcBoundingBox(x + w, y + h);
 }
 
-bool wxGCDCImpl::StartDoc( const wxString& WXUNUSED(message) )
+bool wxGCDCImpl::StartDoc( const wxString& message )
 {
-    return true;
+    return m_graphicContext->StartDoc(message);
 }
 
 void wxGCDCImpl::EndDoc()
 {
+    m_graphicContext->EndDoc();
 }
 
 void wxGCDCImpl::StartPage()
 {
+    m_graphicContext->StartPage();
 }
 
 void wxGCDCImpl::EndPage()
 {
+    m_graphicContext->EndPage();
 }
 
 void wxGCDCImpl::Flush()
@@ -614,14 +542,14 @@ void wxGCDCImpl::DoDrawArc( wxCoord x1, wxCoord y1,
              -atan2(double(y2 - yc), double(x2 - xc)) * RAD2DEG;
     }
 
-    bool fill = m_brush.GetStyle() != wxTRANSPARENT;
+    bool fill = m_brush.GetStyle() != wxBRUSHSTYLE_TRANSPARENT;
 
     wxGraphicsPath path = m_graphicContext->CreatePath();
     if ( fill && ((x1!=x2)||(y1!=y2)) )
         path.MoveToPoint( xc, yc );
     // since these angles (ea,sa) are measured counter-clockwise, we invert them to
     // get clockwise angles
-    path.AddArc( xc, yc , rad , DegToRad(-sa) , DegToRad(-ea), false );
+    path.AddArc( xc, yc , rad, wxDegToRad(-sa), wxDegToRad(-ea), false );
     if ( fill && ((x1!=x2)||(y1!=y2)) )
         path.AddLineToPoint( xc, yc );
     m_graphicContext->DrawPath(path);
@@ -648,22 +576,27 @@ void wxGCDCImpl::DoDrawEllipticArc( wxCoord x, wxCoord y, wxCoord w, wxCoord h,
     m_graphicContext->Scale(factor, 1.0);
     wxGraphicsPath path = m_graphicContext->CreatePath();
 
+    // If end angle equals start angle we want draw a full ellipse.
+    if (ea == sa)
+    {
+        ea += 360.0;
+    }
     // since these angles (ea,sa) are measured counter-clockwise, we invert them to
     // get clockwise angles
-    if ( m_brush.GetStyle() != wxTRANSPARENT )
+    if ( m_brush.GetStyle() != wxBRUSHSTYLE_TRANSPARENT )
     {
         path.MoveToPoint( 0, 0 );
-        path.AddArc( 0, 0, h/2.0 , DegToRad(-sa) , DegToRad(-ea), sa > ea );
+        path.AddArc( 0, 0, h/2.0, wxDegToRad(-sa), wxDegToRad(-ea), false );
         path.AddLineToPoint( 0, 0 );
         m_graphicContext->FillPath( path );
 
         path = m_graphicContext->CreatePath();
-        path.AddArc( 0, 0, h/2.0 , DegToRad(-sa) , DegToRad(-ea), sa > ea );
+        path.AddArc( 0, 0, h/2.0, wxDegToRad(-sa), wxDegToRad(-ea), false );
         m_graphicContext->StrokePath( path );
     }
     else
     {
-        path.AddArc( 0, 0, h/2.0 , DegToRad(-sa) , DegToRad(-ea), sa > ea );
+        path.AddArc( 0, 0, h/2.0, wxDegToRad(-sa), wxDegToRad(-ea), false );
         m_graphicContext->DrawPath( path );
     }
 
@@ -792,7 +725,9 @@ void wxGCDCImpl::DoDrawPolygon( int n, const wxPoint points[],
 {
     wxCHECK_RET( IsOk(), wxT("wxGCDC(cg)::DoDrawPolygon - invalid DC") );
 
-    if ( n <= 0 || (m_brush.GetStyle() == wxTRANSPARENT && m_pen.GetStyle() == wxTRANSPARENT ) )
+    if ( n <= 0 ||
+            (m_brush.GetStyle() == wxBRUSHSTYLE_TRANSPARENT &&
+                m_pen.GetStyle() == wxPENSTYLE_TRANSPARENT) )
         return;
     if ( !m_logicalFunctionSupported )
         return;
@@ -1058,7 +993,7 @@ void wxGCDCImpl::DoDrawRotatedText(const wxString& text, wxCoord x, wxCoord y,
     GetOwner()->GetMultiLineTextExtent(text, &w, &h, &heightLine);
     
     // Compute the shift for the origin of the next line.
-    const double rad = DegToRad(angle);
+    const double rad = wxDegToRad(angle);
     const double dx = heightLine * sin(rad);
     const double dy = heightLine * cos(rad);
     
@@ -1069,9 +1004,9 @@ void wxGCDCImpl::DoDrawRotatedText(const wxString& text, wxCoord x, wxCoord y,
         // Calculate origin for each line to avoid accumulation of
         // rounding errors.
         if ( m_backgroundMode == wxTRANSPARENT )
-            m_graphicContext->DrawText( lines[lineNum], x + wxRound(lineNum*dx), y + wxRound(lineNum*dy), DegToRad(angle ));
+            m_graphicContext->DrawText( lines[lineNum], x + wxRound(lineNum*dx), y + wxRound(lineNum*dy), wxDegToRad(angle ));
         else
-            m_graphicContext->DrawText( lines[lineNum], x + wxRound(lineNum*dx), y + wxRound(lineNum*dy), DegToRad(angle ), m_graphicContext->CreateBrush(m_textBackgroundColour) );
+            m_graphicContext->DrawText( lines[lineNum], x + wxRound(lineNum*dx), y + wxRound(lineNum*dy), wxDegToRad(angle ), m_graphicContext->CreateBrush(m_textBackgroundColour) );
    }
             
     // call the bounding box by adding all four vertices of the rectangle
@@ -1114,7 +1049,7 @@ void wxGCDCImpl::DoDrawText(const wxString& str, wxCoord x, wxCoord y)
     if ( m_backgroundMode == wxTRANSPARENT )
         m_graphicContext->DrawText( str, x ,y);
     else
-        m_graphicContext->DrawText( str, x ,y , m_graphicContext->CreateBrush( wxBrush(m_textBackgroundColour,wxSOLID) ) );
+        m_graphicContext->DrawText( str, x ,y , m_graphicContext->CreateBrush(m_textBackgroundColour) );
 
     wxCoord w, h;
     GetOwner()->GetTextExtent(str, &w, &h);
@@ -1178,7 +1113,7 @@ bool wxGCDCImpl::DoGetPartialTextExtents(const wxString& text, wxArrayInt& width
 
 wxCoord wxGCDCImpl::GetCharWidth(void) const
 {
-    wxCoord width;
+    wxCoord width = 0;
     DoGetTextExtent( wxT("g") , &width , NULL , NULL , NULL , NULL );
 
     return width;
@@ -1186,7 +1121,7 @@ wxCoord wxGCDCImpl::GetCharWidth(void) const
 
 wxCoord wxGCDCImpl::GetCharHeight(void) const
 {
-    wxCoord height;
+    wxCoord height = 0;
     DoGetTextExtent( wxT("g") , NULL , &height , NULL , NULL , NULL );
 
     return height;
@@ -1202,10 +1137,9 @@ void wxGCDCImpl::Clear(void)
     wxCompositionMode formerMode = m_graphicContext->GetCompositionMode();
     m_graphicContext->SetCompositionMode(wxCOMPOSITION_SOURCE);
     // maximum positive coordinate Cairo can handle is 2^23 - 1
-    // Use a value slightly less than this to be sure we avoid the limit
     DoDrawRectangle(
         DeviceToLogicalX(0), DeviceToLogicalY(0),
-        DeviceToLogicalXRel(0x800000 - 64), DeviceToLogicalYRel(0x800000 - 64));
+        DeviceToLogicalXRel(0x007fffff), DeviceToLogicalYRel(0x007fffff));
     m_graphicContext->SetCompositionMode(formerMode);
     m_graphicContext->SetPen( m_pen );
     m_graphicContext->SetBrush( m_brush );
