@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     29/01/98
-// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -38,6 +37,7 @@
 #include "wx/filename.h"
 #include "wx/dir.h"
 
+#include "wx/scopedptr.h"
 #include "wx/tokenzr.h"
 
 // there are just too many of those...
@@ -49,11 +49,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !wxONLY_WATCOM_EARLIER_THAN(1,4)
-    #if !(defined(_MSC_VER) && (_MSC_VER > 800))
-        #include <errno.h>
-    #endif
-#endif
+#include <errno.h>
 
 #if defined(__WXMAC__)
     #include  "wx/osx/private.h"  // includes mac headers
@@ -61,20 +57,22 @@
 
 #ifdef __WINDOWS__
     #include "wx/msw/private.h"
-    #include "wx/msw/mslu.h"
+    #include "wx/msw/missing.h"
 
     // sys/cygwin.h is needed for cygwin_conv_to_full_win32_path()
+    // and for cygwin_conv_path()
     //
     // note that it must be included after <windows.h>
     #ifdef __GNUWIN32__
         #ifdef __CYGWIN__
             #include <sys/cygwin.h>
+            #include <cygwin/version.h>
         #endif
     #endif // __GNUWIN32__
 
     // io.h is needed for _get_osfhandle()
     // Already included by filefn.h for many Windows compilers
-    #if defined __MWERKS__ || defined __CYGWIN__
+    #if defined __CYGWIN__
         #include <io.h>
     #endif
 #endif // __WINDOWS__
@@ -88,6 +86,8 @@
     #define HAVE_WGETCWD
 #endif
 
+wxDECL_FOR_STRICT_MINGW32(int, _fileno, (FILE*))
+
 // ----------------------------------------------------------------------------
 // constants
 // ----------------------------------------------------------------------------
@@ -96,10 +96,6 @@
     #define _MAXPATHLEN 1024
 #endif
 
-#ifndef INVALID_FILE_ATTRIBUTES
-    #define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
-#endif
-
 // ----------------------------------------------------------------------------
 // private globals
 // ----------------------------------------------------------------------------
@@ -108,14 +104,6 @@
 static wxChar wxFileFunctionsBuffer[4*_MAXPATHLEN];
 #endif
 
-#if defined(__VISAGECPP__) && __IBMCPP__ >= 400
-//
-// VisualAge C++ V4.0 cannot have any external linkage const decs
-// in headers included by more than one primary source
-//
-const int wxInvalidOffset = -1;
-#endif
-
 // ============================================================================
 // implementation
 // ============================================================================
@@ -194,17 +182,14 @@ void wxPathList::Add(const wxArrayString &arr)
 }
 
 // Add paths e.g. from the PATH environment variable
-void wxPathList::AddEnvList (const wxString& WXUNUSED_IN_WINCE(envVariable))
+void wxPathList::AddEnvList (const wxString& envVariable)
 {
-    // No environment variables on WinCE
-#ifndef __WXWINCE__
-
     // The space has been removed from the tokenizers, otherwise a
     // path such as "C:\Program Files" would be split into 2 paths:
     // "C:\Program" and "Files"; this is true for both Windows and Unix.
 
     static const wxChar PATH_TOKS[] =
-#if defined(__WINDOWS__) || defined(__OS2__)
+#if defined(__WINDOWS__)
         wxT(";"); // Don't separate with colon in DOS (used for drive)
 #else
         wxT(":;");
@@ -217,7 +202,6 @@ void wxPathList::AddEnvList (const wxString& WXUNUSED_IN_WINCE(envVariable))
         wxArrayString arr = wxStringTokenize(val, PATH_TOKS);
         WX_APPEND_ARRAY(*this, arr);
     }
-#endif // !__WXWINCE__
 }
 
 // Given a full filename (with path), ensure that that file can
@@ -228,13 +212,6 @@ bool wxPathList::EnsureFileAccessible (const wxString& path)
     return Add(wxPathOnly(path));
 }
 
-#if WXWIN_COMPATIBILITY_2_6
-bool wxPathList::Member (const wxString& path) const
-{
-    return Index(path) != wxNOT_FOUND;
-}
-#endif
-
 wxString wxPathList::FindValidPath (const wxString& file) const
 {
     // normalize the given string as it could be a path + a filename
@@ -324,7 +301,7 @@ wxIsAbsolutePath (const wxString& filename)
         if ((filename[0] == wxT('[') && filename[1] != wxT('.')))
             return true;
 #endif
-#if defined(__WINDOWS__) || defined(__OS2__)
+#if defined(__WINDOWS__)
         // MSDOS like
         if (filename[0] == wxT('\\') || (wxIsalpha (filename[0]) && filename[1] == wxT(':')))
             return true;
@@ -369,7 +346,7 @@ template<typename CharType>
 static CharType *wxDoRealPath (CharType *path)
 {
   static const CharType SEP = wxFILE_SEP_PATH;
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
   wxUnix2DosFilename(path);
 #endif
   if (path[0] && path[1]) {
@@ -400,7 +377,7 @@ static CharType *wxDoRealPath (CharType *path)
                         path[0] = SEP;
                         path[1] = wxT('\0');
                       }
-#if defined(__WXMSW__) || defined(__OS2__)
+#if defined(__WINDOWS__)
                     /* Check that path[2] is NULL! */
                     else if (path[1] == wxT(':') && !path[2])
                       {
@@ -449,7 +426,7 @@ wxChar *wxCopyAbsolutePath(const wxString& filename)
     {
         wxString buf = ::wxGetCwd();
         wxChar ch = buf.Last();
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
         if (ch != wxT('\\') && ch != wxT('/'))
             buf << wxT("\\");
 #else
@@ -489,7 +466,7 @@ wxChar *wxCopyAbsolutePath(const wxString& filename)
 template<typename CharType>
 static CharType *wxDoExpandPath(CharType *buf, const wxString& name)
 {
-    register CharType *d, *s, *nm;
+    CharType *d, *s, *nm;
     CharType        lnm[_MAXPATHLEN];
     int             q;
 
@@ -503,7 +480,7 @@ static CharType *wxDoExpandPath(CharType *buf, const wxString& name)
     trimchars[3] = 0;
 
     static const CharType SEP = wxFILE_SEP_PATH;
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
     //wxUnix2DosFilename(path);
 #endif
 
@@ -523,32 +500,15 @@ static CharType *wxDoExpandPath(CharType *buf, const wxString& name)
 
     s = nm;
     d = lnm;
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
     q = FALSE;
 #else
     q = nm[0] == wxT('\\') && nm[1] == wxT('~');
 #endif
 
     /* Expand inline environment variables */
-#ifdef __VISAGECPP__
-    while (*d)
-    {
-      *d++ = *s;
-      if(*s == wxT('\\'))
-      {
-        *(d - 1) = *++s;
-        if (*d)
-        {
-          s++;
-          continue;
-        }
-        else
-           break;
-      }
-      else
-#else
     while ((*d++ = *s) != 0) {
-#  ifndef __WXMSW__
+#  ifndef __WINDOWS__
         if (*s == wxT('\\')) {
             if ((*(d - 1) = *++s)!=0) {
                 s++;
@@ -557,18 +517,15 @@ static CharType *wxDoExpandPath(CharType *buf, const wxString& name)
                 break;
         } else
 #  endif
-#endif
-            // No env variables on WinCE
-#ifndef __WXWINCE__
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
         if (*s++ == wxT('$') && (*s == wxT('{') || *s == wxT(')')))
 #else
         if (*s++ == wxT('$'))
 #endif
         {
-            register CharType  *start = d;
-            register int     braces = (*s == wxT('{') || *s == wxT('('));
-            register CharType  *value;
+            CharType  *start = d;
+            int     braces = (*s == wxT('{') || *s == wxT('('));
+            CharType  *value;
             while ((*d++ = *s) != 0)
                 if (braces ? (*s == wxT('}') || *s == wxT(')')) : !(wxIsalnum(*s) || *s == wxT('_')) )
                     break;
@@ -587,8 +544,6 @@ static CharType *wxDoExpandPath(CharType *buf, const wxString& name)
                     s++;
             }
         }
-#endif
-        // __WXWINCE__
     }
 
     /* Expand ~ and ~user */
@@ -607,7 +562,7 @@ static CharType *wxDoExpandPath(CharType *buf, const wxString& name)
             }
         } else
         {                /* ~user/filename */
-            register CharType  *nnm;
+            CharType  *nnm;
             for (s = nm; *s && *s != SEP; s++)
             {
                 // Empty
@@ -670,7 +625,7 @@ wchar_t *wxExpandPath(wchar_t *buf, const wxString& name)
  */
 wxChar *
 wxContractPath (const wxString& filename,
-                const wxString& WXUNUSED_IN_WINCE(envname),
+                const wxString& envname,
                 const wxString& user)
 {
   static wxChar dest[_MAXPATHLEN];
@@ -679,13 +634,12 @@ wxContractPath (const wxString& filename,
     return NULL;
 
   wxStrcpy (dest, filename);
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
   wxUnix2DosFilename(dest);
 #endif
 
   // Handle environment
   wxString val;
-#ifndef __WXWINCE__
   wxChar *tcp;
   if (!envname.empty() && !(val = wxGetenv (envname)).empty() &&
      (tcp = wxStrstr (dest, val)) != NULL)
@@ -697,7 +651,6 @@ wxContractPath (const wxString& filename,
         wxStrcat (tcp, wxT("}"));
         wxStrcat (tcp, wxFileFunctionsBuffer);
     }
-#endif
 
   // Handle User's home (ignore root homes!)
   val = wxGetUserHome (user);
@@ -733,16 +686,7 @@ wxChar *wxFileNameFromPath (wxChar *path)
 
 wxString wxFileNameFromPath (const wxString& path)
 {
-    wxString name, ext;
-    wxFileName::SplitPath(path, NULL, &name, &ext);
-
-    wxString fullname = name;
-    if ( !ext.empty() )
-    {
-        fullname << wxFILE_SEP_EXT << ext;
-    }
-
-    return fullname;
+    return wxFileName(path).GetFullName();
 }
 
 // Return just the directory, or NULL if no directory
@@ -753,11 +697,13 @@ wxPathOnly (wxChar *path)
     {
         static wxChar buf[_MAXPATHLEN];
 
-        // Local copy
-        wxStrcpy (buf, path);
-
         int l = wxStrlen(path);
         int i = l - 1;
+        if ( i >= _MAXPATHLEN )
+            return NULL;
+
+        // Local copy
+        wxStrcpy (buf, path);
 
         // Search backward for a backward or forward slash
         while (i > -1)
@@ -778,7 +724,7 @@ wxPathOnly (wxChar *path)
             i --;
         }
 
-#if defined(__WXMSW__) || defined(__OS2__)
+#if defined(__WINDOWS__)
         // Try Drive specifier
         if (wxIsalpha (buf[0]) && buf[1] == wxT(':'))
         {
@@ -799,12 +745,15 @@ wxString wxPathOnly (const wxString& path)
     {
         wxChar buf[_MAXPATHLEN];
 
-        // Local copy
-        wxStrcpy(buf, path);
-
         int l = path.length();
         int i = l - 1;
 
+        if ( i >= _MAXPATHLEN )
+            return wxString();
+
+        // Local copy
+        wxStrcpy(buf, path);
+
         // Search backward for a backward or forward slash
         while (i > -1)
         {
@@ -827,7 +776,7 @@ wxString wxPathOnly (const wxString& path)
             i --;
         }
 
-#if defined(__WXMSW__) || defined(__OS2__)
+#if defined(__WINDOWS__)
         // Try Drive specifier
         if (wxIsalpha (buf[0]) && buf[1] == wxT(':'))
         {
@@ -853,6 +802,9 @@ wxString wxMacFSRefToPath( const FSRef *fsRef , CFStringRef additionalPathCompon
 {
     CFURLRef fullURLRef;
     fullURLRef = CFURLCreateFromFSRef(NULL, fsRef);
+    if ( fullURLRef == NULL)
+        return wxEmptyString;
+    
     if ( additionalPathComponent )
     {
         CFURLRef parentURLRef = fullURLRef ;
@@ -860,12 +812,10 @@ wxString wxMacFSRefToPath( const FSRef *fsRef , CFStringRef additionalPathCompon
             additionalPathComponent,false);
         CFRelease( parentURLRef ) ;
     }
-    CFStringRef cfString = CFURLCopyFileSystemPath(fullURLRef, kDefaultPathStyle);
+    wxCFStringRef cfString( CFURLCopyFileSystemPath(fullURLRef, kDefaultPathStyle ));
     CFRelease( fullURLRef ) ;
-    CFMutableStringRef cfMutableString = CFStringCreateMutableCopy(NULL, 0, cfString);
-    CFRelease( cfString );
-    CFStringNormalize(cfMutableString,kCFStringNormalizationFormC);
-    return wxCFStringRef(cfMutableString).AsString();
+
+    return wxCFStringRef::AsStringWithNormalizationFormC(cfString);
 }
 
 OSStatus wxMacPathToFSRef( const wxString&path , FSRef *fsRef )
@@ -890,13 +840,10 @@ OSStatus wxMacPathToFSRef( const wxString&path , FSRef *fsRef )
 
 wxString wxMacHFSUniStrToString( ConstHFSUniStr255Param uniname )
 {
-    CFStringRef cfname = CFStringCreateWithCharacters( kCFAllocatorDefault,
+    wxCFStringRef cfname( CFStringCreateWithCharacters( kCFAllocatorDefault,
                                                       uniname->unicode,
-                                                      uniname->length );
-    CFMutableStringRef cfMutableString = CFStringCreateMutableCopy(NULL, 0, cfname);
-    CFRelease( cfname );
-    CFStringNormalize(cfMutableString,kCFStringNormalizationFormC);
-    return wxCFStringRef(cfMutableString).AsString() ;
+                                                      uniname->length ) );
+    return wxCFStringRef::AsStringWithNormalizationFormC(cfname);
 }
 
 #ifndef __LP64__
@@ -934,7 +881,7 @@ static void wxDoDos2UnixFilename(T *s)
       {
         if (*s == wxT('\\'))
           *s = wxT('/');
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
         else
           *s = wxTolower(*s);        // Case INDEPENDENT
 #endif
@@ -947,14 +894,14 @@ void wxDos2UnixFilename(wchar_t *s) { wxDoDos2UnixFilename(s); }
 
 template<typename T>
 static void
-#if defined(__WXMSW__) || defined(__OS2__)
+#if defined(__WINDOWS__)
 wxDoUnix2DosFilename(T *s)
 #else
 wxDoUnix2DosFilename(T *WXUNUSED(s) )
 #endif
 {
 // Yes, I really mean this to happen under DOS only! JACS
-#if defined(__WXMSW__) || defined(__OS2__)
+#if defined(__WINDOWS__)
   if (s)
     while (*s)
       {
@@ -1009,8 +956,7 @@ wxConcatFiles (const wxString& file1, const wxString& file2, const wxString& fil
 }
 
 // helper of generic implementation of wxCopyFile()
-#if !(defined(__WIN32__) || defined(__OS2__) || defined(__PALMOS__)) && \
-    wxUSE_FILE
+#if !defined(__WIN32__) && wxUSE_FILE
 
 static bool
 wxDoCopyFile(wxFile& fileIn,
@@ -1057,24 +1003,18 @@ wxDoCopyFile(wxFile& fileIn,
 bool
 wxCopyFile (const wxString& file1, const wxString& file2, bool overwrite)
 {
-#if defined(__WIN32__) && !defined(__WXMICROWIN__)
+#if defined(__WIN32__)
     // CopyFile() copies file attributes and modification time too, so use it
     // instead of our code if available
     //
     // NB: 3rd parameter is bFailIfExists i.e. the inverse of overwrite
-    if ( !::CopyFile(file1.fn_str(), file2.fn_str(), !overwrite) )
+    if ( !::CopyFile(file1.t_str(), file2.t_str(), !overwrite) )
     {
         wxLogSysError(_("Failed to copy the file '%s' to '%s'"),
                       file1.c_str(), file2.c_str());
 
         return false;
     }
-#elif defined(__OS2__)
-    if ( ::DosCopy(file1.c_str(), file2.c_str(), overwrite ? DCPY_EXISTING : 0) != 0 )
-        return false;
-#elif defined(__PALMOS__)
-    // TODO with http://www.palmos.com/dev/support/docs/protein_books/Memory_Databases_Files/
-    return false;
 #elif wxUSE_FILE // !Win32
 
     wxStructStat fbuf;
@@ -1104,7 +1044,7 @@ wxCopyFile (const wxString& file1, const wxString& file2, bool overwrite)
 
     wxDoCopyFile(fileIn, fbuf, file2, overwrite);
 
-#if defined(__WXMAC__) || defined(__WXCOCOA__)
+#if defined(__WXMAC__)
     // copy the resource fork of the file too if it's present
     wxString pathRsrcOut;
     wxFile fileRsrcIn;
@@ -1144,18 +1084,14 @@ wxCopyFile (const wxString& file1, const wxString& file2, bool overwrite)
         if ( !wxDoCopyFile(fileRsrcIn, fbuf, pathRsrcOut, overwrite) )
             return false;
     }
-#endif // wxMac || wxCocoa
+#endif // wxMac
 
-#if !defined(__VISAGECPP__) && !defined(__WXMAC__) || defined(__UNIX__)
-    // no chmod in VA.  Should be some permission API for HPFS386 partitions
-    // however
     if ( chmod(file2.fn_str(), fbuf.st_mode) != 0 )
     {
         wxLogSysError(_("Impossible to set permissions for the file '%s'"),
                       file2.c_str());
         return false;
     }
-#endif // OS/2 || Mac
 
 #else // !Win32 && ! wxUSE_FILE
 
@@ -1165,7 +1101,7 @@ wxCopyFile (const wxString& file1, const wxString& file2, bool overwrite)
     wxUnusedVar(overwrite);
     return false;
 
-#endif // __WXMSW__ && __WIN32__
+#endif // __WINDOWS__ && __WIN32__
 
     return true;
 }
@@ -1184,11 +1120,9 @@ wxRenameFile(const wxString& file1, const wxString& file2, bool overwrite)
         return false;
     }
 
-#if !defined(__WXWINCE__) && !defined(__WXPALMOS__)
     // Normal system call
   if ( wxRename (file1, file2) == 0 )
     return true;
-#endif
 
   // Try to copy
   if (wxCopyFile(file1, file2, overwrite)) {
@@ -1196,6 +1130,7 @@ wxRenameFile(const wxString& file1, const wxString& file2, bool overwrite)
     return true;
   }
   // Give up
+  wxLogSysError(_("File '%s' couldn't be renamed '%s'"), file1, file2);
   return false;
 }
 
@@ -1203,36 +1138,30 @@ bool wxRemoveFile(const wxString& file)
 {
 #if defined(__VISUALC__) \
  || defined(__BORLANDC__) \
- || defined(__WATCOMC__) \
- || defined(__DMC__) \
- || defined(__GNUWIN32__) \
- || (defined(__MWERKS__) && defined(__MSL__))
+ || defined(__GNUWIN32__)
     int res = wxRemove(file);
 #elif defined(__WXMAC__)
     int res = unlink(file.fn_str());
-#elif defined(__WXPALMOS__)
-    int res = 1;
-    // TODO with VFSFileDelete()
 #else
     int res = unlink(file.fn_str());
 #endif
-
+    if ( res )
+    {
+        wxLogSysError(_("File '%s' couldn't be removed"), file);
+    }
     return res == 0;
 }
 
 bool wxMkdir(const wxString& dir, int perm)
 {
-#if defined(__WXPALMOS__)
-    return false;
-#else
 #if defined(__WXMAC__) && !defined(__UNIX__)
     if ( mkdir(dir.fn_str(), 0) != 0 )
 
     // assume mkdir() has 2 args on non Windows-OS/2 platforms and on Windows too
     // for the GNU compiler
-#elif (!(defined(__WXMSW__) || defined(__OS2__) || defined(__DOS__))) || \
+#elif (!defined(__WINDOWS__)) || \
       (defined(__GNUWIN32__) && !defined(__MINGW32__)) ||                \
-      defined(__WINE__) || defined(__WXMICROWIN__)
+      defined(__WINE__)
     const wxChar *dirname = dir.c_str();
   #if defined(MSVCRT)
     wxUnusedVar(perm);
@@ -1240,26 +1169,9 @@ bool wxMkdir(const wxString& dir, int perm)
   #else
     if ( mkdir(wxFNCONV(dirname), perm) != 0 )
   #endif
-#elif defined(__OS2__)
+#else  // !MSW and !OS/2 VAC++
     wxUnusedVar(perm);
-    if (::DosCreateDir(dir.c_str(), NULL) != 0) // enhance for EAB's??
-#elif defined(__DOS__)
-    const wxChar *dirname = dir.c_str();
-  #if defined(__WATCOMC__)
-    (void)perm;
-    if ( wxMkDir(wxFNSTRINGCAST wxFNCONV(dirname)) != 0 )
-  #elif defined(__DJGPP__)
-    if ( mkdir(wxFNCONV(dirname), perm) != 0 )
-  #else
-    #error "Unsupported DOS compiler!"
-  #endif
-#else  // !MSW, !DOS and !OS/2 VAC++
-    wxUnusedVar(perm);
-  #ifdef __WXWINCE__
-    if ( CreateDirectory(dir.fn_str(), NULL) == 0 )
-  #else
     if ( wxMkDir(dir.fn_str()) != 0 )
-  #endif
 #endif // !MSW/MSW
     {
         wxLogSysError(_("Directory '%s' couldn't be created"), dir);
@@ -1267,31 +1179,21 @@ bool wxMkdir(const wxString& dir, int perm)
     }
 
     return true;
-#endif // PALMOS/!PALMOS
 }
 
 bool wxRmdir(const wxString& dir, int WXUNUSED(flags))
 {
 #if defined(__VMS__)
     return false; //to be changed since rmdir exists in VMS7.x
-#elif defined(__WXPALMOS__)
-    // TODO with VFSFileRename()
-    return false;
 #else
-  #if defined(__OS2__)
-    if ( ::DosDeleteDir(dir.c_str()) != 0 )
-  #elif defined(__WXWINCE__)
-    if ( RemoveDirectory(dir.fn_str()) == 0 )
-  #else
     if ( wxRmDir(dir.fn_str()) != 0 )
-  #endif
     {
         wxLogSysError(_("Directory '%s' couldn't be deleted"), dir);
         return false;
     }
 
     return true;
-#endif // PALMOS/!PALMOS
+#endif
 }
 
 // does the path exists? (may have or not '/' or '\\' at the end)
@@ -1310,12 +1212,7 @@ wxChar *wxGetTempFileName(const wxString& prefix, wxChar *buf)
         return NULL;
 
     if ( buf )
-#ifdef _PACC_VER
-        // work around the PalmOS pacc compiler bug
-        wxStrcpy(buf, filename.data());
-#else
         wxStrcpy(buf, filename);
-#endif
     else
         buf = MYcopystring(filename);
 
@@ -1340,7 +1237,7 @@ bool wxGetTempFileName(const wxString& prefix, wxString& buf)
 
 // Get first file name matching given wild card.
 
-static wxDir *gs_dir = NULL;
+static wxScopedPtr<wxDir> gs_dir;
 static wxString gs_dirPath;
 
 wxString wxFindFirstFile(const wxString& spec, int flags)
@@ -1351,12 +1248,11 @@ wxString wxFindFirstFile(const wxString& spec, int flags)
     if ( !wxEndsWithPathSeparator(gs_dirPath ) )
         gs_dirPath << wxFILE_SEP_PATH;
 
-    delete gs_dir; // can be NULL, this is ok
-    gs_dir = new wxDir(gs_dirPath);
+    gs_dir.reset(new wxDir(gs_dirPath));
 
     if ( !gs_dir->IsOpened() )
     {
-        wxLogSysError(_("Can not enumerate files '%s'"), spec);
+        wxLogSysError(_("Cannot enumerate files '%s'"), spec);
         return wxEmptyString;
     }
 
@@ -1371,10 +1267,7 @@ wxString wxFindFirstFile(const wxString& spec, int flags)
     wxString result;
     gs_dir->GetFirst(&result, wxFileNameFromPath(spec), dirFlags);
     if ( result.empty() )
-    {
-        wxDELETE(gs_dir);
         return result;
-    }
 
     return gs_dirPath + result;
 }
@@ -1384,13 +1277,8 @@ wxString wxFindNextFile()
     wxCHECK_MSG( gs_dir, "", "You must call wxFindFirstFile before!" );
 
     wxString result;
-    gs_dir->GetNext(&result);
-
-    if ( result.empty() )
-    {
-        wxDELETE(gs_dir);
+    if ( !gs_dir->GetNext(&result) || result.empty() )
         return result;
-    }
 
     return gs_dirPath + result;
 }
@@ -1405,21 +1293,12 @@ wxString wxFindNextFile()
 
 wxChar *wxDoGetCwd(wxChar *buf, int sz)
 {
-#if defined(__WXPALMOS__)
-    // TODO
-    if(buf && sz>0) buf[0] = wxT('\0');
-    return buf;
-#elif defined(__WXWINCE__)
-    // TODO
-    if(buf && sz>0) buf[0] = wxT('\0');
-    return buf;
-#else
     if ( !buf )
     {
         buf = new wxChar[sz + 1];
     }
 
-    bool ok wxDUMMY_INITIALIZE(false);
+    bool ok = false;
 
     // for the compilers which have Unicode version of _getcwd(), call it
     // directly, for the others call the ANSI version and do the translation
@@ -1428,16 +1307,12 @@ wxChar *wxDoGetCwd(wxChar *buf, int sz)
 #else // wxUSE_UNICODE
     bool needsANSI = true;
 
-    #if !defined(HAVE_WGETCWD) || wxUSE_UNICODE_MSLU
+    #if !defined(HAVE_WGETCWD)
         char cbuf[_MAXPATHLEN];
     #endif
 
     #ifdef HAVE_WGETCWD
-        #if wxUSE_UNICODE_MSLU
-            if ( wxGetOsVersion() != wxOS_WINDOWS_9X )
-        #else
             char *cbuf = NULL; // never really used because needsANSI will always be false
-        #endif
             {
                 ok = _wgetcwd(buf, sz) != NULL;
                 needsANSI = false;
@@ -1449,25 +1324,7 @@ wxChar *wxDoGetCwd(wxChar *buf, int sz)
     {
     #if defined(_MSC_VER) || defined(__MINGW32__)
         ok = _getcwd(cbuf, sz) != NULL;
-    #elif defined(__OS2__)
-        APIRET rc;
-        ULONG ulDriveNum = 0;
-        ULONG ulDriveMap = 0;
-        rc = ::DosQueryCurrentDisk(&ulDriveNum, &ulDriveMap);
-        ok = rc == 0;
-        if (ok)
-        {
-            sz -= 3;
-            rc = ::DosQueryCurrentDir( 0 // current drive
-                                      ,(PBYTE)cbuf + 3
-                                      ,(PULONG)&sz
-                                     );
-            cbuf[0] = char('A' + (ulDriveNum - 1));
-            cbuf[1] = ':';
-            cbuf[2] = '\\';
-            ok = rc == 0;
-        }
-    #else // !Win32/VC++ !Mac !OS2
+    #else // !Win32/VC++ !Mac
         ok = getcwd(cbuf, sz) != NULL;
     #endif // platform
 
@@ -1489,27 +1346,25 @@ wxChar *wxDoGetCwd(wxChar *buf, int sz)
     }
     else // ok, but we might need to massage the path into the right format
     {
-#ifdef __DJGPP__
-        // VS: DJGPP is a strange mix of DOS and UNIX API and returns paths
-        //     with / deliminers. We don't like that.
-        for (wxChar *ch = buf; *ch; ch++)
-        {
-            if (*ch == wxT('/'))
-                *ch = wxT('\\');
-        }
-#endif // __DJGPP__
-
 // MBN: we hope that in the case the user is compiling a GTK+/Motif app,
 //      he needs Unix as opposed to Win32 pathnames
 #if defined( __CYGWIN__ ) && defined( __WINDOWS__ )
         // another example of DOS/Unix mix (Cygwin)
         wxString pathUnix = buf;
 #if wxUSE_UNICODE
+    #if CYGWIN_VERSION_DLL_MAJOR >= 1007
+        cygwin_conv_path(CCP_POSIX_TO_WIN_W, pathUnix.mb_str(wxConvFile), buf, sz);
+    #else
         char bufA[_MAXPATHLEN];
         cygwin_conv_to_full_win32_path(pathUnix.mb_str(wxConvFile), bufA);
         wxConvFile.MB2WC(buf, bufA, sz);
+    #endif
 #else
+    #if CYGWIN_VERSION_DLL_MAJOR >= 1007
+        cygwin_conv_path(CCP_POSIX_TO_WIN_A, pathUnix, buf, sz);
+    #else
         cygwin_conv_to_full_win32_path(pathUnix, buf);
+    #endif
 #endif // wxUSE_UNICODE
 #endif // __CYGWIN__
     }
@@ -1520,17 +1375,8 @@ wxChar *wxDoGetCwd(wxChar *buf, int sz)
     #undef cbuf
 #endif
 
-#endif
-    // __WXWINCE__
 }
 
-#if WXWIN_COMPATIBILITY_2_6
-wxChar *wxGetWorkingDirectory(wxChar *buf, int sz)
-{
-    return wxDoGetCwd(buf,sz);
-}
-#endif // WXWIN_COMPATIBILITY_2_6
-
 wxString wxGetCwd()
 {
     wxString str;
@@ -1540,28 +1386,13 @@ wxString wxGetCwd()
 
 bool wxSetWorkingDirectory(const wxString& d)
 {
-#if defined(__OS2__)
-    if (d[1] == ':')
-    {
-        ::DosSetDefaultDisk(wxToupper(d[0]) - wxT('A') + 1);
-    // do not call DosSetCurrentDir when just changing drive,
-    // since it requires e.g. "d:." instead of "d:"!
-    if (d.length() == 2)
-        return true;
-    }
-    return (::DosSetCurrentDir(d.c_str()) == 0);
-#elif defined(__UNIX__) || defined(__WXMAC__) || defined(__DOS__)
-    return (chdir(wxFNSTRINGCAST d.fn_str()) == 0);
+    bool success = false;
+#if defined(__UNIX__) || defined(__WXMAC__)
+    success = (chdir(wxFNSTRINGCAST d.fn_str()) == 0);
 #elif defined(__WINDOWS__)
 
 #ifdef __WIN32__
-#ifdef __WXWINCE__
-    // No equivalent in WinCE
-    wxUnusedVar(d);
-    return false;
-#else
-    return (bool)(SetCurrentDirectory(d.fn_str()) != 0);
-#endif
+    success = (SetCurrentDirectory(d.t_str()) != 0);
 #else
     // Must change drive, too.
     bool isDriveSpec = ((strlen(d) > 1) && (d[1] == ':'));
@@ -1581,26 +1412,29 @@ bool wxSetWorkingDirectory(const wxString& d)
             _dos_setdrive(driveNo, &noDrives);
         }
     }
-    bool success = (chdir(WXSTRINGCAST d) == 0);
+    success = (chdir(WXSTRINGCAST d) == 0);
+#endif
 
+#endif
+    if ( !success )
+    {
+       wxLogSysError(_("Could not set current working directory"));
+    }
     return success;
-#endif
-
-#endif
 }
 
 // Get the OS directory if appropriate (such as the Windows directory).
 // On non-Windows platform, probably just return the empty string.
 wxString wxGetOSDirectory()
 {
-#ifdef __WXWINCE__
-    return wxString(wxT("\\Windows"));
-#elif defined(__WINDOWS__) && !defined(__WXMICROWIN__)
-    wxChar buf[256];
-    GetWindowsDirectory(buf, 256);
+#if defined(__WINDOWS__)
+    wxChar buf[MAX_PATH];
+    if ( !GetWindowsDirectory(buf, MAX_PATH) )
+    {
+        wxLogLastError(wxS("GetWindowsDirectory"));
+    }
+
     return wxString(buf);
-#elif defined(__WXMAC__) && wxOSX_USE_CARBON
-    return wxMacFindFolderNoSeparator(kOnSystemDisk, 'macs', false);
 #else
     return wxEmptyString;
 #endif
@@ -1779,28 +1613,19 @@ int WXDLLIMPEXP_BASE wxParseCommonDialogsFilter(const wxString& filterStr,
     return filters.GetCount();
 }
 
-#if defined(__WINDOWS__) && !(defined(__UNIX__) || defined(__OS2__))
+#if defined(__WINDOWS__) && !defined(__UNIX__)
 static bool wxCheckWin32Permission(const wxString& path, DWORD access)
 {
     // quoting the MSDN: "To obtain a handle to a directory, call the
     // CreateFile function with the FILE_FLAG_BACKUP_SEMANTICS flag", but this
     // doesn't work under Win9x/ME but then it's not needed there anyhow
-    const DWORD dwAttr = ::GetFileAttributes(path.fn_str());
+    const DWORD dwAttr = ::GetFileAttributes(path.t_str());
     if ( dwAttr == INVALID_FILE_ATTRIBUTES )
     {
         // file probably doesn't exist at all
         return false;
     }
 
-    if ( wxGetOsVersion() == wxOS_WINDOWS_9X )
-    {
-        // FAT directories always allow all access, even if they have the
-        // readonly flag set, and FAT files can only be read-only
-        return (dwAttr & FILE_ATTRIBUTE_DIRECTORY) ||
-                    (access != GENERIC_WRITE ||
-                        !(dwAttr & FILE_ATTRIBUTE_READONLY));
-    }
-
     HANDLE h = ::CreateFile
                  (
                     path.t_str(),
@@ -1822,7 +1647,7 @@ static bool wxCheckWin32Permission(const wxString& path, DWORD access)
 
 bool wxIsWritable(const wxString &path)
 {
-#if defined( __UNIX__ ) || defined(__OS2__)
+#if defined( __UNIX__ )
     // access() will take in count also symbolic links
     return wxAccess(path.c_str(), W_OK) == 0;
 #elif defined( __WINDOWS__ )
@@ -1836,7 +1661,7 @@ bool wxIsWritable(const wxString &path)
 
 bool wxIsReadable(const wxString &path)
 {
-#if defined( __UNIX__ ) || defined(__OS2__)
+#if defined( __UNIX__ )
     // access() will take in count also symbolic links
     return wxAccess(path.c_str(), R_OK) == 0;
 #elif defined( __WINDOWS__ )
@@ -1850,7 +1675,7 @@ bool wxIsReadable(const wxString &path)
 
 bool wxIsExecutable(const wxString &path)
 {
-#if defined( __UNIX__ ) || defined(__OS2__)
+#if defined( __UNIX__ )
     // access() will take in count also symbolic links
     return wxAccess(path.c_str(), X_OK) == 0;
 #elif defined( __WINDOWS__ )
@@ -1882,7 +1707,7 @@ bool wxIsExecutable(const wxString &path)
 //
 wxFileKind wxGetFileKind(int fd)
 {
-#if defined __WXMSW__ && !defined __WXWINCE__ && defined wxGetOSFHandle
+#if defined __WINDOWS__ && defined wxGetOSFHandle
     switch (::GetFileType(wxGetOSFHandle(fd)) & ~FILE_TYPE_REMOTE)
     {
         case FILE_TYPE_CHAR:
@@ -1923,12 +1748,10 @@ wxFileKind wxGetFileKind(int fd)
 
 wxFileKind wxGetFileKind(FILE *fp)
 {
-    // Note: The watcom rtl dll doesn't have fileno (the static lib does).
-    //       Should be fixed in version 1.4.
-#if defined(wxFILEKIND_STUB) || wxONLY_WATCOM_EARLIER_THAN(1,4)
+#if defined(wxFILEKIND_STUB)
     (void)fp;
     return wxFILE_KIND_DISK;
-#elif defined(__WINDOWS__) && !defined(__CYGWIN__) && !defined(__WATCOMC__) && !defined(__WINE__)
+#elif defined(__WINDOWS__) && !defined(__CYGWIN__) && !defined(__WINE__)
     return fp ? wxGetFileKind(_fileno(fp)) : wxFILE_KIND_UNKNOWN;
 #else
     return fp ? wxGetFileKind(fileno(fp)) : wxFILE_KIND_UNKNOWN;
