@@ -2,9 +2,8 @@
 // Name:        src/dfb/window.cpp
 // Purpose:     wxWindow
 // Author:      Vaclav Slavik
-//              (based on GTK, MSW, MGL implementations)
+//              (based on GTK, MSW implementations)
 // Created:     2006-80-10
-// RCS-ID:      $Id$
 // Copyright:   (c) 2006 REA Elektronik GmbH
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -67,10 +66,10 @@ WX_DEFINE_ARRAY_PTR(wxOverlayImpl*, wxDfbOverlaysList);
 // ---------------------------------------------------------------------------
 
 // in wxUniv this class is abstract because it doesn't have DoPopupMenu()
-IMPLEMENT_ABSTRACT_CLASS(wxWindowDFB, wxWindowBase)
+wxIMPLEMENT_ABSTRACT_CLASS(wxWindowDFB, wxWindowBase);
 
-BEGIN_EVENT_TABLE(wxWindowDFB, wxWindowBase)
-END_EVENT_TABLE()
+wxBEGIN_EVENT_TABLE(wxWindowDFB, wxWindowBase)
+wxEND_EVENT_TABLE()
 
 //-----------------------------------------------------------------------------
 // global functions
@@ -323,7 +322,7 @@ bool wxWindowDFB::SetCursor(const wxCursor& cursor)
 
 #warning "implement this"
 #if 0
-    if ( m_cursor.Ok() )
+    if ( m_cursor.IsOk() )
         DFB_wmSetWindowCursor(m_wnd, *m_cursor.GetDFBCursor());
     else
         DFB_wmSetWindowCursor(m_wnd, *wxSTANDARD_CURSOR->GetDFBCursor());
@@ -1054,17 +1053,6 @@ void wxWindowDFB::HandleKeyEvent(const wxDFBWindowEvent& event_)
     }
 }
 
-// ---------------------------------------------------------------------------
-// idle events processing
-// ---------------------------------------------------------------------------
-
-void wxWindowDFB::OnInternalIdle()
-{
-    if (wxUpdateUIEvent::CanUpdate(this) && IsShown())
-        UpdateWindowUI(wxUPDATE_UI_FROMIDLE);
-}
-
-
 // Find the wxWindow at the current mouse position, returning the mouse
 // position.
 wxWindow* wxFindWindowAtPointer(wxPoint& pt)
