@@ -3,9 +3,8 @@
 // Purpose:     implements wxSearchCtrl as a composite control
 // Author:      Vince Harron
 // Created:     2006-02-19
-// RCS-ID:      $Id: srchctlg.cpp,v 1.14.2.1 2007/01/25 23:04:44 RD Exp $
 // Copyright:   Vince Harron
-// License:     wxWindows licence
+// Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
 
 // For compilers that support precompilation, includes "wx.h".
@@ -39,66 +38,6 @@
 // the margin between the text control and the search/cancel buttons
 static const wxCoord MARGIN = 2;
 
-// border around all controls to compensate for wxSIMPLE_BORDER
-#if defined(__WXMSW__)
-static const wxCoord BORDER = 0;
-static const wxCoord ICON_MARGIN = 2;
-static const wxCoord ICON_OFFSET = 2;
-#else
-static const wxCoord BORDER = 2;
-static const wxCoord ICON_MARGIN = 0;
-static const wxCoord ICON_OFFSET = 0;
-#endif
-
-// ----------------------------------------------------------------------------
-// TODO: These functions or something like them should probably be made
-// public.  There are similar functions in src/aui/dockart.cpp...
-
-static double wxBlendColour(double fg, double bg, double alpha)
-{
-    double result = bg + (alpha * (fg - bg));
-    if (result < 0.0)
-        result = 0.0;
-    if (result > 255)
-        result = 255;
-    return result;
-}
-
-static wxColor wxStepColour(const wxColor& c, int ialpha)
-{
-    if (ialpha == 100)
-        return c;
-        
-    double r = c.Red(), g = c.Green(), b = c.Blue();
-    double bg;
-    
-    // ialpha is 0..200 where 0 is completely black
-    // and 200 is completely white and 100 is the same
-    // convert that to normal alpha 0.0 - 1.0
-    ialpha = wxMin(ialpha, 200);
-    ialpha = wxMax(ialpha, 0);
-    double alpha = ((double)(ialpha - 100.0))/100.0;
-    
-    if (ialpha > 100)
-    {
-        // blend with white
-        bg = 255.0;
-        alpha = 1.0 - alpha;  // 0 = transparent fg; 1 = opaque fg
-    }
-     else
-    {
-        // blend with black
-        bg = 0.0;
-        alpha = 1.0 + alpha;  // 0 = transparent fg; 1 = opaque fg
-    }
-    
-    r = wxBlendColour(r, bg, alpha);
-    g = wxBlendColour(g, bg, alpha);
-    b = wxBlendColour(b, bg, alpha);
-    
-    return wxColour((unsigned char)r, (unsigned char)g, (unsigned char)b);
-}
-
 #define LIGHT_STEP 160
 
 // ----------------------------------------------------------------------------
@@ -110,30 +49,39 @@ class wxSearchTextCtrl : public wxTextCtrl
 public:
     wxSearchTextCtrl(wxSearchCtrl *search, const wxString& value, int style)
         : wxTextCtrl(search, wxID_ANY, value, wxDefaultPosition, wxDefaultSize,
-                     style | wxNO_BORDER)
+                     (style & ~wxBORDER_MASK) | wxNO_BORDER)
     {
         m_search = search;
-        m_defaultFG = GetForegroundColour();
-        
-        // remove the default minsize, the searchctrl will have one instead
-        SetSizeHints(wxDefaultCoord,wxDefaultCoord);
+
+        SetHint(_("Search"));
+
+        // Ensure that our best size is recomputed using our overridden
+        // DoGetBestSize().
+        InvalidateBestSize();
     }
 
-    void SetDescriptiveText(const wxString& text)
+    virtual wxWindow* GetMainWindowOfCompositeControl() wxOVERRIDE
     {
-        if ( GetValue() == m_descriptiveText )
-        {
-            ChangeValue(wxEmptyString);
-        }
-        
-        m_descriptiveText = text;
+        return m_search;
     }
-    
-    wxString GetDescriptiveText() const
+
+    // provide access to the base class protected methods to wxSearchCtrl which
+    // needs to forward to them
+    void DoSetValue(const wxString& value, int flags) wxOVERRIDE
     {
-        return m_descriptiveText;
+        wxTextCtrl::DoSetValue(value, flags);
     }
-    
+
+    bool DoLoadFile(const wxString& file, int fileType) wxOVERRIDE
+    {
+        return wxTextCtrl::DoLoadFile(file, fileType);
+    }
+
+    bool DoSaveFile(const wxString& file, int fileType) wxOVERRIDE
+    {
+        return wxTextCtrl::DoSaveFile(file, fileType);
+    }
+
 protected:
     void OnText(wxCommandEvent& eventText)
     {
@@ -159,42 +107,52 @@ protected:
         m_search->GetEventHandler()->ProcessEvent(event);
     }
 
-    void OnIdle(wxIdleEvent& WXUNUSED(event))
+#ifdef __WXMSW__
+    // We increase the text control height to be the same as for the controls
+    // with border as this is what we actually need here because even though
+    // this control itself is borderless, it's inside wxSearchCtrl which does
+    // have the border and so should have the same height as the normal text
+    // entries with border.
+    //
+    // This is a bit ugly and it would arguably be better to use whatever size
+    // the base class version returns and just centre the text vertically in
+    // the search control but I failed to modify the code in LayoutControls()
+    // to do this easily and as there is much in that code I don't understand
+    // (notably what is the logic for buttons sizing?) I prefer to not touch it
+    // at all.
+    virtual wxSize DoGetBestSize() const
     {
-        if ( IsEmpty() && !(wxWindow::FindFocus() == this) )
-        {
-            ChangeValue(m_descriptiveText);
-            SetInsertionPoint(0);
-            SetForegroundColour(wxStepColour(m_defaultFG, LIGHT_STEP));
-        }
-    }
+        const long flags = GetWindowStyleFlag();
+        wxSearchTextCtrl* const self = const_cast<wxSearchTextCtrl*>(this);
 
-    void OnFocus(wxFocusEvent& event)
-    {
-        event.Skip();
-        if ( GetValue() == m_descriptiveText )
-        {
-            ChangeValue(wxEmptyString);
-            SetForegroundColour(m_defaultFG);
-        }
+        self->SetWindowStyleFlag((flags & ~wxBORDER_MASK) | wxBORDER_DEFAULT);
+        wxSize size = wxTextCtrl::DoGetBestSize();
+
+        // The calculation for no external borders in wxTextCtrl::DoGetSizeFromTextSize also
+        // removes any padding around the value, which is wrong for this situation. So we
+        // can't use wxBORDER_NONE to calculate a good height, in which case we just have to
+        // assume a border in the code above and then subtract the space that would be taken up
+        // by a themed border (the thin blue border and the white internal border).
+        size.y -= 4;
+
+        self->SetWindowStyleFlag(flags);
+
+        return size;
     }
-    
+#endif // __WXMSW__
+
 private:
     wxSearchCtrl* m_search;
-    wxString      m_descriptiveText;
-    wxColour      m_defaultFG;
-    
-    DECLARE_EVENT_TABLE()
+
+    wxDECLARE_EVENT_TABLE();
 };
 
-BEGIN_EVENT_TABLE(wxSearchTextCtrl, wxTextCtrl)
+wxBEGIN_EVENT_TABLE(wxSearchTextCtrl, wxTextCtrl)
     EVT_TEXT(wxID_ANY, wxSearchTextCtrl::OnText)
     EVT_TEXT_ENTER(wxID_ANY, wxSearchTextCtrl::OnText)
     EVT_TEXT_URL(wxID_ANY, wxSearchTextCtrl::OnTextUrl)
     EVT_TEXT_MAXLEN(wxID_ANY, wxSearchTextCtrl::OnText)
-    EVT_IDLE(wxSearchTextCtrl::OnIdle)
-    EVT_SET_FOCUS(wxSearchTextCtrl::OnFocus)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // ----------------------------------------------------------------------------
 // wxSearchButton: search button used by search control
@@ -210,11 +168,27 @@ public:
           m_bmp(bmp)
     { }
 
-    void SetBitmapLabel(const wxBitmap& label) { m_bmp = label; }
+    void SetBitmapLabel(const wxBitmap& label)
+    {
+        m_bmp = label;
+        InvalidateBestSize();
+    }
 
+    // The buttons in wxSearchCtrl shouldn't accept focus from keyboard because
+    // this would interfere with the usual TAB processing: the user expects
+    // that pressing TAB in the search control should switch focus to the next
+    // control and not give it to the button inside the same control. Besides,
+    // the search button can be already activated by pressing "Enter" so there
+    // is really no reason for it to be able to get focus from keyboard.
+    virtual bool AcceptsFocusFromKeyboard() const wxOVERRIDE { return false; }
+
+    virtual wxWindow* GetMainWindowOfCompositeControl() wxOVERRIDE
+    {
+        return m_search;
+    }
 
 protected:
-    wxSize DoGetBestSize() const
+    wxSize DoGetBestSize() const wxOVERRIDE
     {
         return wxSize(m_bmp.GetWidth(), m_bmp.GetHeight());
     }
@@ -224,15 +198,25 @@ protected:
         wxCommandEvent event(m_eventType, m_search->GetId());
         event.SetEventObject(m_search);
 
+        if ( m_eventType == wxEVT_SEARCHCTRL_SEARCH_BTN )
+        {
+            // it's convenient to have the string to search for directly in the
+            // event instead of having to retrieve it from the control in the
+            // event handler code later, so provide it here
+            event.SetString(m_search->GetValue());
+        }
+
         GetEventHandler()->ProcessEvent(event);
 
         m_search->SetFocus();
 
-        if ( m_eventType == wxEVT_COMMAND_SEARCHCTRL_SEARCH_BTN )
+#if wxUSE_MENUS
+        if ( m_eventType == wxEVT_SEARCHCTRL_SEARCH_BTN )
         {
             // this happens automatically, just like on Mac OS X
             m_search->PopupSearchMenu();
         }
+#endif // wxUSE_MENUS
     }
 
     void OnPaint(wxPaintEvent&)
@@ -247,21 +231,21 @@ private:
     wxEventType   m_eventType;
     wxBitmap      m_bmp;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
-BEGIN_EVENT_TABLE(wxSearchButton, wxControl)
+wxBEGIN_EVENT_TABLE(wxSearchButton, wxControl)
     EVT_LEFT_UP(wxSearchButton::OnLeftUp)
     EVT_PAINT(wxSearchButton::OnPaint)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-BEGIN_EVENT_TABLE(wxSearchCtrl, wxSearchCtrlBase)
-    EVT_SEARCHCTRL_SEARCH_BTN(wxID_ANY, wxSearchCtrl::OnSearchButton)
+wxBEGIN_EVENT_TABLE(wxSearchCtrl, wxSearchCtrlBase)
+    EVT_SEARCHCTRL_CANCEL_BTN(wxID_ANY, wxSearchCtrl::OnCancelButton)
     EVT_SET_FOCUS(wxSearchCtrl::OnSetFocus)
     EVT_SIZE(wxSearchCtrl::OnSize)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-IMPLEMENT_DYNAMIC_CLASS(wxSearchCtrl, wxSearchCtrlBase)
+wxIMPLEMENT_DYNAMIC_CLASS(wxSearchCtrl, wxSearchCtrlBase);
 
 // ============================================================================
 // implementation
@@ -294,17 +278,21 @@ wxSearchCtrl::wxSearchCtrl(wxWindow *parent, wxWindowID id,
 
 void wxSearchCtrl::Init()
 {
-    m_text = 0;
-    m_searchButton = 0;
-    m_cancelButton = 0;
-    m_menu = 0;
+    m_text = NULL;
+    m_searchButton = NULL;
+    m_cancelButton = NULL;
+#if wxUSE_MENUS
+    m_menu = NULL;
+#endif // wxUSE_MENUS
 
     m_searchButtonVisible = true;
     m_cancelButtonVisible = false;
 
-    m_searchMenuBitmapUser = false;
     m_searchBitmapUser = false;
     m_cancelBitmapUser = false;
+#if wxUSE_MENUS
+    m_searchMenuBitmapUser = false;
+#endif // wxUSE_MENUS
 }
 
 bool wxSearchCtrl::Create(wxWindow *parent, wxWindowID id,
@@ -315,30 +303,35 @@ bool wxSearchCtrl::Create(wxWindow *parent, wxWindowID id,
             const wxValidator& validator,
             const wxString& name)
 {
+    // force border style for more native appearance
+    style &= ~wxBORDER_MASK;
 #ifdef __WXGTK__
-    if ( !wxTextCtrlBase::Create(parent, id, pos, size, wxSUNKEN_BORDER | style, validator, name) )
+    style |= wxBORDER_SUNKEN;
+#elif defined(__WXMSW__)
+    // Don't set the style explicitly, let GetDefaultBorder() work it out, unless
+    // we will get a sunken border (e.g. on Windows 200) in which case we must
+    // override with a simple border.
+    if (GetDefaultBorder() == wxBORDER_SUNKEN)
+        style |= wxBORDER_SIMPLE;
 #else
-    if ( !wxTextCtrlBase::Create(parent, id, pos, size, wxSIMPLE_BORDER | style, validator, name) )
+    style |= wxBORDER_SIMPLE;
 #endif
+    if ( !wxSearchCtrlBaseBaseClass::Create(parent, id, pos, size,
+                                            style, validator, name) )
     {
         return false;
     }
 
-    m_text = new wxSearchTextCtrl(this, value, style & ~wxBORDER_MASK);
-    m_text->SetDescriptiveText(_("Search"));
-    
-    wxSize sizeText = m_text->GetBestSize();
+    m_text = new wxSearchTextCtrl(this, value, style);
 
-    m_searchButton = new wxSearchButton(this,wxEVT_COMMAND_SEARCHCTRL_SEARCH_BTN,m_searchBitmap);
-    m_cancelButton = new wxSearchButton(this,wxEVT_COMMAND_SEARCHCTRL_CANCEL_BTN,m_cancelBitmap);
-
-    SetForegroundColour( m_text->GetForegroundColour() );
-    m_searchButton->SetForegroundColour( m_text->GetForegroundColour() );
-    m_cancelButton->SetForegroundColour( m_text->GetForegroundColour() );
+    m_searchButton = new wxSearchButton(this,
+                                        wxEVT_SEARCHCTRL_SEARCH_BTN,
+                                        m_searchBitmap);
+    m_cancelButton = new wxSearchButton(this,
+                                        wxEVT_SEARCHCTRL_CANCEL_BTN,
+                                        m_cancelBitmap);
 
     SetBackgroundColour( m_text->GetBackgroundColour() );
-    m_searchButton->SetBackgroundColour( m_text->GetBackgroundColour() );
-    m_cancelButton->SetBackgroundColour( m_text->GetBackgroundColour() );
 
     RecalcBitmaps();
 
@@ -352,11 +345,15 @@ wxSearchCtrl::~wxSearchCtrl()
     delete m_text;
     delete m_searchButton;
     delete m_cancelButton;
+#if wxUSE_MENUS
     delete m_menu;
+#endif // wxUSE_MENUS
 }
 
 
 // search control specific interfaces
+#if wxUSE_MENUS
+
 void wxSearchCtrl::SetMenu( wxMenu* menu )
 {
     if ( menu == m_menu )
@@ -381,8 +378,7 @@ void wxSearchCtrl::SetMenu( wxMenu* menu )
             m_searchButton->Refresh();
         }
     }
-    wxRect rect = GetRect();
-    LayoutControls(0, 0, rect.GetWidth(), rect.GetHeight());    
+    LayoutControls();
 }
 
 wxMenu* wxSearchCtrl::GetMenu()
@@ -390,6 +386,8 @@ wxMenu* wxSearchCtrl::GetMenu()
     return m_menu;
 }
 
+#endif // wxUSE_MENUS
+
 void wxSearchCtrl::ShowSearchButton( bool show )
 {
     if ( m_searchButtonVisible == show )
@@ -403,8 +401,7 @@ void wxSearchCtrl::ShowSearchButton( bool show )
         RecalcBitmaps();
     }
 
-    wxRect rect = GetRect();
-    LayoutControls(0, 0, rect.GetWidth(), rect.GetHeight());
+    LayoutControls();
 }
 
 bool wxSearchCtrl::IsSearchButtonVisible() const
@@ -422,8 +419,7 @@ void wxSearchCtrl::ShowCancelButton( bool show )
     }
     m_cancelButtonVisible = show;
 
-    wxRect rect = GetRect();
-    LayoutControls(0, 0, rect.GetWidth(), rect.GetHeight());
+    LayoutControls();
 }
 
 bool wxSearchCtrl::IsCancelButtonVisible() const
@@ -433,26 +429,26 @@ bool wxSearchCtrl::IsCancelButtonVisible() const
 
 void wxSearchCtrl::SetDescriptiveText(const wxString& text)
 {
-    m_text->SetDescriptiveText(text);
+    m_text->SetHint(text);
 }
 
 wxString wxSearchCtrl::GetDescriptiveText() const
 {
-    return m_text->GetDescriptiveText();
+    return m_text->GetHint();
 }
 
 // ----------------------------------------------------------------------------
 // geometry
 // ----------------------------------------------------------------------------
 
-wxSize wxSearchCtrl::DoGetBestSize() const
+wxSize wxSearchCtrl::DoGetBestClientSize() const
 {
     wxSize sizeText = m_text->GetBestSize();
     wxSize sizeSearch(0,0);
     wxSize sizeCancel(0,0);
     int searchMargin = 0;
     int cancelMargin = 0;
-    if ( m_searchButtonVisible || m_menu )
+    if ( m_searchButtonVisible || HasMenu() )
     {
         sizeSearch = m_searchButton->GetBestSize();
         searchMargin = MARGIN;
@@ -468,34 +464,36 @@ wxSize wxSearchCtrl::DoGetBestSize() const
     // buttons are square and equal to the height of the text control
     int height = sizeText.y;
     return wxSize(sizeSearch.x + searchMargin + sizeText.x + cancelMargin + sizeCancel.x + 2*horizontalBorder,
-                  height + 2*BORDER);
+                  height);
 }
 
 void wxSearchCtrl::DoMoveWindow(int x, int y, int width, int height)
 {
     wxSearchCtrlBase::DoMoveWindow(x, y, width, height);
 
-    LayoutControls(0, 0, width, height);
+    LayoutControls();
 }
 
-void wxSearchCtrl::LayoutControls(int x, int y, int width, int height)
+void wxSearchCtrl::LayoutControls()
 {
     if ( !m_text )
         return;
-    
+
+    const wxSize sizeTotal = GetClientSize();
+    int width = sizeTotal.x,
+        height = sizeTotal.y;
+
     wxSize sizeText = m_text->GetBestSize();
     // make room for the search menu & clear button
-    int horizontalBorder = ( sizeText.y - sizeText.y * 14 / 21 ) / 2;
-    x += horizontalBorder;
-    y += BORDER;
+    int horizontalBorder = 1 + ( sizeText.y - sizeText.y * 14 / 21 ) / 2;
+    int x = horizontalBorder;
     width -= horizontalBorder*2;
-    height -= BORDER*2;
 
     wxSize sizeSearch(0,0);
     wxSize sizeCancel(0,0);
     int searchMargin = 0;
     int cancelMargin = 0;
-    if ( m_searchButtonVisible || m_menu )
+    if ( m_searchButtonVisible || HasMenu() )
     {
         sizeSearch = m_searchButton->GetBestSize();
         searchMargin = MARGIN;
@@ -505,7 +503,7 @@ void wxSearchCtrl::LayoutControls(int x, int y, int width, int height)
         sizeCancel = m_cancelButton->GetBestSize();
         cancelMargin = MARGIN;
     }
-    m_searchButton->Show( m_searchButtonVisible || m_menu );
+    m_searchButton->Show( m_searchButtonVisible || HasMenu() );
     m_cancelButton->Show( m_cancelButtonVisible );
 
     if ( sizeSearch.x + sizeCancel.x > width )
@@ -515,36 +513,50 @@ void wxSearchCtrl::LayoutControls(int x, int y, int width, int height)
         searchMargin = 0;
         cancelMargin = 0;
     }
-    wxCoord textWidth = width - sizeSearch.x - sizeCancel.x - searchMargin - cancelMargin;
+    wxCoord textWidth = width - sizeSearch.x - sizeCancel.x - searchMargin - cancelMargin - 1;
+    if (textWidth < 0) textWidth = 0;
 
     // position the subcontrols inside the client area
 
-    m_searchButton->SetSize(x, y + ICON_OFFSET - 1, sizeSearch.x, height);
-    m_text->SetSize( x + sizeSearch.x + searchMargin, 
-                     y + ICON_OFFSET - BORDER,
-                     textWidth, 
-                     height);
-    m_cancelButton->SetSize(x + sizeSearch.x + searchMargin + textWidth + cancelMargin,
-                            y + ICON_OFFSET - 1, sizeCancel.x, height);
+    m_searchButton->SetSize(x, (height - sizeSearch.y) / 2,
+                            sizeSearch.x, height);
+    x += sizeSearch.x;
+    x += searchMargin;
+
+#ifdef __WXMSW__
+    // The text control is too high up on Windows; normally a text control looks OK because
+    // of the white border that's part of the theme border. We can also remove a pixel from
+    // the height to fit the text control in, because the padding in EDIT_HEIGHT_FROM_CHAR_HEIGHT
+    // is already generous.
+    int textY = 1;
+#else
+    int textY = 0;
+#endif
+
+    m_text->SetSize(x, textY, textWidth, height-textY);
+    x += textWidth;
+    x += cancelMargin;
+
+    m_cancelButton->SetSize(x, (height - sizeCancel.y) / 2,
+                            sizeCancel.x, height);
 }
 
+wxWindowList wxSearchCtrl::GetCompositeWindowParts() const
+{
+    wxWindowList parts;
+    parts.push_back(m_text);
+    parts.push_back(m_searchButton);
+    parts.push_back(m_cancelButton);
+    return parts;
+}
 
 // accessors
 // ---------
 
-wxString wxSearchCtrl::GetValue() const
+wxString wxSearchCtrl::DoGetValue() const
 {
-    wxString value = m_text->GetValue();
-    if (value == m_text->GetDescriptiveText())
-        return wxEmptyString;
-    else
-        return value;
+    return m_text->GetValue();
 }
-void wxSearchCtrl::SetValue(const wxString& value)
-{
-    m_text->SetValue(value);
-}
-
 wxString wxSearchCtrl::GetRange(long from, long to) const
 {
     return m_text->GetRange(from, to);
@@ -766,7 +778,7 @@ long wxSearchCtrl::GetInsertionPoint() const
 {
     return m_text->GetInsertionPoint();
 }
-wxTextPos wxSearchCtrl::GetLastPosition() const
+long wxSearchCtrl::GetLastPosition() const
 {
     return m_text->GetLastPosition();
 }
@@ -787,23 +799,35 @@ void wxSearchCtrl::SetEditable(bool editable)
 
 bool wxSearchCtrl::SetFont(const wxFont& font)
 {
-    bool result = wxSearchCtrlBase::SetFont(font);
-    if ( result && m_text )
-    {
-        result = m_text->SetFont(font);
-    }
+    if ( !wxSearchCtrlBase::SetFont(font) )
+        return false;
+
+    // Recreate the bitmaps as their size may have changed.
     RecalcBitmaps();
-    return result;
+
+    return true;
+}
+
+bool wxSearchCtrl::SetBackgroundColour(const wxColour& colour)
+{
+    if ( !wxSearchCtrlBase::SetBackgroundColour(colour) )
+        return false;
+
+    // When the background changes, re-render the bitmaps so that the correct
+    // colour shows in their "transparent" area.
+    RecalcBitmaps();
+
+    return true;
 }
 
 // search control generic only
 void wxSearchCtrl::SetSearchBitmap( const wxBitmap& bitmap )
 {
     m_searchBitmap = bitmap;
-    m_searchBitmapUser = bitmap.Ok();
+    m_searchBitmapUser = bitmap.IsOk();
     if ( m_searchBitmapUser )
     {
-        if ( m_searchButton && !m_menu )
+        if ( m_searchButton && !HasMenu() )
         {
             m_searchButton->SetBitmapLabel( m_searchBitmap );
         }
@@ -815,10 +839,12 @@ void wxSearchCtrl::SetSearchBitmap( const wxBitmap& bitmap )
     }
 }
 
+#if wxUSE_MENUS
+
 void wxSearchCtrl::SetSearchMenuBitmap( const wxBitmap& bitmap )
 {
     m_searchMenuBitmap = bitmap;
-    m_searchMenuBitmapUser = bitmap.Ok();
+    m_searchMenuBitmapUser = bitmap.IsOk();
     if ( m_searchMenuBitmapUser )
     {
         if ( m_searchButton && m_menu )
@@ -833,10 +859,12 @@ void wxSearchCtrl::SetSearchMenuBitmap( const wxBitmap& bitmap )
     }
 }
 
+#endif // wxUSE_MENUS
+
 void wxSearchCtrl::SetCancelBitmap( const wxBitmap& bitmap )
 {
     m_cancelBitmap = bitmap;
-    m_cancelBitmapUser = bitmap.Ok();
+    m_cancelBitmapUser = bitmap.IsOk();
     if ( m_cancelBitmapUser )
     {
         if ( m_cancelButton )
@@ -870,9 +898,17 @@ wxTextCtrl& operator<<(const wxChar c);
 
 void wxSearchCtrl::DoSetValue(const wxString& value, int flags)
 {
-    m_text->ChangeValue( value );
-    if ( flags & SetValue_SendEvent )
-        SendTextUpdatedEvent();
+    m_text->DoSetValue(value, flags);
+}
+
+bool wxSearchCtrl::DoLoadFile(const wxString& file, int fileType)
+{
+    return m_text->DoLoadFile(file, fileType);
+}
+
+bool wxSearchCtrl::DoSaveFile(const wxString& file, int fileType)
+{
+    return m_text->DoSaveFile(file, fileType);
 }
 
 // do the window-specific processing after processing the update event
@@ -890,10 +926,6 @@ bool wxSearchCtrl::ShouldInheritColours() const
 // antialiasing
 static int GetMultiplier()
 {
-#ifdef __WXWINCE__
-    // speed up bitmap generation by using a small bitmap
-    return 3;
-#else
     int depth = ::wxDisplayDepth();
 
     if  ( depth >= 24 )
@@ -901,13 +933,38 @@ static int GetMultiplier()
         return 8;
     }
     return 6;
-#endif
+}
+
+static void RescaleBitmap(wxBitmap& bmp, const wxSize& sizeNeeded)
+{
+    wxCHECK_RET( sizeNeeded.IsFullySpecified(), wxS("New size must be given") );
+
+#if wxUSE_IMAGE
+    wxImage img = bmp.ConvertToImage();
+    img.Rescale(sizeNeeded.x, sizeNeeded.y);
+    bmp = wxBitmap(img);
+#else // !wxUSE_IMAGE
+    // Fallback method of scaling the bitmap
+    wxBitmap newBmp(sizeNeeded, bmp.GetDepth());
+#if defined(__WXMSW__) || defined(__WXOSX__)
+    // wxBitmap::UseAlpha() is used only on wxMSW and wxOSX.
+    newBmp.UseAlpha(bmp.HasAlpha());
+#endif // __WXMSW__ || __WXOSX__
+    {
+        wxMemoryDC dc(newBmp);
+        double scX = (double)sizeNeeded.GetWidth() / bmp.GetWidth();
+        double scY = (double)sizeNeeded.GetHeight() / bmp.GetHeight();
+        dc.SetUserScale(scX, scY);
+        dc.DrawBitmap(bmp, 0, 0);
+    }
+    bmp = newBmp;
+#endif // wxUSE_IMAGE/!wxUSE_IMAGE
 }
 
 wxBitmap wxSearchCtrl::RenderSearchBitmap( int x, int y, bool renderDrop )
 {
     wxColour bg = GetBackgroundColour();
-    wxColour fg = wxStepColour(GetForegroundColour(), LIGHT_STEP-20);
+    wxColour fg = GetForegroundColour().ChangeLightness(LIGHT_STEP-20);
 
     //===============================================================================
     // begin drawing code
@@ -989,16 +1046,14 @@ wxBitmap wxSearchCtrl::RenderSearchBitmap( int x, int y, bool renderDrop )
         mem.DrawPolygon(WXSIZEOF(dropPolygon),dropPolygon,multiplier*triangleX,multiplier*triangleY);
     }
     mem.SelectObject(wxNullBitmap);
-    
+
     //===============================================================================
     // end drawing code
     //===============================================================================
 
     if ( multiplier != 1 )
     {
-        wxImage image = bitmap.ConvertToImage();
-        image.Rescale(x,y);
-        bitmap = wxBitmap( image );
+        RescaleBitmap(bitmap, wxSize(x, y));
     }
     if ( !renderDrop )
     {
@@ -1012,7 +1067,7 @@ wxBitmap wxSearchCtrl::RenderSearchBitmap( int x, int y, bool renderDrop )
 wxBitmap wxSearchCtrl::RenderCancelBitmap( int x, int y )
 {
     wxColour bg = GetBackgroundColour();
-    wxColour fg = wxStepColour(GetForegroundColour(), LIGHT_STEP);
+    wxColour fg = GetForegroundColour().ChangeLightness(LIGHT_STEP);
 
     //===============================================================================
     // begin drawing code
@@ -1086,9 +1141,7 @@ wxBitmap wxSearchCtrl::RenderCancelBitmap( int x, int y )
 
     if ( multiplier != 1 )
     {
-        wxImage image = bitmap.ConvertToImage();
-        image.Rescale(x,y);
-        bitmap = wxBitmap( image );
+        RescaleBitmap(bitmap, wxSize(x, y));
     }
 
     return bitmap;
@@ -1102,19 +1155,19 @@ void wxSearchCtrl::RecalcBitmaps()
     }
     wxSize sizeText = m_text->GetBestSize();
 
-    int bitmapHeight = sizeText.y - 2 * ICON_MARGIN;
+    int bitmapHeight = sizeText.y - 4;
     int bitmapWidth  = sizeText.y * 20 / 14;
 
     if ( !m_searchBitmapUser )
     {
         if (
-            !m_searchBitmap.Ok() ||
+            !m_searchBitmap.IsOk() ||
             m_searchBitmap.GetHeight() != bitmapHeight ||
             m_searchBitmap.GetWidth() != bitmapWidth
             )
         {
             m_searchBitmap = RenderSearchBitmap(bitmapWidth,bitmapHeight,false);
-            if ( !m_menu )
+            if ( !HasMenu() )
             {
                 m_searchButton->SetBitmapLabel(m_searchBitmap);
             }
@@ -1122,10 +1175,11 @@ void wxSearchCtrl::RecalcBitmaps()
         // else this bitmap was set by user, don't alter
     }
 
+#if wxUSE_MENUS
     if ( !m_searchMenuBitmapUser )
     {
         if (
-            !m_searchMenuBitmap.Ok() ||
+            !m_searchMenuBitmap.IsOk() ||
             m_searchMenuBitmap.GetHeight() != bitmapHeight ||
             m_searchMenuBitmap.GetWidth() != bitmapWidth
             )
@@ -1138,24 +1192,26 @@ void wxSearchCtrl::RecalcBitmaps()
         }
         // else this bitmap was set by user, don't alter
     }
+#endif // wxUSE_MENUS
 
     if ( !m_cancelBitmapUser )
     {
         if (
-            !m_cancelBitmap.Ok() ||
+            !m_cancelBitmap.IsOk() ||
             m_cancelBitmap.GetHeight() != bitmapHeight ||
             m_cancelBitmap.GetWidth() != bitmapHeight
             )
         {
-            m_cancelBitmap = RenderCancelBitmap(bitmapHeight-BORDER,bitmapHeight-BORDER); // square
+            m_cancelBitmap = RenderCancelBitmap(bitmapHeight,bitmapHeight); // square
             m_cancelButton->SetBitmapLabel(m_cancelBitmap);
         }
         // else this bitmap was set by user, don't alter
     }
 }
 
-void wxSearchCtrl::OnSearchButton( wxCommandEvent& event )
+void wxSearchCtrl::OnCancelButton( wxCommandEvent& event )
 {
+    m_text->Clear();
     event.Skip();
 }
 
@@ -1169,11 +1225,11 @@ void wxSearchCtrl::OnSetFocus( wxFocusEvent& /*event*/ )
 
 void wxSearchCtrl::OnSize( wxSizeEvent& WXUNUSED(event) )
 {
-    int width, height;
-    GetSize(&width, &height);
-    LayoutControls(0, 0, width, height);
+    LayoutControls();
 }
 
+#if wxUSE_MENUS
+
 void wxSearchCtrl::PopupSearchMenu()
 {
     if ( m_menu )
@@ -1183,6 +1239,8 @@ void wxSearchCtrl::PopupSearchMenu()
     }
 }
 
+#endif // wxUSE_MENUS
+
 #endif // !wxUSE_NATIVE_SEARCH_CONTROL
 
 #endif // wxUSE_SEARCHCTRL
