@@ -2,7 +2,6 @@
 // Name:        src/gtk1/listbox.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -160,7 +159,7 @@ static gint gtk_listitem_focus_out_callback( GtkWidget *WXUNUSED(widget),
 // "button_release_event"
 //-----------------------------------------------------------------------------
 
-/* we would normally emit a wxEVT_COMMAND_LISTBOX_DOUBLECLICKED event once
+/* we would normally emit a wxEVT_LISTBOX_DCLICK event once
    a GDK_2BUTTON_PRESS occurs, but this has the particular problem of the
    listbox keeping the focus until it receives a GDK_BUTTON_RELEASE event.
    this can lead to race conditions so that we emit the dclick event
@@ -181,7 +180,7 @@ gtk_listbox_button_release_callback( GtkWidget * WXUNUSED(widget),
 
     if (!g_hasDoubleClicked) return FALSE;
 
-    wxCommandEvent event( wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, listbox->GetId() );
+    wxCommandEvent event( wxEVT_LISTBOX_DCLICK, listbox->GetId() );
     event.SetEventObject( listbox );
 
     wxArrayInt aSelections;
@@ -234,7 +233,7 @@ gtk_listbox_button_press_callback( GtkWidget *widget,
 
         clb->Check( sel, !clb->IsChecked(sel) );
 
-        wxCommandEvent event( wxEVT_COMMAND_CHECKLISTBOX_TOGGLED, listbox->GetId() );
+        wxCommandEvent event( wxEVT_CHECKLISTBOX, listbox->GetId() );
         event.SetEventObject( listbox );
         event.SetInt( sel );
         listbox->HandleWindowEvent( event );
@@ -257,7 +256,7 @@ gtk_listbox_button_press_callback( GtkWidget *widget,
             return false;
     }
 
-    /* emit wxEVT_COMMAND_LISTBOX_DOUBLECLICKED later */
+    /* emit wxEVT_LISTBOX_DCLICK later */
     g_hasDoubleClicked = (gdk_event->type == GDK_2BUTTON_PRESS);
 
     return FALSE;
@@ -306,7 +305,7 @@ gtk_listbox_key_press_callback( GtkWidget *widget, GdkEventKey *gdk_event, wxLis
 
         clb->Check( sel, !clb->IsChecked(sel) );
 
-        wxCommandEvent new_event( wxEVT_COMMAND_CHECKLISTBOX_TOGGLED, listbox->GetId() );
+        wxCommandEvent new_event( wxEVT_CHECKLISTBOX, listbox->GetId() );
         new_event.SetEventObject( listbox );
         new_event.SetInt( sel );
         ret = listbox->HandleWindowEvent( new_event );
@@ -329,7 +328,7 @@ gtk_listbox_key_press_callback( GtkWidget *widget, GdkEventKey *gdk_event, wxLis
             else
                 gtk_list_select_item( listbox->m_list, sel );
 
-            wxCommandEvent new_event(wxEVT_COMMAND_LISTBOX_SELECTED, listbox->GetId() );
+            wxCommandEvent new_event(wxEVT_LISTBOX, listbox->GetId() );
             new_event.SetEventObject( listbox );
             wxArrayInt aSelections;
             int n, count = listbox->GetSelections(aSelections);
@@ -376,7 +375,7 @@ static void gtk_listitem_select_cb( GtkWidget *widget,
 
     if (listbox->m_blockEvent) return;
 
-    wxCommandEvent event(wxEVT_COMMAND_LISTBOX_SELECTED, listbox->GetId() );
+    wxCommandEvent event(wxEVT_LISTBOX, listbox->GetId() );
     event.SetEventObject( listbox );
 
     // indicate whether this is a selection or a deselection
@@ -452,8 +451,6 @@ gtk_listbox_realized_callback( GtkWidget *WXUNUSED(widget), wxListBox *win )
 // wxListBox
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxListBox, wxControlWithItems)
-
 // ----------------------------------------------------------------------------
 // construction
 // ----------------------------------------------------------------------------
@@ -544,7 +541,7 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
     if ( style & wxLB_SORT )
     {
         // this will change Append() behaviour
-        m_strings = new wxSortedArrayString;
+        m_strings = new wxSortedArrayString(wxDictionaryStringSortAscending);
     }
     else
     {
@@ -1014,7 +1011,7 @@ bool wxListBox::IsOwnGtkWindow( GdkWindow *window )
 
 void wxListBox::DoApplyWidgetStyle(GtkRcStyle *style)
 {
-    if (m_hasBgCol && m_backgroundColour.Ok())
+    if (m_hasBgCol && m_backgroundColour.IsOk())
     {
         GdkWindow *window = GTK_WIDGET(m_list)->window;
         if ( window )
@@ -1041,9 +1038,9 @@ void wxListBox::DoApplyWidgetStyle(GtkRcStyle *style)
 void wxListBox::OnInternalIdle()
 {
     wxCursor cursor = m_cursor;
-    if (g_globalCursor.Ok()) cursor = g_globalCursor;
+    if (g_globalCursor.IsOk()) cursor = g_globalCursor;
 
-    if (GTK_WIDGET(m_list)->window && cursor.Ok())
+    if (GTK_WIDGET(m_list)->window && cursor.IsOk())
     {
         /* I now set the cursor the anew in every OnInternalIdle call
            as setting the cursor in a parent window also effects the
