@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -47,12 +46,6 @@
 // macros and constants
 // ----------------------------------------------------------------------------
 
-#ifdef __WIN32__
-    #define _EXPORT
-#else
-    #define _EXPORT _export
-#endif
-
 #if wxUSE_UNICODE
     #define DDE_CP      CP_WINUNICODE
 #else
@@ -72,14 +65,15 @@ static wxDDEConnection *DDEFindConnection(HCONV hConv);
 static void DDEDeleteConnection(HCONV hConv);
 static wxDDEServer *DDEFindServer(const wxString& s);
 
-extern "C" HDDEDATA EXPENTRY _EXPORT _DDECallback(WORD wType,
-                                                  WORD wFmt,
-                                                  HCONV hConv,
-                                                  HSZ hsz1,
-                                                  HSZ hsz2,
-                                                  HDDEDATA hData,
-                                                  DWORD lData1,
-                                                  DWORD lData2);
+extern "C" HDDEDATA EXPENTRY
+_DDECallback(WORD wType,
+             WORD wFmt,
+             HCONV hConv,
+             HSZ hsz1,
+             HSZ hsz2,
+             HDDEDATA hData,
+             DWORD lData1,
+             DWORD lData2);
 
 // Add topic name to atom table before using in conversations
 static HSZ DDEAddAtom(const wxString& string);
@@ -130,17 +124,17 @@ public:
     void OnExit() { wxDDECleanUp(); }
 
 private:
-    DECLARE_DYNAMIC_CLASS(wxDDEModule)
+    wxDECLARE_DYNAMIC_CLASS(wxDDEModule);
 };
 
 // ----------------------------------------------------------------------------
 // wxWin macros
 // ----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxDDEServer, wxServerBase)
-IMPLEMENT_DYNAMIC_CLASS(wxDDEClient, wxClientBase)
-IMPLEMENT_CLASS(wxDDEConnection, wxConnectionBase)
-IMPLEMENT_DYNAMIC_CLASS(wxDDEModule, wxModule)
+wxIMPLEMENT_DYNAMIC_CLASS(wxDDEServer, wxServerBase);
+wxIMPLEMENT_DYNAMIC_CLASS(wxDDEClient, wxClientBase);
+wxIMPLEMENT_DYNAMIC_CLASS(wxDDEConnection, wxConnectionBase);
+wxIMPLEMENT_DYNAMIC_CLASS(wxDDEModule, wxModule);
 
 // ============================================================================
 // implementation
@@ -580,7 +574,7 @@ wxDDEConnection::DoExecute(const void *data, size_t size, wxIPCFormat format)
     if ( conv )
     {
         const char * const text = (const char *)data;
-        const size_t len = size/sizeof(char);
+        const size_t len = size;
 
         realSize = conv->ToWChar(NULL, 0, text, len);
         if ( realSize == wxCONV_FAILED )
@@ -593,6 +587,10 @@ wxDDEConnection::DoExecute(const void *data, size_t size, wxIPCFormat format)
         realSize = conv->ToWChar((wchar_t *)realData, realSize, text, len);
         if ( realSize == wxCONV_FAILED )
             return false;
+
+        // We need to pass the size of the buffer to DdeClientTransaction() and
+        // not the length of the string.
+        realSize *= sizeof(wchar_t);
     }
 #else // !wxUSE_UNICODE
     if ( format == wxIPC_UNICODETEXT )
@@ -623,7 +621,7 @@ wxDDEConnection::DoExecute(const void *data, size_t size, wxIPCFormat format)
         if ( realSize == wxCONV_FAILED )
             return false;
 
-        realData = (LPBYTE)buffer.GetWriteBuf(realSize*sizeof(char));
+        realData = (LPBYTE)buffer.GetWriteBuf(realSize);
         if ( !realData )
             return false;
 
@@ -635,7 +633,7 @@ wxDDEConnection::DoExecute(const void *data, size_t size, wxIPCFormat format)
 
     DWORD result;
     bool ok = DdeClientTransaction(realData,
-                                   realSize*sizeof(wxChar),
+                                   realSize,
                                    GetHConv(),
                                    NULL,
                                    // MSDN: if the transaction specified by
@@ -774,7 +772,7 @@ bool wxDDEConnection::DoAdvise(const wxString& item,
 
 #define DDERETURN HDDEDATA
 
-HDDEDATA EXPENTRY _EXPORT
+HDDEDATA EXPENTRY
 _DDECallback(WORD wType,
              WORD wFmt,
              HCONV hConv,
@@ -1064,7 +1062,7 @@ static HSZ DDEAtomFromString(const wxString& s)
 {
     wxASSERT_MSG( DDEIdInst, wxT("DDE not initialized") );
 
-    HSZ hsz = DdeCreateStringHandle(DDEIdInst, (wxChar*)s.wx_str(), DDE_CP);
+    HSZ hsz = DdeCreateStringHandle(DDEIdInst, wxMSW_CONV_LPTSTR(s), DDE_CP);
     if ( !hsz )
     {
         DDELogError(_("Failed to create DDE string"));
