@@ -4,9 +4,8 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     03.03.03 (replaces the old file with the same name)
-// RCS-ID:      $Id: dib.cpp,v 1.58 2004/10/07 13:36:40 ABX Exp $
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwindows.org>
-// License:     wxWindows licence
+// Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
 
 /*
@@ -30,31 +29,25 @@
     #pragma hdrstop
 #endif
 
+#if wxUSE_WXDIB
+
 #ifndef WX_PRECOMP
     #include "wx/string.h"
     #include "wx/log.h"
+    #include "wx/intl.h"
+    #include "wx/bitmap.h"
+    #include "wx/image.h"
 #endif //WX_PRECOMP
 
-#if wxUSE_WXDIB
-
-#include "wx/bitmap.h"
-#include "wx/intl.h"
 #include "wx/file.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#if !defined(__MWERKS__) && !defined(__SALFORDC__)
-    #include <memory.h>
-#endif
+#include <memory.h>
 
-#include "wx/image.h"
 #include "wx/msw/dib.h"
 
-#ifdef __WXWINCE__
-    #include <shellapi.h>       // for SHLoadDIBitmap()
-#endif
-
 // ----------------------------------------------------------------------------
 // private functions
 // ----------------------------------------------------------------------------
@@ -91,44 +84,38 @@ bool wxDIB::Create(int width, int height, int depth)
 {
     // we don't support formats using palettes right now so we only create
     // either 24bpp (RGB) or 32bpp (RGBA) bitmaps
-    wxASSERT_MSG( depth, _T("invalid image depth in wxDIB::Create()") );
+    wxASSERT_MSG( depth, wxT("invalid image depth in wxDIB::Create()") );
     if ( depth < 24 )
         depth = 24;
 
     // allocate memory for bitmap structures
-    static const int sizeHeader = sizeof(BITMAPINFOHEADER);
+    BITMAPINFO info;
+    wxZeroMemory(info);
 
-    BITMAPINFO *info = (BITMAPINFO *)malloc(sizeHeader);
-    wxCHECK_MSG( info, false, _T("malloc(BITMAPINFO) failed") );
-
-    memset(info, 0, sizeHeader);
-
-    info->bmiHeader.biSize = sizeHeader;
-    info->bmiHeader.biWidth = width;
+    info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
+    info.bmiHeader.biWidth = width;
 
     // we use positive height here which corresponds to a DIB with normal, i.e.
     // bottom to top, order -- normally using negative height (which means
     // reversed for MS and hence natural for all the normal people top to
     // bottom line scan order) could be used to avoid the need for the image
     // reversal in Create(image) but this doesn't work under NT, only Win9x!
-    info->bmiHeader.biHeight = height;
+    info.bmiHeader.biHeight = height;
 
-    info->bmiHeader.biPlanes = 1;
-    info->bmiHeader.biBitCount = (WORD)depth;
-    info->bmiHeader.biSizeImage = GetLineSize(width, depth)*height;
+    info.bmiHeader.biPlanes = 1;
+    info.bmiHeader.biBitCount = (WORD)depth;
+    info.bmiHeader.biSizeImage = GetLineSize(width, depth)*height;
 
     m_handle = ::CreateDIBSection
                  (
                     0,              // hdc (unused with DIB_RGB_COLORS)
-                    info,           // bitmap description
+                    &info,          // bitmap description
                     DIB_RGB_COLORS, // use RGB, not palette
                     &m_data,        // [out] DIB bits
                     NULL,           // don't use file mapping
                     0               // file mapping offset (not used here)
                  );
 
-    free(info);
-
     if ( !m_handle )
     {
         wxLogLastError(wxT("CreateDIBSection"));
@@ -143,20 +130,10 @@ bool wxDIB::Create(int width, int height, int depth)
     return true;
 }
 
-bool wxDIB::Create(const wxBitmap& bmp)
-{
-    wxCHECK_MSG( bmp.Ok(), false, _T("wxDIB::Create(): invalid bitmap") );
-
-    if ( !Create(GetHbitmapOf(bmp)) )
-        return false;
-
-    m_hasAlpha = bmp.HasAlpha();
-
-    return true;
-}
-
 bool wxDIB::Create(HBITMAP hbmp)
 {
+    wxCHECK_MSG( hbmp, false, wxT("wxDIB::Create(): invalid bitmap") );
+
     // this bitmap could already be a DIB section in which case we don't need
     // to convert it to DIB
     DIBSECTION ds;
@@ -196,59 +173,13 @@ bool wxDIB::Create(HBITMAP hbmp)
     return true;
 }
 
-// Windows CE doesn't have GetDIBits() so use an alternative implementation
-// for it
-//
-// in fact I'm not sure if GetDIBits() is really much better than using
-// BitBlt() like this -- it should be faster but I didn't do any tests, if
-// anybody has time to do them and by chance finds that GetDIBits() is not
-// much faster than BitBlt(), we could always use the Win CE version here
-#ifdef __WXWINCE__
-
-bool wxDIB::CopyFromDDB(HBITMAP hbmp)
-{
-    MemoryHDC hdcSrc;
-    if ( !hdcSrc )
-        return false;
-
-    SelectInHDC selectSrc(hdcSrc, hbmp);
-    if ( !selectSrc )
-        return false;
-
-    MemoryHDC hdcDst;
-    if ( !hdcDst )
-        return false;
-
-    SelectInHDC selectDst(hdcDst, m_handle);
-    if ( !selectDst )
-        return false;
-
-
-    if ( !::BitBlt(
-                    hdcDst,
-                    0, 0, m_width, m_height,
-                    hdcSrc,
-                    0, 0,
-                    SRCCOPY
-                  ) )
-    {
-        wxLogLastError(_T("BitBlt(DDB -> DIB)"));
-
-        return false;
-    }
-
-    return true;
-}
-
-#else // !__WXWINCE__
-
 bool wxDIB::CopyFromDDB(HBITMAP hbmp)
 {
     DIBSECTION ds;
     if ( !GetDIBSection(m_handle, &ds) )
     {
         // we're sure that our handle is a DIB section, so this should work
-        wxFAIL_MSG( _T("GetObject(DIBSECTION) unexpectedly failed") );
+        wxFAIL_MSG( wxT("GetObject(DIBSECTION) unexpectedly failed") );
 
         return false;
     }
@@ -272,30 +203,24 @@ bool wxDIB::CopyFromDDB(HBITMAP hbmp)
     return true;
 }
 
-#endif // __WXWINCE__/!__WXWINCE__
-
 // ----------------------------------------------------------------------------
 // Loading/saving the DIBs
 // ----------------------------------------------------------------------------
 
 bool wxDIB::Load(const wxString& filename)
 {
-#ifdef __WXWINCE__
-    m_handle = SHLoadDIBitmap(filename);
-#else // !__WXWINCE__
     m_handle = (HBITMAP)::LoadImage
                          (
                             wxGetInstance(),
-                            filename,
+                            filename.t_str(),
                             IMAGE_BITMAP,
                             0, 0, // don't specify the size
                             LR_CREATEDIBSECTION | LR_LOADFROMFILE
                          );
-#endif // __WXWINCE__
 
     if ( !m_handle )
     {
-        wxLogLastError(_T("Loading DIB from file"));
+        wxLogLastError(wxT("Loading DIB from file"));
 
         return false;
     }
@@ -305,8 +230,9 @@ bool wxDIB::Load(const wxString& filename)
 
 bool wxDIB::Save(const wxString& filename)
 {
-    wxCHECK_MSG( m_handle, false, _T("wxDIB::Save(): invalid object") );
+    wxCHECK_MSG( m_handle, false, wxT("wxDIB::Save(): invalid object") );
 
+#if wxUSE_FILE
     wxFile file(filename, wxFile::write);
     bool ok = file.IsOpened();
     if ( ok )
@@ -314,7 +240,7 @@ bool wxDIB::Save(const wxString& filename)
         DIBSECTION ds;
         if ( !GetDIBSection(m_handle, &ds) )
         {
-            wxLogLastError(_T("GetObject(hDIB)"));
+            wxLogLastError(wxT("GetObject(hDIB)"));
         }
         else
         {
@@ -335,6 +261,9 @@ bool wxDIB::Save(const wxString& filename)
                         file.Write(ds.dsBm.bmBits, sizeImage) == sizeImage;
         }
     }
+#else // !wxUSE_FILE
+    bool ok = false;
+#endif // wxUSE_FILE/!wxUSE_FILE
 
     if ( !ok )
     {
@@ -363,7 +292,7 @@ void wxDIB::DoGetObject() const
         DIBSECTION ds;
         if ( !GetDIBSection(m_handle, &ds) )
         {
-            wxLogLastError(_T("GetObject(hDIB)"));
+            wxLogLastError(wxT("GetObject(hDIB)"));
             return;
         }
 
@@ -380,27 +309,49 @@ void wxDIB::DoGetObject() const
 // DDB <-> DIB conversions
 // ----------------------------------------------------------------------------
 
-#ifndef __WXWINCE__
-
 HBITMAP wxDIB::CreateDDB(HDC hdc) const
 {
-    wxCHECK_MSG( m_handle, 0, _T("wxDIB::CreateDDB(): invalid object") );
+    wxCHECK_MSG( m_handle, 0, wxT("wxDIB::CreateDDB(): invalid object") );
 
     DIBSECTION ds;
     if ( !GetDIBSection(m_handle, &ds) )
     {
-        wxLogLastError(_T("GetObject(hDIB)"));
+        wxLogLastError(wxT("GetObject(hDIB)"));
 
         return 0;
     }
 
-    return ConvertToBitmap((BITMAPINFO *)&ds.dsBmih, hdc, ds.dsBm.bmBits);
+    // how many colours are we going to have in the palette?
+    DWORD biClrUsed = ds.dsBmih.biClrUsed;
+    if ( !biClrUsed )
+    {
+        // biClrUsed field might not be set
+        biClrUsed = GetNumberOfColours(ds.dsBmih.biBitCount);
+    }
+
+    if ( !biClrUsed )
+    {
+        return ConvertToBitmap((BITMAPINFO *)&ds.dsBmih, hdc, ds.dsBm.bmBits);
+    }
+    else
+    {
+        // fake a BITMAPINFO w/o bits, just the palette info
+        wxCharBuffer bmi(sizeof(BITMAPINFO) + (biClrUsed - 1)*sizeof(RGBQUAD));
+        BITMAPINFO *pBmi = (BITMAPINFO *)bmi.data();
+        MemoryHDC hDC;
+        // get the colour table
+        SelectInHDC sDC(hDC, m_handle);
+        ::GetDIBColorTable(hDC, 0, biClrUsed, pBmi->bmiColors);
+        memcpy(&pBmi->bmiHeader, &ds.dsBmih, ds.dsBmih.biSize);
+
+        return ConvertToBitmap(pBmi, hdc, ds.dsBm.bmBits);
+    }
 }
 
 /* static */
 HBITMAP wxDIB::ConvertToBitmap(const BITMAPINFO *pbmi, HDC hdc, void *bits)
 {
-    wxCHECK_MSG( pbmi, 0, _T("invalid DIB in ConvertToBitmap") );
+    wxCHECK_MSG( pbmi, 0, wxT("invalid DIB in ConvertToBitmap") );
 
     // here we get BITMAPINFO struct followed by the actual bitmap bits and
     // BITMAPINFO starts with BITMAPINFOHEADER followed by colour info
@@ -536,17 +487,17 @@ HGLOBAL wxDIB::ConvertFromBitmap(HBITMAP hbmp)
     {
         // this is an error which does risk to happen especially under Win9x
         // and which the user may understand so let him know about it
-        wxLogError(_("Failed to allocated %luKb of memory for bitmap data."),
+        wxLogError(_("Failed to allocate %luKb of memory for bitmap data."),
                    (unsigned long)(size / 1024));
 
         return NULL;
     }
 
-    if ( !ConvertFromBitmap((BITMAPINFO *)(void *)GlobalPtr(hDIB), hbmp) )
+    if ( !ConvertFromBitmap((BITMAPINFO *)(void *)GlobalPtrLock(hDIB), hbmp) )
     {
         // this really shouldn't happen... it worked the first time, why not
         // now?
-        wxFAIL_MSG( _T("wxDIB::ConvertFromBitmap() unexpectedly failed") );
+        wxFAIL_MSG( wxT("wxDIB::ConvertFromBitmap() unexpectedly failed") );
 
         return NULL;
     }
@@ -554,8 +505,6 @@ HGLOBAL wxDIB::ConvertFromBitmap(HBITMAP hbmp)
     return hDIB;
 }
 
-#endif // __WXWINCE__
-
 // ----------------------------------------------------------------------------
 // palette support
 // ----------------------------------------------------------------------------
@@ -564,12 +513,12 @@ HGLOBAL wxDIB::ConvertFromBitmap(HBITMAP hbmp)
 
 wxPalette *wxDIB::CreatePalette() const
 {
-    wxCHECK_MSG( m_handle, NULL, _T("wxDIB::CreatePalette(): invalid object") );
+    wxCHECK_MSG( m_handle, NULL, wxT("wxDIB::CreatePalette(): invalid object") );
 
     DIBSECTION ds;
     if ( !GetDIBSection(m_handle, &ds) )
     {
-        wxLogLastError(_T("GetObject(hDIB)"));
+        wxLogLastError(wxT("GetObject(hDIB)"));
 
         return 0;
     }
@@ -591,18 +540,23 @@ wxPalette *wxDIB::CreatePalette() const
         return NULL;
     }
 
+    MemoryHDC hDC;
+
     // LOGPALETTE struct has only 1 element in palPalEntry array, we're
     // going to have biClrUsed of them so add necessary space
     LOGPALETTE *pPalette = (LOGPALETTE *)
         malloc(sizeof(LOGPALETTE) + (biClrUsed - 1)*sizeof(PALETTEENTRY));
-    wxCHECK_MSG( pPalette, NULL, _T("out of memory") );
+    wxCHECK_MSG( pPalette, NULL, wxT("out of memory") );
 
     // initialize the palette header
     pPalette->palVersion = 0x300;  // magic number, not in docs but works
     pPalette->palNumEntries = (WORD)biClrUsed;
 
-    // and the colour table (it starts right after the end of the header)
-    const RGBQUAD *pRGB = (RGBQUAD *)((char *)&ds.dsBmih + ds.dsBmih.biSize);
+    // and the colour table
+    wxCharBuffer rgb(sizeof(RGBQUAD) * biClrUsed);
+    RGBQUAD *pRGB = (RGBQUAD*)rgb.data();
+    SelectInHDC selectHandle(hDC, m_handle);
+    ::GetDIBColorTable(hDC, 0, biClrUsed, pRGB);
     for ( DWORD i = 0; i < biClrUsed; i++, pRGB++ )
     {
         pPalette->palPalEntry[i].peRed = pRGB->rgbRed;
@@ -617,7 +571,7 @@ wxPalette *wxDIB::CreatePalette() const
 
     if ( !hPalette )
     {
-        wxLogLastError(_T("CreatePalette"));
+        wxLogLastError(wxT("CreatePalette"));
 
         return NULL;
     }
@@ -636,17 +590,17 @@ wxPalette *wxDIB::CreatePalette() const
 
 #if wxUSE_IMAGE
 
-bool wxDIB::Create(const wxImage& image)
+bool wxDIB::Create(const wxImage& image, PixelFormat pf)
 {
-    wxCHECK_MSG( image.Ok(), false, _T("invalid wxImage in wxDIB ctor") );
+    wxCHECK_MSG( image.IsOk(), false, wxT("invalid wxImage in wxDIB ctor") );
 
     const int h = image.GetHeight();
     const int w = image.GetWidth();
 
     // if we have alpha channel, we need to create a 32bpp RGBA DIB, otherwise
     // a 24bpp RGB is sufficient
-    m_hasAlpha = image.HasAlpha();
-    const int bpp = m_hasAlpha ? 32 : 24;
+    const bool hasAlpha = image.HasAlpha();
+    const int bpp = hasAlpha ? 32 : 24;
 
     if ( !Create(w, h, bpp) )
         return false;
@@ -656,32 +610,53 @@ bool wxDIB::Create(const wxImage& image)
     const int srcBytesPerLine = w * 3;
     const int dstBytesPerLine = GetLineSize(w, bpp);
     const unsigned char *src = image.GetData() + ((h - 1) * srcBytesPerLine);
-    const unsigned char *alpha = m_hasAlpha ? image.GetAlpha() + (h - 1)*w
-                                            : NULL;
+    const unsigned char *alpha = hasAlpha ? image.GetAlpha() + (h - 1)*w
+                                          : NULL;
     unsigned char *dstLineStart = (unsigned char *)m_data;
     for ( int y = 0; y < h; y++ )
     {
-        // copy one DIB line
+        // Copy one DIB line. Note that RGB components order is reversed in
+        // Windows bitmaps compared to wxImage and is actually BGR.
         unsigned char *dst = dstLineStart;
         if ( alpha )
         {
-            for ( int x = 0; x < w; x++ )
+            int x;
+
+            switch ( pf )
             {
-                // RGB order is reversed, and we need to premultiply
-                // all channels by alpha value for use with ::AlphaBlend.
-                const unsigned char a = *alpha++;
-                *dst++ = (unsigned char)((src[2] * a + 127) / 255);
-                *dst++ = (unsigned char)((src[1] * a + 127) / 255);
-                *dst++ = (unsigned char)((src[0] * a + 127) / 255);
-                *dst++ = a;
-                src += 3;
+                case PixelFormat_PreMultiplied:
+                    // Pre-multiply pixel values so that the DIB could be used
+                    // with ::AlphaBlend().
+                    for ( x = 0; x < w; x++ )
+                    {
+                        const unsigned char a = *alpha++;
+                        *dst++ = (unsigned char)((src[2] * a + 127) / 255);
+                        *dst++ = (unsigned char)((src[1] * a + 127) / 255);
+                        *dst++ = (unsigned char)((src[0] * a + 127) / 255);
+                        *dst++ = a;
+                        src += 3;
+                    }
+                    break;
+
+                case PixelFormat_NotPreMultiplied:
+                    // Just copy pixel data without changing it.
+                    for ( x = 0; x < w; x++ )
+                    {
+                        *dst++ = src[2];
+                        *dst++ = src[1];
+                        *dst++ = src[0];
+
+                        *dst++ = *alpha++;
+                        src += 3;
+                    }
+                    break;
             }
+
         }
         else // no alpha channel
         {
             for ( int x = 0; x < w; x++ )
             {
-                // RGB order is reversed.
                 *dst++ = src[2];
                 *dst++ = src[1];
                 *dst++ = src[0];
@@ -701,7 +676,7 @@ bool wxDIB::Create(const wxImage& image)
     return true;
 }
 
-wxImage wxDIB::ConvertToImage() const
+wxImage wxDIB::ConvertToImage(ConversionFlags flags) const
 {
     wxCHECK_MSG( IsOk(), wxNullImage,
                     wxT("can't convert invalid DIB to wxImage") );
@@ -710,26 +685,36 @@ wxImage wxDIB::ConvertToImage() const
     const int w = GetWidth();
     const int h = GetHeight();
     wxImage image(w, h, false /* don't bother clearing memory */);
-    if ( !image.Ok() )
+    if ( !image.IsOk() )
     {
         wxFAIL_MSG( wxT("could not allocate data for image") );
         return wxNullImage;
     }
 
-    if ( m_hasAlpha )
+    const int bpp = GetDepth();
+
+    // Remember if we have any "real" transparency, i.e. either any partially
+    // transparent pixels or not all pixels are fully opaque or fully
+    // transparent.
+    bool hasAlpha = false;
+    bool hasOpaque = false;
+    bool hasTransparent = false;
+
+    if ( bpp == 32 )
     {
+        // 32 bit bitmaps may be either 0RGB or ARGB and we don't know in
+        // advance which one do we have so suppose we have alpha of them and
+        // get rid of it later if it turns out we didn't.
         image.SetAlpha();
     }
 
     // this is the same loop as in Create() just above but with copy direction
     // reversed
-    const int bpp = GetDepth();
     const int dstBytesPerLine = w * 3;
     const int srcBytesPerLine = GetLineSize(w, bpp);
     unsigned char *dst = image.GetData() + ((h - 1) * dstBytesPerLine);
     unsigned char *alpha = image.HasAlpha() ? image.GetAlpha() + (h - 1)*w
                                             : NULL;
-    const bool is32bit = bpp == 32;
     const unsigned char *srcLineStart = (unsigned char *)GetData();
     for ( int y = 0; y < h; y++ )
     {
@@ -741,14 +726,42 @@ wxImage wxDIB::ConvertToImage() const
             dst[1] = *src++;
             dst[0] = *src++;
 
-            dst += 3;
-
-            if ( is32bit )
+            if ( bpp == 32 )
             {
-                if ( alpha )
-                    *alpha++ = *src;
+                // wxImage uses non premultiplied alpha so undo
+                // premultiplication done in Create() above
+                const unsigned char a = *src;
+                *alpha++ = a;
+
+                // Check what kind of alpha do we have.
+                switch ( a )
+                {
+                    case 0:
+                        hasTransparent = true;
+                        break;
+
+                    default:
+                        // Anything in between means we have real transparency
+                        // and must use alpha channel.
+                        hasAlpha = true;
+                        break;
+
+                    case 255:
+                        hasOpaque = true;
+                        break;
+                }
+
+                if ( a > 0 )
+                {
+                    dst[0] = (dst[0] * 255) / a;
+                    dst[1] = (dst[1] * 255) / a;
+                    dst[2] = (dst[2] * 255) / a;
+                }
+
                 src++;
             }
+
+            dst += 3;
         }
 
         // pass to the previous line in the image
@@ -760,10 +773,15 @@ wxImage wxDIB::ConvertToImage() const
         srcLineStart += srcBytesPerLine;
     }
 
+    if ( hasOpaque && hasTransparent )
+        hasAlpha = true;
+
+    if ( !hasAlpha && image.HasAlpha() && flags == Convert_AlphaAuto )
+        image.ClearAlpha();
+
     return image;
 }
 
 #endif // wxUSE_IMAGE
 
 #endif // wxUSE_WXDIB
-
