@@ -35,25 +35,6 @@
 #include "wx/dynlib.h"
 #include "wx/link.h"
 
-#ifdef __WXWINCE__
-/*
- * As WSAAsyncSelect is not present on WinCE, it now uses WSACreateEvent,
- * WSAEventSelect, WSAWaitForMultipleEvents and WSAEnumNetworkEvents. When
- * enabling eventhandling for a socket a new thread it created that keeps track
- * of the events and posts a messageto the hidden window to use the standard
- * message loop.
- */
-#include "wx/msw/wince/net.h"
-#include "wx/hashmap.h"
-WX_DECLARE_HASH_MAP(int,bool,wxIntegerHash,wxIntegerEqual,SocketHash);
-
-#ifndef isdigit
-#define isdigit(x) (x > 47 && x < 58)
-#endif
-#include "wx/msw/wince/net.h"
-
-#endif // __WXWINCE__
-
 #ifdef _MSC_VER
 #  pragma warning(default:4115) /* named type definition in parentheses */
 #endif
@@ -72,20 +53,7 @@ WX_DECLARE_HASH_MAP(int,bool,wxIntegerHash,wxIntegerEqual,SocketHash);
 #error "MAXSOCKETS is too big!"
 #endif
 
-#ifndef __WXWINCE__
 typedef int (PASCAL *WSAAsyncSelect_t)(SOCKET,HWND,u_int,long);
-#else
-/* Typedef the needed function prototypes and the WSANETWORKEVENTS structure
-*/
-typedef struct _WSANETWORKEVENTS {
-       long lNetworkEvents;
-       int iErrorCode[10];
-} WSANETWORKEVENTS, FAR * LPWSANETWORKEVENTS;
-typedef HANDLE (PASCAL *WSACreateEvent_t)();
-typedef int (PASCAL *WSAEventSelect_t)(SOCKET,HANDLE,long);
-typedef int (PASCAL *WSAWaitForMultipleEvents_t)(long,HANDLE,BOOL,long,BOOL);
-typedef int (PASCAL *WSAEnumNetworkEvents_t)(SOCKET,HANDLE,LPWSANETWORKEVENTS);
-#endif //__WXWINCE__
 
 LRESULT CALLBACK wxSocket_Internal_WinProc(HWND, UINT, WPARAM, LPARAM);
 
@@ -96,73 +64,7 @@ wxCRIT_SECT_DECLARE_MEMBER(gs_critical);
 static wxSocketImplMSW *socketList[MAXSOCKETS];
 static int firstAvailable;
 
-#ifndef __WXWINCE__
 static WSAAsyncSelect_t gs_WSAAsyncSelect = NULL;
-#else
-static SocketHash socketHash;
-static unsigned int currSocket;
-HANDLE hThread[MAXSOCKETS];
-static WSACreateEvent_t gs_WSACreateEvent = NULL;
-static WSAEventSelect_t gs_WSAEventSelect = NULL;
-static WSAWaitForMultipleEvents_t gs_WSAWaitForMultipleEvents = NULL;
-static WSAEnumNetworkEvents_t gs_WSAEnumNetworkEvents = NULL;
-/* This structure will be used to pass data on to the thread that handles socket events.
-*/
-typedef struct thread_data{
-    HWND hEvtWin;
-    unsigned long msgnumber;
-    unsigned long fd;
-    unsigned long lEvent;
-}thread_data;
-#endif
-
-#ifdef __WXWINCE__
-/* This thread handles socket events on WinCE using WSAEventSelect() as
- * WSAAsyncSelect is not supported. When an event occurs for the socket, it is
- * checked what kind of event happened and the correct message gets posted so
- * that the hidden window can handle it as it would in other MSW builds.
-*/
-DWORD WINAPI SocketThread(LPVOID data)
-{
-    WSANETWORKEVENTS NetworkEvents;
-    thread_data* d = (thread_data *)data;
-
-    HANDLE NetworkEvent = gs_WSACreateEvent();
-    gs_WSAEventSelect(d->fd, NetworkEvent, d->lEvent);
-
-    while(socketHash[d->fd] == true)
-    {
-        if ((gs_WSAWaitForMultipleEvents(1, &NetworkEvent, FALSE,INFINITE, FALSE)) == WAIT_FAILED)
-        {
-            printf("WSAWaitForMultipleEvents failed with error %d\n", WSAGetLastError());
-            return 0;
-        }
-        if (gs_WSAEnumNetworkEvents(d->fd ,NetworkEvent, &NetworkEvents) == SOCKET_ERROR)
-        {
-            printf("WSAEnumNetworkEvents failed with error %d\n", WSAGetLastError());
-            return 0;
-        }
-
-        long flags = NetworkEvents.lNetworkEvents;
-        if (flags & FD_READ)
-            ::PostMessage(d->hEvtWin, d->msgnumber,d->fd, FD_READ);
-        if (flags & FD_WRITE)
-            ::PostMessage(d->hEvtWin, d->msgnumber,d->fd, FD_WRITE);
-        if (flags & FD_OOB)
-            ::PostMessage(d->hEvtWin, d->msgnumber,d->fd, FD_OOB);
-        if (flags & FD_ACCEPT)
-            ::PostMessage(d->hEvtWin, d->msgnumber,d->fd, FD_ACCEPT);
-        if (flags & FD_CONNECT)
-            ::PostMessage(d->hEvtWin, d->msgnumber,d->fd, FD_CONNECT);
-        if (flags & FD_CLOSE)
-            ::PostMessage(d->hEvtWin, d->msgnumber,d->fd, FD_CLOSE);
-
-    }
-    gs_WSAEventSelect(d->fd, NetworkEvent, 0);
-    ExitThread(0);
-    return 0;
-}
-#endif
 
 // ----------------------------------------------------------------------------
 // MSW implementation of wxSocketManager
@@ -206,42 +108,18 @@ bool wxSocketMSWManager::OnInit()
   }
   firstAvailable = 0;
 
-  // we don't link with wsock32.dll (or ws2 in CE case) statically to avoid
+  // we don't link with wsock32.dll statically to avoid
   // dependencies on it for all the application using wx even if they don't use
   // sockets
-#ifdef __WXWINCE__
-    #define WINSOCK_DLL_NAME wxT("ws2.dll")
-#else
     #define WINSOCK_DLL_NAME wxT("wsock32.dll")
-#endif
 
     gs_wsock32dll.Load(WINSOCK_DLL_NAME, wxDL_VERBATIM | wxDL_QUIET);
     if ( !gs_wsock32dll.IsLoaded() )
         return false;
 
-#ifndef __WXWINCE__
     wxDL_INIT_FUNC(gs_, WSAAsyncSelect, gs_wsock32dll);
     if ( !gs_WSAAsyncSelect )
         return false;
-#else
-    wxDL_INIT_FUNC(gs_, WSAEventSelect, gs_wsock32dll);
-    if ( !gs_WSAEventSelect )
-        return false;
-
-    wxDL_INIT_FUNC(gs_, WSACreateEvent, gs_wsock32dll);
-    if ( !gs_WSACreateEvent )
-        return false;
-
-    wxDL_INIT_FUNC(gs_, WSAWaitForMultipleEvents, gs_wsock32dll);
-    if ( !gs_WSAWaitForMultipleEvents )
-        return false;
-
-    wxDL_INIT_FUNC(gs_, WSAEnumNetworkEvents, gs_wsock32dll);
-    if ( !gs_WSAEnumNetworkEvents )
-        return false;
-
-    currSocket = 0;
-#endif // !__WXWINCE__/__WXWINCE__
 
   // finally initialize WinSock
   WSADATA wsaData;
@@ -250,11 +128,6 @@ bool wxSocketMSWManager::OnInit()
 
 void wxSocketMSWManager::OnExit()
 {
-#ifdef __WXWINCE__
-/* Delete the threads here */
-    for(unsigned int i=0; i < currSocket; i++)
-        CloseHandle(hThread[i]);
-#endif
   /* Destroy internal window */
   DestroyWindow(hWin);
   UnregisterClass(CLASSNAME, wxGetInstance());
@@ -401,22 +274,7 @@ void wxSocketMSWManager::Install_Callback(wxSocketImpl *socket_,
      */
     long lEvent = socket->m_server?
                   FD_ACCEPT : (FD_READ | FD_WRITE | FD_CONNECT | FD_CLOSE);
-#ifndef __WXWINCE__
     gs_WSAAsyncSelect(socket->m_fd, hWin, socket->m_msgnumber, lEvent);
-#else
-/*
-*  WinCE creates a thread for socket event handling.
-*  All needed parameters get passed through the thread_data structure.
-*/
-
-    thread_data* d = new thread_data;
-    d->lEvent = lEvent;
-    d->hEvtWin = hWin;
-    d->msgnumber = socket->m_msgnumber;
-    d->fd = socket->m_fd;
-    socketHash[socket->m_fd] = true;
-    hThread[currSocket++] = CreateThread(NULL, 0, &SocketThread,(LPVOID)d, 0, NULL);
-#endif
   }
 }
 
@@ -430,12 +288,7 @@ void wxSocketMSWManager::Uninstall_Callback(wxSocketImpl *socket_,
 
   if (socket->m_fd != INVALID_SOCKET)
   {
-#ifndef __WXWINCE__
     gs_WSAAsyncSelect(socket->m_fd, hWin, socket->m_msgnumber, 0);
-#else
-    //Destroy the thread
-    socketHash[socket->m_fd] = false;
-#endif
   }
 }
 
