@@ -4,7 +4,6 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     2008-06-20
-// RCS-ID:      $Id: window.mm 48805 2007-09-19 14:52:25Z SC $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -327,10 +326,10 @@ void wxOSXIPhoneClassAddWXMethods(Class c)
 
 @end
 
-IMPLEMENT_DYNAMIC_CLASS( wxWidgetIPhoneImpl , wxWidgetImpl )
+wxIMPLEMENT_DYNAMIC_CLASS(wxWidgetIPhoneImpl , wxWidgetImpl);
 
-wxWidgetIPhoneImpl::wxWidgetIPhoneImpl( wxWindowMac* peer , WXWidget w, bool isRootControl ) :
-    wxWidgetImpl( peer, isRootControl ), m_osxView(w)
+wxWidgetIPhoneImpl::wxWidgetIPhoneImpl( wxWindowMac* peer , WXWidget w, bool isRootControl, bool isUserPane ) :
+    wxWidgetImpl( peer, isRootControl, isUserPane ), m_osxView(w)
 {
 }
 
@@ -392,6 +391,8 @@ void wxWidgetIPhoneImpl::Move(int x, int y, int width, int height)
     [m_osxView setFrame:r];
 }
 
+
+
 void wxWidgetIPhoneImpl::GetPosition( int &x, int &y ) const
 {
     CGRect r = [m_osxView frame];
@@ -477,7 +478,14 @@ void wxWidgetIPhoneImpl::SetBackgroundColour( const wxColour &col )
 
 bool wxWidgetIPhoneImpl::SetBackgroundStyle(wxBackgroundStyle style) 
 {
-    [m_osxView setOpaque: (style == wxBG_STYLE_PAINT) ];
+    if ( style == wxBG_STYLE_PAINT )
+        [m_osxView setOpaque: YES ];
+    else
+    {
+        [m_osxView setOpaque: NO ];
+        m_osxView.backgroundColor = [UIColor clearColor];
+    }
+    return true;
 }
 
 void wxWidgetIPhoneImpl::SetLabel(const wxString& title, wxFontEncoding encoding)
@@ -487,11 +495,13 @@ void wxWidgetIPhoneImpl::SetLabel(const wxString& title, wxFontEncoding encoding
         wxCFStringRef cf( title , encoding );
         [m_osxView setTitle:cf.AsNSString() forState:UIControlStateNormal ];
     }
+#if 0 // nonpublic API problems
     else if ( [m_osxView respondsToSelector:@selector(setStringValue:) ] )
     {
         wxCFStringRef cf( title , encoding );
         [m_osxView setStringValue:cf.AsNSString()];
     }
+#endif
 }
 
 
@@ -550,10 +560,22 @@ void wxWidgetIPhoneImpl::GetBestRect( wxRect *r ) const
 
 bool wxWidgetIPhoneImpl::IsEnabled() const
 {
+    UIView* targetView = m_osxView;
+    // TODO add support for documentViews
+
+    if ( [targetView respondsToSelector:@selector(isEnabled) ] )
+        return [targetView isEnabled];
+    
+    return true;
 }
 
 void wxWidgetIPhoneImpl::Enable( bool enable )
 {
+    UIView* targetView = m_osxView;
+    // TODO add support for documentViews
+
+    if ( [targetView respondsToSelector:@selector(setEnabled:) ] )
+        [targetView setEnabled:enable];
 }
 
 void wxWidgetIPhoneImpl::SetMinimum( wxInt32 v )
@@ -566,10 +588,12 @@ void wxWidgetIPhoneImpl::SetMaximum( wxInt32 v )
 
 wxInt32 wxWidgetIPhoneImpl::GetMinimum() const
 {
+    return 0;
 }
 
 wxInt32 wxWidgetIPhoneImpl::GetMaximum() const
 {
+    return 0;
 }
 
 void wxWidgetIPhoneImpl::PulseGauge()
@@ -584,6 +608,14 @@ void wxWidgetIPhoneImpl::SetControlSize( wxWindowVariant variant )
 {
 }
 
+double wxWidgetIPhoneImpl::GetContentScaleFactor() const 
+{
+    if ( [m_osxView respondsToSelector:@selector(contentScaleFactor) ])
+        return [m_osxView contentScaleFactor];
+    else 
+        return 1.0;
+}
+
 void wxWidgetIPhoneImpl::SetFont( const wxFont & font , const wxColour& foreground , long windowStyle, bool ignoreBlack )
 {
 }
@@ -681,10 +713,10 @@ void wxWidgetIPhoneImpl::drawRect(CGRect* rect, WXWidget slf, void *WXUNUSED(_cm
     CGContextRef context = (CGContextRef) UIGraphicsGetCurrentContext();
     CGContextSaveGState( context );
     // draw background
-
+/*
     CGContextSetFillColorWithColor( context, GetWXPeer()->GetBackgroundColour().GetCGColor());
     CGContextFillRect(context, *rect );
-
+*/
     GetWXPeer()->MacSetCGContextRef( context );
 
     GetWXPeer()->GetUpdateRegion() =
@@ -760,7 +792,7 @@ void wxWidgetIPhoneImpl::controlTextDidChange()
     wxTextCtrl* wxpeer = wxDynamicCast((wxWindow*)GetWXPeer(),wxTextCtrl);
     if ( wxpeer ) 
     {
-        wxCommandEvent event(wxEVT_COMMAND_TEXT_UPDATED, wxpeer->GetId());
+        wxCommandEvent event(wxEVT_TEXT, wxpeer->GetId());
         event.SetEventObject( wxpeer );
         event.SetString( wxpeer->GetValue() );
         wxpeer->HandleWindowEvent( event );
@@ -783,7 +815,7 @@ wxWidgetImpl* wxWidgetImpl::CreateUserPane( wxWindowMac* wxpeer, wxWindowMac* WX
     sv.clipsToBounds = YES;
     sv.contentMode =  UIViewContentModeRedraw;
     sv.clearsContextBeforeDrawing = NO;
-    wxWidgetIPhoneImpl* c = new wxWidgetIPhoneImpl( wxpeer, v );
+    wxWidgetIPhoneImpl* c = new wxWidgetIPhoneImpl( wxpeer, v, false, true );
     return c;
 }
 
