@@ -1,10 +1,9 @@
 ///////////////////////////////////////////////////////////////////////////////
-// Name:        src/univ/window.cpp
+// Name:        src/univ/winuniv.cpp
 // Purpose:     implementation of extra wxWindow methods for wxUniv port
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     06.08.00
-// RCS-ID:      $Id$
 // Copyright:   (c) 2000 SciTech Software, Inc. (www.scitechsoft.com)
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -52,7 +51,7 @@
     #define WXDEBUG_REFRESH
 #endif
 
-#if defined(WXDEBUG_REFRESH) && defined(__WXMSW__) && !defined(__WXMICROWIN__)
+#if defined(WXDEBUG_REFRESH) && defined(__WXMSW__)
 #include "wx/msw/private.h"
 #endif
 
@@ -89,22 +88,18 @@ public:
 
 // we can't use wxWindowNative here as it won't be expanded inside the macro
 #if defined(__WXMSW__)
-    IMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowMSW)
+    wxIMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowMSW);
 #elif defined(__WXGTK__)
-    IMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowGTK)
-#elif defined(__WXOSX_OR_COCOA__)
-    IMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowMac)
-#elif defined(__WXMGL__)
-    IMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowMGL)
+    wxIMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowGTK);
+#elif defined(__WXOSX__)
+    wxIMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowMac);
 #elif defined(__WXDFB__)
-    IMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowDFB)
+    wxIMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowDFB);
 #elif defined(__WXX11__)
-    IMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowX11)
-#elif defined(__WXPM__)
-    IMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowOS2)
+    wxIMPLEMENT_DYNAMIC_CLASS(wxWindow, wxWindowX11);
 #endif
 
-BEGIN_EVENT_TABLE(wxWindow, wxWindowNative)
+wxBEGIN_EVENT_TABLE(wxWindow, wxWindowNative)
     EVT_SIZE(wxWindow::OnSize)
 
 #if wxUSE_ACCEL || wxUSE_MENUS
@@ -119,7 +114,7 @@ BEGIN_EVENT_TABLE(wxWindow, wxWindowNative)
     EVT_PAINT(wxWindow::OnPaint)
     EVT_NC_PAINT(wxWindow::OnNcPaint)
     EVT_ERASE_BACKGROUND(wxWindow::OnErase)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // ----------------------------------------------------------------------------
 // creation
@@ -258,7 +253,7 @@ void wxWindow::SetBackground(const wxBitmap& bitmap,
 const wxBitmap& wxWindow::GetBackgroundBitmap(int *alignment,
                                                wxStretch *stretch) const
 {
-    if ( m_bitmapBg.Ok() )
+    if ( m_bitmapBg.IsOk() )
     {
         if ( alignment )
             *alignment = m_alignBgBitmap;
@@ -408,7 +403,7 @@ bool wxWindow::DoDrawBackground(wxDC& dc)
 
 void wxWindow::EraseBackground(wxDC& dc, const wxRect& rect)
 {
-    if ( GetBackgroundBitmap().Ok() )
+    if ( GetBackgroundBitmap().IsOk() )
     {
         // Get the bitmap and the flags
         int alignment;
@@ -482,7 +477,7 @@ void wxWindow::Refresh(bool eraseBackground, const wxRect *rect)
         dc.DrawRectangle(rectWin);
 
         // under Unix we use "--sync" X option for this
-        #if defined(__WXMSW__) && !defined(__WXMICROWIN__)
+        #if defined(__WXMSW__)
             ::GdiFlush();
             ::Sleep(200);
         #endif // __WXMSW__
@@ -718,16 +713,6 @@ void wxWindow::OnSize(wxSizeEvent& event)
 #endif
 }
 
-wxSize wxWindow::DoGetBestSize() const
-{
-    return AdjustSize(DoGetBestClientSize());
-}
-
-wxSize wxWindow::DoGetBestClientSize() const
-{
-    return wxWindowNative::DoGetBestSize();
-}
-
 wxSize wxWindow::DoGetBorderSize() const
 {
     return AdjustSize(wxSize(0, 0));
@@ -1334,7 +1319,7 @@ void wxWindow::OnKeyDown(wxKeyEvent& event)
         int command = win->GetAcceleratorTable()->GetCommand(event);
         if ( command != -1 )
         {
-            wxCommandEvent eventCmd(wxEVT_COMMAND_MENU_SELECTED, command);
+            wxCommandEvent eventCmd(wxEVT_MENU, command);
             if ( win->GetEventHandler()->ProcessEvent(eventCmd) )
             {
                 // skip "event.Skip()" below
@@ -1365,7 +1350,7 @@ void wxWindow::OnKeyDown(wxKeyEvent& event)
                 wxWindow* child = win->FindWindow(command);
                 if ( child && wxDynamicCast(child, wxButton) )
                 {
-                    wxCommandEvent eventCmd(wxEVT_COMMAND_BUTTON_CLICKED, command);
+                    wxCommandEvent eventCmd(wxEVT_BUTTON, command);
                     eventCmd.SetEventObject(child);
                     if ( child->GetEventHandler()->ProcessEvent(eventCmd) )
                     {
@@ -1440,7 +1425,7 @@ void wxWindow::OnChar(wxKeyEvent& event)
             wxButton *btn = wxDynamicCast(tlw->GetDefaultItem(), wxButton);
             if ( btn )
             {
-                wxCommandEvent evt(wxEVT_COMMAND_BUTTON_CLICKED, btn->GetId());
+                wxCommandEvent evt(wxEVT_BUTTON, btn->GetId());
                 evt.SetEventObject(btn);
                 btn->Command(evt);
                 return;
