@@ -3,7 +3,6 @@
 // Purpose:     wxArray unit test
 // Author:      Vadim Zeitlin, Wlodzimierz ABX Skiba
 // Created:     2004-04-01
-// RCS-ID:      $Id$
 // Copyright:   (c) 2004 Vadim Zeitlin, Wlodzimierz Skiba
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -160,6 +159,7 @@ public:
 private:
     CPPUNIT_TEST_SUITE( ArraysTestCase );
         CPPUNIT_TEST( wxStringArrayTest );
+        CPPUNIT_TEST( SortedArray );
         CPPUNIT_TEST( wxStringArraySplitTest );
         CPPUNIT_TEST( wxStringArrayJoinTest );
         CPPUNIT_TEST( wxStringArraySplitJoinTest );
@@ -172,9 +172,11 @@ private:
         CPPUNIT_TEST( Alloc );
         CPPUNIT_TEST( Clear );
         CPPUNIT_TEST( Swap );
+        CPPUNIT_TEST( IndexFromEnd );
     CPPUNIT_TEST_SUITE_END();
 
     void wxStringArrayTest();
+    void SortedArray();
     void wxStringArraySplitTest();
     void wxStringArrayJoinTest();
     void wxStringArraySplitJoinTest();
@@ -186,14 +188,15 @@ private:
     void Alloc();
     void Clear();
     void Swap();
+    void IndexFromEnd();
 
-    DECLARE_NO_COPY_CLASS(ArraysTestCase)
+    wxDECLARE_NO_COPY_CLASS(ArraysTestCase);
 };
 
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION( ArraysTestCase );
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ArraysTestCase, "ArraysTestCase" );
 
 void ArraysTestCase::wxStringArrayTest()
@@ -305,6 +308,13 @@ void ArraysTestCase::wxStringArrayTest()
     CPPUNIT_ASSERT( a1.Index( wxT("thermit") ) == 3 );
     CPPUNIT_ASSERT( a1.Index( wxT("alligator") ) == 4 );
 
+    CPPUNIT_ASSERT( a1.Index( wxT("dog"), /*bCase=*/true, /*fromEnd=*/true ) == 0 );
+    CPPUNIT_ASSERT( a1.Index( wxT("human"), /*bCase=*/true, /*fromEnd=*/true ) == 1 );
+    CPPUNIT_ASSERT( a1.Index( wxT("humann"), /*bCase=*/true, /*fromEnd=*/true ) == wxNOT_FOUND );
+    CPPUNIT_ASSERT( a1.Index( wxT("condor"), /*bCase=*/true, /*fromEnd=*/true ) == 2 );
+    CPPUNIT_ASSERT( a1.Index( wxT("thermit"), /*bCase=*/true, /*fromEnd=*/true ) == 3 );
+    CPPUNIT_ASSERT( a1.Index( wxT("alligator"), /*bCase=*/true, /*fromEnd=*/true ) == 4 );
+
     wxArrayString a5;
 
     CPPUNIT_ASSERT( a5.Add( wxT("x"), 1 ) == 0 );
@@ -321,12 +331,10 @@ void ArraysTestCase::wxStringArrayTest()
     a5.assign(a1.begin(), a1.end());
     CPPUNIT_ASSERT( a5 == a1 );
 
-#ifdef wxHAS_VECTOR_TEMPLATE_ASSIGN
     const wxString months[] = { "Jan", "Feb", "Mar" };
     a5.assign(months, months + WXSIZEOF(months));
     CPPUNIT_ASSERT_EQUAL( WXSIZEOF(months), a5.size() );
     CPPUNIT_ASSERT( COMPARE_3_VALUES(a5, "Jan", "Feb", "Mar") );
-#endif // wxHAS_VECTOR_TEMPLATE_ASSIGN
 
     a5.clear();
     CPPUNIT_ASSERT_EQUAL( 0, a5.size() );
@@ -338,6 +346,36 @@ void ArraysTestCase::wxStringArrayTest()
     a5.resize(3);
     CPPUNIT_ASSERT_EQUAL( 3, a5.size() );
     CPPUNIT_ASSERT_EQUAL( "Foo", a5[2] );
+
+    wxArrayString a6;
+    a6.Add("Foo");
+    a6.Insert(a6[0], 1, 100);
+}
+
+void ArraysTestCase::SortedArray()
+{
+    wxSortedArrayString a;
+    a.Add("d");
+    a.Add("c");
+    CPPUNIT_ASSERT_EQUAL( 0, a.Index("c") );
+
+    a.push_back("b");
+    a.push_back("a");
+    CPPUNIT_ASSERT_EQUAL( 0, a.Index("a") );
+
+
+    wxSortedArrayString ar(wxStringSortDescending);
+    ar.Add("a");
+    ar.Add("b");
+    CPPUNIT_ASSERT_EQUAL( "b", ar[0] );
+    CPPUNIT_ASSERT_EQUAL( "a", ar[1] );
+
+    wxSortedArrayString ad(wxDictionaryStringSortAscending);
+    ad.Add("AB");
+    ad.Add("a");
+    ad.Add("Aa");
+    CPPUNIT_ASSERT_EQUAL( "a", ad[0] );
+    CPPUNIT_ASSERT_EQUAL( "Aa", ad[1] );
 }
 
 void ArraysTestCase::wxStringArraySplitTest()
@@ -602,8 +640,7 @@ namespace
 {
 
 template <typename A, typename T>
-void DoTestSwap(T v1, T v2, T v3,
-                A * WXUNUSED(dummyUglyVC6Workaround))
+void DoTestSwap(T v1, T v2, T v3)
 {
     A a1, a2;
     a1.swap(a2);
@@ -631,10 +668,10 @@ void DoTestSwap(T v1, T v2, T v3,
 
 void ArraysTestCase::Swap()
 {
-    DoTestSwap("Foo", "Bar", "Baz", (wxArrayString *)NULL);
+    DoTestSwap<wxArrayString>("Foo", "Bar", "Baz");
 
-    DoTestSwap(1, 10, 100, (wxArrayInt *)NULL);
-    DoTestSwap(6, 28, 496, (wxArrayLong *)NULL);
+    DoTestSwap<wxArrayInt>(1, 10, 100);
+    DoTestSwap<wxArrayLong>(6, 28, 496);
 }
 
 void ArraysTestCase::TestSTL()
@@ -691,4 +728,20 @@ void ArraysTestCase::TestSTL()
     items.push_back(new Item(17));
     CPPUNIT_ASSERT_EQUAL( 17, (*(items.rbegin()))->n );
     CPPUNIT_ASSERT_EQUAL( 17, (**items.begin()).n );
+    WX_CLEAR_ARRAY(items);
+}
+
+void ArraysTestCase::IndexFromEnd()
+{
+    wxArrayInt a;
+    a.push_back(10);
+    a.push_back(1);
+    a.push_back(42);
+
+    CPPUNIT_ASSERT_EQUAL( 0, a.Index(10) );
+    CPPUNIT_ASSERT_EQUAL( 1, a.Index(1) );
+    CPPUNIT_ASSERT_EQUAL( 2, a.Index(42) );
+    CPPUNIT_ASSERT_EQUAL( 0, a.Index(10, /*bFromEnd=*/true) );
+    CPPUNIT_ASSERT_EQUAL( 1, a.Index(1, /*bFromEnd=*/true) );
+    CPPUNIT_ASSERT_EQUAL( 2, a.Index(42, /*bFromEnd=*/true) );
 }
