@@ -3,7 +3,6 @@
 // Purpose:     wxURI unit test
 // Author:      Ryan Norton
 // Created:     2004-08-14
-// RCS-ID:      $Id$
 // Copyright:   (c) 2004 Ryan Norton
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -82,13 +81,13 @@ private:
 #endif
 #endif
 
-    DECLARE_NO_COPY_CLASS(URITestCase)
+    wxDECLARE_NO_COPY_CLASS(URITestCase);
 };
 
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION( URITestCase );
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( URITestCase, "URITestCase" );
 
 URITestCase::URITestCase()
@@ -327,6 +326,7 @@ void URITestCase::Unescaping()
     CPPUNIT_ASSERT_EQUAL( unescaped, wxURI::Unescape(escaped) );
 
 
+#if wxUSE_UNICODE
     escaped = "http://ru.wikipedia.org/wiki/"
               "%D0%A6%D0%B5%D0%BB%D0%BE%D0%B5_%D1%87%D0%B8%D1%81%D0%BB%D0%BE";
 
@@ -338,6 +338,16 @@ void URITestCase::Unescaping()
                             "\xD1\x87\xD0\xB8\xD1\x81\xD0\xBB\xD0\xBE"
                           ),
                           unescaped );
+
+    escaped = L"file://\u043C\u043E\u0439%5C%d1%84%d0%b0%d0%b9%d0%bb";
+    unescaped = wxURI::Unescape(escaped);
+
+    CPPUNIT_ASSERT_EQUAL
+    (
+        L"file://\u043C\u043E\u0439\\\u0444\u0430\u0439\u043B",
+        unescaped
+    );
+#endif // wxUSE_UNICODE
 }
 
 void URITestCase::FileScheme()
